/*
    Copyright (C) 2014 Parrot SA

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the 
      distribution.
    * Neither the name of Parrot nor the names
      of its contributors may be used to endorse or promote products
      derived from this software without specific prior written
      permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
    OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
    AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/
/**
 * @file ARUPDATER_Manager.h
 * @brief libARUpdater Manager header file.
 * @date 07/01/2014
 * @author david.flattin.ext@parrot.com
 **/

#ifndef _ARUPDATER_MANAGER_H_
#define _ARUPDATER_MANAGER_H_

#include <libARUpdater/ARUPDATER_Error.h>
#include <libARDataTransfer/ARDataTransfer.h>
#include <libARDataTransfer/ARDATATRANSFER_Uploader.h>
#include <libARDiscovery/ARDISCOVERY_Discovery.h>

#define ARUPDATER_MANAGER_DEVICE_STRING_MAX_SIZE        10
#define ARUPDATER_MANAGER_FOLDER_SEPARATOR              "/"
#define ARUPDATER_MANAGER_PLF_FOLDER                    "plfFolder/"
#define ARUPDATER_MANAGER_PLF_EXTENSION                 ".plf"
#define ARUPDATER_MANAGER_TAR_EXTENSION                 ".tar"
#define ARUPDATER_MANAGER_GZ_EXTENSION                  ".gz"

/**
 * @brief Manager structure
 * @see ARUPDATER_Manager_New ()
 */
typedef struct ARUPDATER_Manager_t ARUPDATER_Manager_t;

/**
 * @brief Create a new ARUpdater Manager
 * @warning This function allocates memory
 * @param[out] error A pointer on the error output. Can be null
 * @return Pointer on the new ARUpdater Manager
 * @see ARUPDATER_Manager_Delete ()
 */
ARUPDATER_Manager_t* ARUPDATER_Manager_New(eARUPDATER_ERROR *error);

/**
 * @brief Delete an ARUpdater Manager
 * @warning This function frees memory
 * @param manager The address of the pointer on the ARUpdater Manager
 * @see ARUPDATER_Manager_New ()
 */
void ARUPDATER_Manager_Delete (ARUPDATER_Manager_t **managerPtrAddr);

/**
 * @brief get the version of a given plf file
 * @param manager : pointer on the manager
 * @param[in] product : product type to be tested
 * @param[in] remoteVersion : the remote version to be compared with local one
 * @param[in] rootFolder : root folder of the local plf.
 * @param[out] localVersionBuffer : out buffer that will contain the version of the local plf
 * @param[in] bufferSize : The size of the localVersionBuffer
 * @param[out] error : ARUPDATER_OK if operation went well, the description of the error otherwise. Can be null
 * @return 1 if the product plf is at the same version (or more recent) as the local plf
 */
int ARUPDATER_Manager_PlfVersionIsUpToDate(ARUPDATER_Manager_t *manager, eARDISCOVERY_PRODUCT product, const char *remoteVersion, const char *rootFolder, char *localVersionBuffer, size_t bufferSize, eARUPDATER_ERROR *error);

#endif /* _ARUPDATER_MANAGER_H_ */


