/*
    Copyright (C) 2014 Parrot SA

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
    * Neither the name of Parrot nor the names
    of its contributors may be used to endorse or promote products
    derived from this software without specific prior written
    permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
    OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
    AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/
/**
 * @file libARCommands/ARCOMMANDS_Decoder.h
 * @brief libARCommands decoder header.
 * This file contains all declarations needed to decode commands
 * @note Autogenerated file
 **/
#ifndef _LIBARCOMMANDS_ARCOMMANDS_DECODER_H_
#define _LIBARCOMMANDS_ARCOMMANDS_DECODER_H_
#include <libARCommands/ARCOMMANDS_Types.h>
#include <inttypes.h>

#ifdef __GNUC__
#define DEPRECATED __attribute__ ((deprecated))
#elif defined(_MSC_VER)
#define DEPRECATED __declspec(deprecated)
#else
#define DEPRECATED
#endif

/**
 * @brief Error codes for ARCOMMANDS_Decoder_DecodeBuffer function
 */
typedef enum {
    ARCOMMANDS_DECODER_OK = 0, ///< No error occured
    ARCOMMANDS_DECODER_ERROR_NO_CALLBACK, ///< No error, but no callback was set (so the call had no effect)
    ARCOMMANDS_DECODER_ERROR_UNKNOWN_COMMAND, ///< The command buffer contained an unknown command
    ARCOMMANDS_DECODER_ERROR_NOT_ENOUGH_DATA, ///< The command buffer did not contain enough data for the specified command
    ARCOMMANDS_DECODER_ERROR_NOT_ENOUGH_SPACE, ///< The string buffer was not big enough for the command description
    ARCOMMANDS_DECODER_ERROR, ///< Any other error
} eARCOMMANDS_DECODER_ERROR;

/**
 * @brief ARCOMMANDS_Decoder object holder
 */
typedef struct ARCOMMANDS_Decoder_t ARCOMMANDS_Decoder_t;

/**
 * @brief Creates a new ARCOMMANDS_Decoder_t
 * @warning This function allocates memory.
 * @note The memory must be freed by a call to ARCOMMANDS_Decoder_DeleteDecoder.
 * @return A new ARCOMMANDS_Decoder_t instance. NULL in case of error.
 */
ARCOMMANDS_Decoder_t* ARCOMMANDS_Decoder_NewDecoder (eARCOMMANDS_DECODER_ERROR *error);

/**
 * @brief Deletes an ARCOMMANDS_Decoder_t
 * @param decoder The Decoder to delete.
 */
void ARCOMMANDS_Decoder_DeleteDecoder (ARCOMMANDS_Decoder_t **decoder);

/**
 * @brief Decode a comand
 * On success, the callback set for the command will be called in the current thread.
 * @param decoder the decoder instance
 * @param buffer the command buffer to decode
 * @param buffLen the length of the command buffer
 * @return ARCOMMANDS_DECODER_OK on success, any error code otherwise
 */
eARCOMMANDS_DECODER_ERROR
ARCOMMANDS_Decoder_DecodeCommand (ARCOMMANDS_Decoder_t *decoder, const uint8_t *buffer, int32_t buffLen);

/**
 * @brief Decode a comand buffer
 * On success, the callback set for the command will be called in the current thread.
 * @param buffer the command buffer to decode
 * @param buffLen the length of the command buffer
 * @return ARCOMMANDS_DECODER_OK on success, any error code otherwise
 */
eARCOMMANDS_DECODER_ERROR
ARCOMMANDS_Decoder_DecodeBuffer (const uint8_t *buffer, int32_t buffLen) DEPRECATED;


/**
 * @brief Describe a comand buffer
 * @param buffer the command buffer to decode
 * @param buffLen the length of the command buffer
 * @param resString the string pointer in which the description will be stored
 * @param stringLen the length of the string pointer
 * @return ARCOMMANDS_DECODER_OK on success, any error code otherwise
 */
eARCOMMANDS_DECODER_ERROR
ARCOMMANDS_Decoder_DescribeBuffer (uint8_t *buffer, int32_t buffLen, char *resString, int32_t stringLen);

// Feature generic


/**
 * @brief callback type for the command generic.default
 */
typedef void (*ARCOMMANDS_Decoder_GenericDefaultCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command generic.default
 * @param decoder the decoder instance
 * @param callback new callback for the command generic.default
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetGenericDefaultCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_GenericDefaultCallback_t callback, void *custom);
/**
 * @brief callback setter for the command generic.default
 * @param callback new callback for the command generic.default
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetGenericDefaultCallback (ARCOMMANDS_Decoder_GenericDefaultCallback_t callback, void *custom) DEPRECATED;
/**
 * @brief Decode a ARCOMMANDS_Generic_DroneSettings_t
 * On success, the callback set for each commands of the multisetting will be called in the current thread.
 * @param decoder the decoder instance
 * @param multisetting the multisetting to decode
 * @return ARCOMMANDS_DECODER_OK on success, any error code otherwise
 */
eARCOMMANDS_DECODER_ERROR
ARCOMMANDS_Decoder_DecodeGenericSetDroneSettings (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Generic_DroneSettings_t * multisetting);


/**
 * @brief callback type for the command generic.SetDroneSettings
 */
typedef void (*ARCOMMANDS_Decoder_GenericSetDroneSettingsCallback_t) (ARCOMMANDS_Generic_DroneSettings_t * settings, void *custom);
/**
 * @brief Set Decoder callback setter for the command generic.SetDroneSettings
 * @param decoder the decoder instance
 * @param callback new callback for the command generic.SetDroneSettings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetGenericSetDroneSettingsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_GenericSetDroneSettingsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command generic.SetDroneSettings
 * @param callback new callback for the command generic.SetDroneSettings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetGenericSetDroneSettingsCallback (ARCOMMANDS_Decoder_GenericSetDroneSettingsCallback_t callback, void *custom) DEPRECATED;
/**
 * @brief Decode a ARCOMMANDS_Generic_DroneSettingsChanged_t
 * On success, the callback set for each commands of the multisetting will be called in the current thread.
 * @param decoder the decoder instance
 * @param multisetting the multisetting to decode
 * @return ARCOMMANDS_DECODER_OK on success, any error code otherwise
 */
eARCOMMANDS_DECODER_ERROR
ARCOMMANDS_Decoder_DecodeGenericDroneSettingsChanged (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Generic_DroneSettingsChanged_t * multisetting);


/**
 * @brief callback type for the command generic.DroneSettingsChanged
 */
typedef void (*ARCOMMANDS_Decoder_GenericDroneSettingsChangedCallback_t) (ARCOMMANDS_Generic_DroneSettingsChanged_t * settings, void *custom);
/**
 * @brief Set Decoder callback setter for the command generic.DroneSettingsChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command generic.DroneSettingsChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetGenericDroneSettingsChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_GenericDroneSettingsChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command generic.DroneSettingsChanged
 * @param callback new callback for the command generic.DroneSettingsChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetGenericDroneSettingsChangedCallback (ARCOMMANDS_Decoder_GenericDroneSettingsChangedCallback_t callback, void *custom) DEPRECATED;

// Feature animation


/**
 * @brief callback type for the command animation.cancel
 */
typedef void (*ARCOMMANDS_Decoder_AnimationCancelCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.cancel
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.cancel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationCancelCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationCancelCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.cancel
 * @param callback new callback for the command animation.cancel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationCancelCallback (ARCOMMANDS_Decoder_AnimationCancelCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.start_flip
 */
typedef void (*ARCOMMANDS_Decoder_AnimationStartFlipCallback_t) (eARCOMMANDS_ANIMATION_FLIP_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.start_flip
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.start_flip
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartFlipCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationStartFlipCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.start_flip
 * @param callback new callback for the command animation.start_flip
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartFlipCallback (ARCOMMANDS_Decoder_AnimationStartFlipCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.start_horizontal_panorama
 */
typedef void (*ARCOMMANDS_Decoder_AnimationStartHorizontalPanoramaCallback_t) (uint8_t provided_params, float rotation_angle, float rotation_speed, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.start_horizontal_panorama
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.start_horizontal_panorama
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartHorizontalPanoramaCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationStartHorizontalPanoramaCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.start_horizontal_panorama
 * @param callback new callback for the command animation.start_horizontal_panorama
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartHorizontalPanoramaCallback (ARCOMMANDS_Decoder_AnimationStartHorizontalPanoramaCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.start_dronie
 */
typedef void (*ARCOMMANDS_Decoder_AnimationStartDronieCallback_t) (uint8_t provided_params, float speed, float distance, eARCOMMANDS_ANIMATION_PLAY_MODE play_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.start_dronie
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.start_dronie
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartDronieCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationStartDronieCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.start_dronie
 * @param callback new callback for the command animation.start_dronie
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartDronieCallback (ARCOMMANDS_Decoder_AnimationStartDronieCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.start_horizontal_reveal
 */
typedef void (*ARCOMMANDS_Decoder_AnimationStartHorizontalRevealCallback_t) (uint8_t provided_params, float speed, float distance, eARCOMMANDS_ANIMATION_PLAY_MODE play_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.start_horizontal_reveal
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.start_horizontal_reveal
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartHorizontalRevealCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationStartHorizontalRevealCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.start_horizontal_reveal
 * @param callback new callback for the command animation.start_horizontal_reveal
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartHorizontalRevealCallback (ARCOMMANDS_Decoder_AnimationStartHorizontalRevealCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.start_vertical_reveal
 */
typedef void (*ARCOMMANDS_Decoder_AnimationStartVerticalRevealCallback_t) (uint8_t provided_params, float speed, float vertical_distance, float rotation_angle, float rotation_speed, eARCOMMANDS_ANIMATION_PLAY_MODE play_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.start_vertical_reveal
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.start_vertical_reveal
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartVerticalRevealCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationStartVerticalRevealCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.start_vertical_reveal
 * @param callback new callback for the command animation.start_vertical_reveal
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartVerticalRevealCallback (ARCOMMANDS_Decoder_AnimationStartVerticalRevealCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.start_spiral
 */
typedef void (*ARCOMMANDS_Decoder_AnimationStartSpiralCallback_t) (uint8_t provided_params, float speed, float radius_variation, float vertical_distance, float revolution_nb, eARCOMMANDS_ANIMATION_PLAY_MODE play_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.start_spiral
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.start_spiral
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartSpiralCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationStartSpiralCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.start_spiral
 * @param callback new callback for the command animation.start_spiral
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartSpiralCallback (ARCOMMANDS_Decoder_AnimationStartSpiralCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.start_parabola
 */
typedef void (*ARCOMMANDS_Decoder_AnimationStartParabolaCallback_t) (uint8_t provided_params, float speed, float vertical_distance, eARCOMMANDS_ANIMATION_PLAY_MODE play_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.start_parabola
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.start_parabola
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartParabolaCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationStartParabolaCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.start_parabola
 * @param callback new callback for the command animation.start_parabola
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartParabolaCallback (ARCOMMANDS_Decoder_AnimationStartParabolaCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.start_candle
 */
typedef void (*ARCOMMANDS_Decoder_AnimationStartCandleCallback_t) (uint8_t provided_params, float speed, float vertical_distance, eARCOMMANDS_ANIMATION_PLAY_MODE play_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.start_candle
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.start_candle
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartCandleCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationStartCandleCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.start_candle
 * @param callback new callback for the command animation.start_candle
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartCandleCallback (ARCOMMANDS_Decoder_AnimationStartCandleCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.start_dolly_slide
 */
typedef void (*ARCOMMANDS_Decoder_AnimationStartDollySlideCallback_t) (uint8_t provided_params, float speed, float angle, float horizontal_distance, eARCOMMANDS_ANIMATION_PLAY_MODE play_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.start_dolly_slide
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.start_dolly_slide
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartDollySlideCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationStartDollySlideCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.start_dolly_slide
 * @param callback new callback for the command animation.start_dolly_slide
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStartDollySlideCallback (ARCOMMANDS_Decoder_AnimationStartDollySlideCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.availability
 */
typedef void (*ARCOMMANDS_Decoder_AnimationAvailabilityCallback_t) (uint32_t values, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.availability
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.availability
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationAvailabilityCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationAvailabilityCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.availability
 * @param callback new callback for the command animation.availability
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationAvailabilityCallback (ARCOMMANDS_Decoder_AnimationAvailabilityCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.state
 */
typedef void (*ARCOMMANDS_Decoder_AnimationStateCallback_t) (eARCOMMANDS_ANIMATION_TYPE type, uint8_t percent, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.state
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.state
 * @param callback new callback for the command animation.state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationStateCallback (ARCOMMANDS_Decoder_AnimationStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.flip_state
 */
typedef void (*ARCOMMANDS_Decoder_AnimationFlipStateCallback_t) (eARCOMMANDS_ANIMATION_STATE state, eARCOMMANDS_ANIMATION_FLIP_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.flip_state
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.flip_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationFlipStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationFlipStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.flip_state
 * @param callback new callback for the command animation.flip_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationFlipStateCallback (ARCOMMANDS_Decoder_AnimationFlipStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.horizontal_panorama_state
 */
typedef void (*ARCOMMANDS_Decoder_AnimationHorizontalPanoramaStateCallback_t) (eARCOMMANDS_ANIMATION_STATE state, float rotation_angle, float rotation_speed, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.horizontal_panorama_state
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.horizontal_panorama_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationHorizontalPanoramaStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationHorizontalPanoramaStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.horizontal_panorama_state
 * @param callback new callback for the command animation.horizontal_panorama_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationHorizontalPanoramaStateCallback (ARCOMMANDS_Decoder_AnimationHorizontalPanoramaStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.dronie_state
 */
typedef void (*ARCOMMANDS_Decoder_AnimationDronieStateCallback_t) (eARCOMMANDS_ANIMATION_STATE state, float speed, float distance, eARCOMMANDS_ANIMATION_PLAY_MODE play_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.dronie_state
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.dronie_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationDronieStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationDronieStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.dronie_state
 * @param callback new callback for the command animation.dronie_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationDronieStateCallback (ARCOMMANDS_Decoder_AnimationDronieStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.horizontal_reveal_state
 */
typedef void (*ARCOMMANDS_Decoder_AnimationHorizontalRevealStateCallback_t) (eARCOMMANDS_ANIMATION_STATE state, float speed, float distance, eARCOMMANDS_ANIMATION_PLAY_MODE play_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.horizontal_reveal_state
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.horizontal_reveal_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationHorizontalRevealStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationHorizontalRevealStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.horizontal_reveal_state
 * @param callback new callback for the command animation.horizontal_reveal_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationHorizontalRevealStateCallback (ARCOMMANDS_Decoder_AnimationHorizontalRevealStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.vertical_reveal_state
 */
typedef void (*ARCOMMANDS_Decoder_AnimationVerticalRevealStateCallback_t) (eARCOMMANDS_ANIMATION_STATE state, float speed, float vertical_distance, float rotation_angle, float rotation_speed, eARCOMMANDS_ANIMATION_PLAY_MODE play_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.vertical_reveal_state
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.vertical_reveal_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationVerticalRevealStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationVerticalRevealStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.vertical_reveal_state
 * @param callback new callback for the command animation.vertical_reveal_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationVerticalRevealStateCallback (ARCOMMANDS_Decoder_AnimationVerticalRevealStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.spiral_state
 */
typedef void (*ARCOMMANDS_Decoder_AnimationSpiralStateCallback_t) (eARCOMMANDS_ANIMATION_STATE state, float speed, float radius_variation, float vertical_distance, float revolution_nb, eARCOMMANDS_ANIMATION_PLAY_MODE play_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.spiral_state
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.spiral_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationSpiralStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationSpiralStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.spiral_state
 * @param callback new callback for the command animation.spiral_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationSpiralStateCallback (ARCOMMANDS_Decoder_AnimationSpiralStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.parabola_state
 */
typedef void (*ARCOMMANDS_Decoder_AnimationParabolaStateCallback_t) (eARCOMMANDS_ANIMATION_STATE state, float speed, float vertical_distance, eARCOMMANDS_ANIMATION_PLAY_MODE play_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.parabola_state
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.parabola_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationParabolaStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationParabolaStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.parabola_state
 * @param callback new callback for the command animation.parabola_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationParabolaStateCallback (ARCOMMANDS_Decoder_AnimationParabolaStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.candle_state
 */
typedef void (*ARCOMMANDS_Decoder_AnimationCandleStateCallback_t) (eARCOMMANDS_ANIMATION_STATE state, float speed, float vertical_distance, eARCOMMANDS_ANIMATION_PLAY_MODE play_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.candle_state
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.candle_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationCandleStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationCandleStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.candle_state
 * @param callback new callback for the command animation.candle_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationCandleStateCallback (ARCOMMANDS_Decoder_AnimationCandleStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command animation.dolly_slide_state
 */
typedef void (*ARCOMMANDS_Decoder_AnimationDollySlideStateCallback_t) (eARCOMMANDS_ANIMATION_STATE state, float speed, float angle, float horizontal_distance, eARCOMMANDS_ANIMATION_PLAY_MODE play_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command animation.dolly_slide_state
 * @param decoder the decoder instance
 * @param callback new callback for the command animation.dolly_slide_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationDollySlideStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_AnimationDollySlideStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command animation.dolly_slide_state
 * @param callback new callback for the command animation.dolly_slide_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetAnimationDollySlideStateCallback (ARCOMMANDS_Decoder_AnimationDollySlideStateCallback_t callback, void *custom) DEPRECATED;

// Feature ARDrone3


/**
 * @brief callback type for the command ARDrone3.Piloting.FlatTrim
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingFlatTrimCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Piloting.FlatTrim
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Piloting.FlatTrim
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingFlatTrimCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingFlatTrimCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Piloting.FlatTrim
 * @param callback new callback for the command ARDrone3.Piloting.FlatTrim
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingFlatTrimCallback (ARCOMMANDS_Decoder_ARDrone3PilotingFlatTrimCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Piloting.TakeOff
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingTakeOffCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Piloting.TakeOff
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Piloting.TakeOff
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingTakeOffCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingTakeOffCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Piloting.TakeOff
 * @param callback new callback for the command ARDrone3.Piloting.TakeOff
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingTakeOffCallback (ARCOMMANDS_Decoder_ARDrone3PilotingTakeOffCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Piloting.PCMD
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingPCMDCallback_t) (uint8_t flag, int8_t roll, int8_t pitch, int8_t yaw, int8_t gaz, uint32_t timestampAndSeqNum, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Piloting.PCMD
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Piloting.PCMD
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingPCMDCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingPCMDCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Piloting.PCMD
 * @param callback new callback for the command ARDrone3.Piloting.PCMD
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingPCMDCallback (ARCOMMANDS_Decoder_ARDrone3PilotingPCMDCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Piloting.Landing
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingLandingCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Piloting.Landing
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Piloting.Landing
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingLandingCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingLandingCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Piloting.Landing
 * @param callback new callback for the command ARDrone3.Piloting.Landing
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingLandingCallback (ARCOMMANDS_Decoder_ARDrone3PilotingLandingCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Piloting.Emergency
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingEmergencyCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Piloting.Emergency
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Piloting.Emergency
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingEmergencyCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingEmergencyCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Piloting.Emergency
 * @param callback new callback for the command ARDrone3.Piloting.Emergency
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingEmergencyCallback (ARCOMMANDS_Decoder_ARDrone3PilotingEmergencyCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Piloting.NavigateHome
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingNavigateHomeCallback_t) (uint8_t start, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Piloting.NavigateHome
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Piloting.NavigateHome
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingNavigateHomeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingNavigateHomeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Piloting.NavigateHome
 * @param callback new callback for the command ARDrone3.Piloting.NavigateHome
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingNavigateHomeCallback (ARCOMMANDS_Decoder_ARDrone3PilotingNavigateHomeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Piloting.AutoTakeOffMode
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingAutoTakeOffModeCallback_t) (uint8_t state, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Piloting.AutoTakeOffMode
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Piloting.AutoTakeOffMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingAutoTakeOffModeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingAutoTakeOffModeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Piloting.AutoTakeOffMode
 * @param callback new callback for the command ARDrone3.Piloting.AutoTakeOffMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingAutoTakeOffModeCallback (ARCOMMANDS_Decoder_ARDrone3PilotingAutoTakeOffModeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Piloting.moveBy
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingMoveByCallback_t) (float dX, float dY, float dZ, float dPsi, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Piloting.moveBy
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Piloting.moveBy
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingMoveByCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingMoveByCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Piloting.moveBy
 * @param callback new callback for the command ARDrone3.Piloting.moveBy
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingMoveByCallback (ARCOMMANDS_Decoder_ARDrone3PilotingMoveByCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Piloting.UserTakeOff
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingUserTakeOffCallback_t) (uint8_t state, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Piloting.UserTakeOff
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Piloting.UserTakeOff
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingUserTakeOffCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingUserTakeOffCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Piloting.UserTakeOff
 * @param callback new callback for the command ARDrone3.Piloting.UserTakeOff
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingUserTakeOffCallback (ARCOMMANDS_Decoder_ARDrone3PilotingUserTakeOffCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Piloting.Circle
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingCircleCallback_t) (eARCOMMANDS_ARDRONE3_PILOTING_CIRCLE_DIRECTION direction, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Piloting.Circle
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Piloting.Circle
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingCircleCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingCircleCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Piloting.Circle
 * @param callback new callback for the command ARDrone3.Piloting.Circle
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingCircleCallback (ARCOMMANDS_Decoder_ARDrone3PilotingCircleCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Piloting.moveTo
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingMoveToCallback_t) (double latitude, double longitude, double altitude, eARCOMMANDS_ARDRONE3_PILOTING_MOVETO_ORIENTATION_MODE orientation_mode, float heading, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Piloting.moveTo
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Piloting.moveTo
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingMoveToCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingMoveToCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Piloting.moveTo
 * @param callback new callback for the command ARDrone3.Piloting.moveTo
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingMoveToCallback (ARCOMMANDS_Decoder_ARDrone3PilotingMoveToCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Piloting.CancelMoveTo
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingCancelMoveToCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Piloting.CancelMoveTo
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Piloting.CancelMoveTo
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingCancelMoveToCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingCancelMoveToCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Piloting.CancelMoveTo
 * @param callback new callback for the command ARDrone3.Piloting.CancelMoveTo
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingCancelMoveToCallback (ARCOMMANDS_Decoder_ARDrone3PilotingCancelMoveToCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Piloting.StartPilotedPOI
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStartPilotedPOICallback_t) (double latitude, double longitude, double altitude, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Piloting.StartPilotedPOI
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Piloting.StartPilotedPOI
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStartPilotedPOICb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStartPilotedPOICallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Piloting.StartPilotedPOI
 * @param callback new callback for the command ARDrone3.Piloting.StartPilotedPOI
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStartPilotedPOICallback (ARCOMMANDS_Decoder_ARDrone3PilotingStartPilotedPOICallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Piloting.StopPilotedPOI
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStopPilotedPOICallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Piloting.StopPilotedPOI
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Piloting.StopPilotedPOI
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStopPilotedPOICb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStopPilotedPOICallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Piloting.StopPilotedPOI
 * @param callback new callback for the command ARDrone3.Piloting.StopPilotedPOI
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStopPilotedPOICallback (ARCOMMANDS_Decoder_ARDrone3PilotingStopPilotedPOICallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Animations.Flip
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3AnimationsFlipCallback_t) (eARCOMMANDS_ARDRONE3_ANIMATIONS_FLIP_DIRECTION direction, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Animations.Flip
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Animations.Flip
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3AnimationsFlipCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3AnimationsFlipCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Animations.Flip
 * @param callback new callback for the command ARDrone3.Animations.Flip
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3AnimationsFlipCallback (ARCOMMANDS_Decoder_ARDrone3AnimationsFlipCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Camera.Orientation
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3CameraOrientationCallback_t) (int8_t tilt, int8_t pan, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Camera.Orientation
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Camera.Orientation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraOrientationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3CameraOrientationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Camera.Orientation
 * @param callback new callback for the command ARDrone3.Camera.Orientation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraOrientationCallback (ARCOMMANDS_Decoder_ARDrone3CameraOrientationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Camera.OrientationV2
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3CameraOrientationV2Callback_t) (float tilt, float pan, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Camera.OrientationV2
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Camera.OrientationV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraOrientationV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3CameraOrientationV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Camera.OrientationV2
 * @param callback new callback for the command ARDrone3.Camera.OrientationV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraOrientationV2Callback (ARCOMMANDS_Decoder_ARDrone3CameraOrientationV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Camera.Velocity
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3CameraVelocityCallback_t) (float tilt, float pan, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Camera.Velocity
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Camera.Velocity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraVelocityCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3CameraVelocityCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Camera.Velocity
 * @param callback new callback for the command ARDrone3.Camera.Velocity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraVelocityCallback (ARCOMMANDS_Decoder_ARDrone3CameraVelocityCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaRecord.Picture
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaRecordPictureCallback_t) (uint8_t mass_storage_id, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaRecord.Picture
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaRecord.Picture
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordPictureCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaRecordPictureCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaRecord.Picture
 * @param callback new callback for the command ARDrone3.MediaRecord.Picture
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordPictureCallback (ARCOMMANDS_Decoder_ARDrone3MediaRecordPictureCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaRecord.Video
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaRecordVideoCallback_t) (eARCOMMANDS_ARDRONE3_MEDIARECORD_VIDEO_RECORD record, uint8_t mass_storage_id, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaRecord.Video
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaRecord.Video
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordVideoCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaRecordVideoCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaRecord.Video
 * @param callback new callback for the command ARDrone3.MediaRecord.Video
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordVideoCallback (ARCOMMANDS_Decoder_ARDrone3MediaRecordVideoCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaRecord.PictureV2
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaRecordPictureV2Callback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaRecord.PictureV2
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaRecord.PictureV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordPictureV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaRecordPictureV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaRecord.PictureV2
 * @param callback new callback for the command ARDrone3.MediaRecord.PictureV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordPictureV2Callback (ARCOMMANDS_Decoder_ARDrone3MediaRecordPictureV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaRecord.VideoV2
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaRecordVideoV2Callback_t) (eARCOMMANDS_ARDRONE3_MEDIARECORD_VIDEOV2_RECORD record, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaRecord.VideoV2
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaRecord.VideoV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordVideoV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaRecordVideoV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaRecord.VideoV2
 * @param callback new callback for the command ARDrone3.MediaRecord.VideoV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordVideoV2Callback (ARCOMMANDS_Decoder_ARDrone3MediaRecordVideoV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Network.WifiScan
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3NetworkWifiScanCallback_t) (eARCOMMANDS_ARDRONE3_NETWORK_WIFISCAN_BAND band, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Network.WifiScan
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Network.WifiScan
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkWifiScanCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3NetworkWifiScanCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Network.WifiScan
 * @param callback new callback for the command ARDrone3.Network.WifiScan
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkWifiScanCallback (ARCOMMANDS_Decoder_ARDrone3NetworkWifiScanCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Network.WifiAuthChannel
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3NetworkWifiAuthChannelCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Network.WifiAuthChannel
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Network.WifiAuthChannel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkWifiAuthChannelCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3NetworkWifiAuthChannelCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Network.WifiAuthChannel
 * @param callback new callback for the command ARDrone3.Network.WifiAuthChannel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkWifiAuthChannelCallback (ARCOMMANDS_Decoder_ARDrone3NetworkWifiAuthChannelCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.MaxAltitude
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsMaxAltitudeCallback_t) (float current, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.MaxAltitude
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.MaxAltitude
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsMaxAltitudeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsMaxAltitudeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.MaxAltitude
 * @param callback new callback for the command ARDrone3.PilotingSettings.MaxAltitude
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsMaxAltitudeCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsMaxAltitudeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.MaxTilt
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsMaxTiltCallback_t) (float current, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.MaxTilt
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.MaxTilt
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsMaxTiltCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsMaxTiltCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.MaxTilt
 * @param callback new callback for the command ARDrone3.PilotingSettings.MaxTilt
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsMaxTiltCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsMaxTiltCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.AbsolutControl
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsAbsolutControlCallback_t) (uint8_t on, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.AbsolutControl
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.AbsolutControl
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsAbsolutControlCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsAbsolutControlCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.AbsolutControl
 * @param callback new callback for the command ARDrone3.PilotingSettings.AbsolutControl
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsAbsolutControlCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsAbsolutControlCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.MaxDistance
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsMaxDistanceCallback_t) (float value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.MaxDistance
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.MaxDistance
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsMaxDistanceCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsMaxDistanceCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.MaxDistance
 * @param callback new callback for the command ARDrone3.PilotingSettings.MaxDistance
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsMaxDistanceCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsMaxDistanceCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.NoFlyOverMaxDistance
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsNoFlyOverMaxDistanceCallback_t) (uint8_t shouldNotFlyOver, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.NoFlyOverMaxDistance
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.NoFlyOverMaxDistance
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsNoFlyOverMaxDistanceCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsNoFlyOverMaxDistanceCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.NoFlyOverMaxDistance
 * @param callback new callback for the command ARDrone3.PilotingSettings.NoFlyOverMaxDistance
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsNoFlyOverMaxDistanceCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsNoFlyOverMaxDistanceCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxHorizontalSpeed
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxHorizontalSpeedCallback_t) (float value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxHorizontalSpeed
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxHorizontalSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsSetAutonomousFlightMaxHorizontalSpeedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxHorizontalSpeedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxHorizontalSpeed
 * @param callback new callback for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxHorizontalSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsSetAutonomousFlightMaxHorizontalSpeedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxHorizontalSpeedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxVerticalSpeed
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxVerticalSpeedCallback_t) (float value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxVerticalSpeed
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxVerticalSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsSetAutonomousFlightMaxVerticalSpeedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxVerticalSpeedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxVerticalSpeed
 * @param callback new callback for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxVerticalSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsSetAutonomousFlightMaxVerticalSpeedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxVerticalSpeedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxHorizontalAcceleration
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxHorizontalAccelerationCallback_t) (float value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxHorizontalAcceleration
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxHorizontalAcceleration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsSetAutonomousFlightMaxHorizontalAccelerationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxHorizontalAccelerationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxHorizontalAcceleration
 * @param callback new callback for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxHorizontalAcceleration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsSetAutonomousFlightMaxHorizontalAccelerationCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxHorizontalAccelerationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxVerticalAcceleration
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxVerticalAccelerationCallback_t) (float value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxVerticalAcceleration
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxVerticalAcceleration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsSetAutonomousFlightMaxVerticalAccelerationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxVerticalAccelerationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxVerticalAcceleration
 * @param callback new callback for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxVerticalAcceleration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsSetAutonomousFlightMaxVerticalAccelerationCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxVerticalAccelerationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxRotationSpeed
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxRotationSpeedCallback_t) (float value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxRotationSpeed
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxRotationSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsSetAutonomousFlightMaxRotationSpeedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxRotationSpeedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxRotationSpeed
 * @param callback new callback for the command ARDrone3.PilotingSettings.setAutonomousFlightMaxRotationSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsSetAutonomousFlightMaxRotationSpeedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetAutonomousFlightMaxRotationSpeedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.BankedTurn
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsBankedTurnCallback_t) (uint8_t value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.BankedTurn
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.BankedTurn
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsBankedTurnCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsBankedTurnCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.BankedTurn
 * @param callback new callback for the command ARDrone3.PilotingSettings.BankedTurn
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsBankedTurnCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsBankedTurnCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.MinAltitude
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsMinAltitudeCallback_t) (float current, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.MinAltitude
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.MinAltitude
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsMinAltitudeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsMinAltitudeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.MinAltitude
 * @param callback new callback for the command ARDrone3.PilotingSettings.MinAltitude
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsMinAltitudeCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsMinAltitudeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.CirclingDirection
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsCirclingDirectionCallback_t) (eARCOMMANDS_ARDRONE3_PILOTINGSETTINGS_CIRCLINGDIRECTION_VALUE value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.CirclingDirection
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.CirclingDirection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsCirclingDirectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsCirclingDirectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.CirclingDirection
 * @param callback new callback for the command ARDrone3.PilotingSettings.CirclingDirection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsCirclingDirectionCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsCirclingDirectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.CirclingRadius
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsCirclingRadiusCallback_t) (uint16_t value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.CirclingRadius
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.CirclingRadius
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsCirclingRadiusCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsCirclingRadiusCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.CirclingRadius
 * @param callback new callback for the command ARDrone3.PilotingSettings.CirclingRadius
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsCirclingRadiusCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsCirclingRadiusCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.CirclingAltitude
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsCirclingAltitudeCallback_t) (uint16_t value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.CirclingAltitude
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.CirclingAltitude
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsCirclingAltitudeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsCirclingAltitudeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.CirclingAltitude
 * @param callback new callback for the command ARDrone3.PilotingSettings.CirclingAltitude
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsCirclingAltitudeCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsCirclingAltitudeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.PitchMode
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsPitchModeCallback_t) (eARCOMMANDS_ARDRONE3_PILOTINGSETTINGS_PITCHMODE_VALUE value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.PitchMode
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.PitchMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsPitchModeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsPitchModeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.PitchMode
 * @param callback new callback for the command ARDrone3.PilotingSettings.PitchMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsPitchModeCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsPitchModeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettings.SetMotionDetectionMode
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetMotionDetectionModeCallback_t) (uint8_t enable, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettings.SetMotionDetectionMode
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettings.SetMotionDetectionMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsSetMotionDetectionModeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetMotionDetectionModeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettings.SetMotionDetectionMode
 * @param callback new callback for the command ARDrone3.PilotingSettings.SetMotionDetectionMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsSetMotionDetectionModeCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsSetMotionDetectionModeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SpeedSettings.MaxVerticalSpeed
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SpeedSettingsMaxVerticalSpeedCallback_t) (float current, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SpeedSettings.MaxVerticalSpeed
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SpeedSettings.MaxVerticalSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsMaxVerticalSpeedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SpeedSettingsMaxVerticalSpeedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SpeedSettings.MaxVerticalSpeed
 * @param callback new callback for the command ARDrone3.SpeedSettings.MaxVerticalSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsMaxVerticalSpeedCallback (ARCOMMANDS_Decoder_ARDrone3SpeedSettingsMaxVerticalSpeedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SpeedSettings.MaxRotationSpeed
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SpeedSettingsMaxRotationSpeedCallback_t) (float current, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SpeedSettings.MaxRotationSpeed
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SpeedSettings.MaxRotationSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsMaxRotationSpeedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SpeedSettingsMaxRotationSpeedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SpeedSettings.MaxRotationSpeed
 * @param callback new callback for the command ARDrone3.SpeedSettings.MaxRotationSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsMaxRotationSpeedCallback (ARCOMMANDS_Decoder_ARDrone3SpeedSettingsMaxRotationSpeedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SpeedSettings.HullProtection
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SpeedSettingsHullProtectionCallback_t) (uint8_t present, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SpeedSettings.HullProtection
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SpeedSettings.HullProtection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsHullProtectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SpeedSettingsHullProtectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SpeedSettings.HullProtection
 * @param callback new callback for the command ARDrone3.SpeedSettings.HullProtection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsHullProtectionCallback (ARCOMMANDS_Decoder_ARDrone3SpeedSettingsHullProtectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SpeedSettings.Outdoor
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SpeedSettingsOutdoorCallback_t) (uint8_t outdoor, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SpeedSettings.Outdoor
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SpeedSettings.Outdoor
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsOutdoorCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SpeedSettingsOutdoorCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SpeedSettings.Outdoor
 * @param callback new callback for the command ARDrone3.SpeedSettings.Outdoor
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsOutdoorCallback (ARCOMMANDS_Decoder_ARDrone3SpeedSettingsOutdoorCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SpeedSettings.MaxPitchRollRotationSpeed
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SpeedSettingsMaxPitchRollRotationSpeedCallback_t) (float current, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SpeedSettings.MaxPitchRollRotationSpeed
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SpeedSettings.MaxPitchRollRotationSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsMaxPitchRollRotationSpeedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SpeedSettingsMaxPitchRollRotationSpeedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SpeedSettings.MaxPitchRollRotationSpeed
 * @param callback new callback for the command ARDrone3.SpeedSettings.MaxPitchRollRotationSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsMaxPitchRollRotationSpeedCallback (ARCOMMANDS_Decoder_ARDrone3SpeedSettingsMaxPitchRollRotationSpeedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.NetworkSettings.WifiSelection
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3NetworkSettingsWifiSelectionCallback_t) (eARCOMMANDS_ARDRONE3_NETWORKSETTINGS_WIFISELECTION_TYPE type, eARCOMMANDS_ARDRONE3_NETWORKSETTINGS_WIFISELECTION_BAND band, uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.NetworkSettings.WifiSelection
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.NetworkSettings.WifiSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkSettingsWifiSelectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3NetworkSettingsWifiSelectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.NetworkSettings.WifiSelection
 * @param callback new callback for the command ARDrone3.NetworkSettings.WifiSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkSettingsWifiSelectionCallback (ARCOMMANDS_Decoder_ARDrone3NetworkSettingsWifiSelectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.NetworkSettings.wifiSecurity
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3NetworkSettingsWifiSecurityCallback_t) (eARCOMMANDS_ARDRONE3_NETWORKSETTINGS_WIFISECURITY_TYPE type, const char * key, eARCOMMANDS_ARDRONE3_NETWORKSETTINGS_WIFISECURITY_KEYTYPE keyType, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.NetworkSettings.wifiSecurity
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.NetworkSettings.wifiSecurity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkSettingsWifiSecurityCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3NetworkSettingsWifiSecurityCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.NetworkSettings.wifiSecurity
 * @param callback new callback for the command ARDrone3.NetworkSettings.wifiSecurity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkSettingsWifiSecurityCallback (ARCOMMANDS_Decoder_ARDrone3NetworkSettingsWifiSecurityCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettings.PictureFormatSelection
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsPictureFormatSelectionCallback_t) (eARCOMMANDS_ARDRONE3_PICTURESETTINGS_PICTUREFORMATSELECTION_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettings.PictureFormatSelection
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettings.PictureFormatSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsPictureFormatSelectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsPictureFormatSelectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettings.PictureFormatSelection
 * @param callback new callback for the command ARDrone3.PictureSettings.PictureFormatSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsPictureFormatSelectionCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsPictureFormatSelectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettings.AutoWhiteBalanceSelection
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsAutoWhiteBalanceSelectionCallback_t) (eARCOMMANDS_ARDRONE3_PICTURESETTINGS_AUTOWHITEBALANCESELECTION_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettings.AutoWhiteBalanceSelection
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettings.AutoWhiteBalanceSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsAutoWhiteBalanceSelectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsAutoWhiteBalanceSelectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettings.AutoWhiteBalanceSelection
 * @param callback new callback for the command ARDrone3.PictureSettings.AutoWhiteBalanceSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsAutoWhiteBalanceSelectionCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsAutoWhiteBalanceSelectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettings.ExpositionSelection
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsExpositionSelectionCallback_t) (float value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettings.ExpositionSelection
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettings.ExpositionSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsExpositionSelectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsExpositionSelectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettings.ExpositionSelection
 * @param callback new callback for the command ARDrone3.PictureSettings.ExpositionSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsExpositionSelectionCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsExpositionSelectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettings.SaturationSelection
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsSaturationSelectionCallback_t) (float value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettings.SaturationSelection
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettings.SaturationSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsSaturationSelectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsSaturationSelectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettings.SaturationSelection
 * @param callback new callback for the command ARDrone3.PictureSettings.SaturationSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsSaturationSelectionCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsSaturationSelectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettings.TimelapseSelection
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsTimelapseSelectionCallback_t) (uint8_t enabled, float interval, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettings.TimelapseSelection
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettings.TimelapseSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsTimelapseSelectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsTimelapseSelectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettings.TimelapseSelection
 * @param callback new callback for the command ARDrone3.PictureSettings.TimelapseSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsTimelapseSelectionCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsTimelapseSelectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettings.VideoAutorecordSelection
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoAutorecordSelectionCallback_t) (uint8_t enabled, uint8_t mass_storage_id, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettings.VideoAutorecordSelection
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettings.VideoAutorecordSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsVideoAutorecordSelectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoAutorecordSelectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettings.VideoAutorecordSelection
 * @param callback new callback for the command ARDrone3.PictureSettings.VideoAutorecordSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsVideoAutorecordSelectionCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoAutorecordSelectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettings.VideoStabilizationMode
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoStabilizationModeCallback_t) (eARCOMMANDS_ARDRONE3_PICTURESETTINGS_VIDEOSTABILIZATIONMODE_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettings.VideoStabilizationMode
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettings.VideoStabilizationMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsVideoStabilizationModeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoStabilizationModeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettings.VideoStabilizationMode
 * @param callback new callback for the command ARDrone3.PictureSettings.VideoStabilizationMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsVideoStabilizationModeCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoStabilizationModeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettings.VideoRecordingMode
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoRecordingModeCallback_t) (eARCOMMANDS_ARDRONE3_PICTURESETTINGS_VIDEORECORDINGMODE_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettings.VideoRecordingMode
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettings.VideoRecordingMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsVideoRecordingModeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoRecordingModeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettings.VideoRecordingMode
 * @param callback new callback for the command ARDrone3.PictureSettings.VideoRecordingMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsVideoRecordingModeCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoRecordingModeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettings.VideoFramerate
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoFramerateCallback_t) (eARCOMMANDS_ARDRONE3_PICTURESETTINGS_VIDEOFRAMERATE_FRAMERATE framerate, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettings.VideoFramerate
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettings.VideoFramerate
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsVideoFramerateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoFramerateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettings.VideoFramerate
 * @param callback new callback for the command ARDrone3.PictureSettings.VideoFramerate
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsVideoFramerateCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoFramerateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettings.VideoResolutions
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoResolutionsCallback_t) (eARCOMMANDS_ARDRONE3_PICTURESETTINGS_VIDEORESOLUTIONS_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettings.VideoResolutions
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettings.VideoResolutions
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsVideoResolutionsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoResolutionsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettings.VideoResolutions
 * @param callback new callback for the command ARDrone3.PictureSettings.VideoResolutions
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsVideoResolutionsCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsVideoResolutionsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaStreaming.VideoEnable
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaStreamingVideoEnableCallback_t) (uint8_t enable, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaStreaming.VideoEnable
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaStreaming.VideoEnable
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaStreamingVideoEnableCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaStreamingVideoEnableCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaStreaming.VideoEnable
 * @param callback new callback for the command ARDrone3.MediaStreaming.VideoEnable
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaStreamingVideoEnableCallback (ARCOMMANDS_Decoder_ARDrone3MediaStreamingVideoEnableCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaStreaming.VideoStreamMode
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaStreamingVideoStreamModeCallback_t) (eARCOMMANDS_ARDRONE3_MEDIASTREAMING_VIDEOSTREAMMODE_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaStreaming.VideoStreamMode
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaStreaming.VideoStreamMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaStreamingVideoStreamModeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaStreamingVideoStreamModeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaStreaming.VideoStreamMode
 * @param callback new callback for the command ARDrone3.MediaStreaming.VideoStreamMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaStreamingVideoStreamModeCallback (ARCOMMANDS_Decoder_ARDrone3MediaStreamingVideoStreamModeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSSettings.SetHome
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSSettingsSetHomeCallback_t) (double latitude, double longitude, double altitude, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSSettings.SetHome
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSSettings.SetHome
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsSetHomeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSSettingsSetHomeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSSettings.SetHome
 * @param callback new callback for the command ARDrone3.GPSSettings.SetHome
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsSetHomeCallback (ARCOMMANDS_Decoder_ARDrone3GPSSettingsSetHomeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSSettings.ResetHome
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSSettingsResetHomeCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSSettings.ResetHome
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSSettings.ResetHome
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsResetHomeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSSettingsResetHomeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSSettings.ResetHome
 * @param callback new callback for the command ARDrone3.GPSSettings.ResetHome
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsResetHomeCallback (ARCOMMANDS_Decoder_ARDrone3GPSSettingsResetHomeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSSettings.SendControllerGPS
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSSettingsSendControllerGPSCallback_t) (double latitude, double longitude, double altitude, double horizontalAccuracy, double verticalAccuracy, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSSettings.SendControllerGPS
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSSettings.SendControllerGPS
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsSendControllerGPSCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSSettingsSendControllerGPSCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSSettings.SendControllerGPS
 * @param callback new callback for the command ARDrone3.GPSSettings.SendControllerGPS
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsSendControllerGPSCallback (ARCOMMANDS_Decoder_ARDrone3GPSSettingsSendControllerGPSCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSSettings.HomeType
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSSettingsHomeTypeCallback_t) (eARCOMMANDS_ARDRONE3_GPSSETTINGS_HOMETYPE_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSSettings.HomeType
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSSettings.HomeType
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsHomeTypeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSSettingsHomeTypeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSSettings.HomeType
 * @param callback new callback for the command ARDrone3.GPSSettings.HomeType
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsHomeTypeCallback (ARCOMMANDS_Decoder_ARDrone3GPSSettingsHomeTypeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSSettings.ReturnHomeDelay
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSSettingsReturnHomeDelayCallback_t) (uint16_t delay, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSSettings.ReturnHomeDelay
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSSettings.ReturnHomeDelay
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsReturnHomeDelayCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSSettingsReturnHomeDelayCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSSettings.ReturnHomeDelay
 * @param callback new callback for the command ARDrone3.GPSSettings.ReturnHomeDelay
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsReturnHomeDelayCallback (ARCOMMANDS_Decoder_ARDrone3GPSSettingsReturnHomeDelayCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Antiflickering.electricFrequency
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3AntiflickeringElectricFrequencyCallback_t) (eARCOMMANDS_ARDRONE3_ANTIFLICKERING_ELECTRICFREQUENCY_FREQUENCY frequency, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Antiflickering.electricFrequency
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Antiflickering.electricFrequency
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3AntiflickeringElectricFrequencyCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3AntiflickeringElectricFrequencyCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Antiflickering.electricFrequency
 * @param callback new callback for the command ARDrone3.Antiflickering.electricFrequency
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3AntiflickeringElectricFrequencyCallback (ARCOMMANDS_Decoder_ARDrone3AntiflickeringElectricFrequencyCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Antiflickering.setMode
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3AntiflickeringSetModeCallback_t) (eARCOMMANDS_ARDRONE3_ANTIFLICKERING_SETMODE_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Antiflickering.setMode
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Antiflickering.setMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3AntiflickeringSetModeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3AntiflickeringSetModeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Antiflickering.setMode
 * @param callback new callback for the command ARDrone3.Antiflickering.setMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3AntiflickeringSetModeCallback (ARCOMMANDS_Decoder_ARDrone3AntiflickeringSetModeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Sound.StartAlertSound
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SoundStartAlertSoundCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Sound.StartAlertSound
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Sound.StartAlertSound
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SoundStartAlertSoundCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SoundStartAlertSoundCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Sound.StartAlertSound
 * @param callback new callback for the command ARDrone3.Sound.StartAlertSound
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SoundStartAlertSoundCallback (ARCOMMANDS_Decoder_ARDrone3SoundStartAlertSoundCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.Sound.StopAlertSound
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SoundStopAlertSoundCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.Sound.StopAlertSound
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.Sound.StopAlertSound
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SoundStopAlertSoundCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SoundStopAlertSoundCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.Sound.StopAlertSound
 * @param callback new callback for the command ARDrone3.Sound.StopAlertSound
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SoundStopAlertSoundCallback (ARCOMMANDS_Decoder_ARDrone3SoundStopAlertSoundCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaRecordState.PictureStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaRecordStatePictureStateChangedCallback_t) (uint8_t state, uint8_t mass_storage_id, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaRecordState.PictureStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaRecordState.PictureStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordStatePictureStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaRecordStatePictureStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaRecordState.PictureStateChanged
 * @param callback new callback for the command ARDrone3.MediaRecordState.PictureStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordStatePictureStateChangedCallback (ARCOMMANDS_Decoder_ARDrone3MediaRecordStatePictureStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaRecordState.VideoStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaRecordStateVideoStateChangedCallback_t) (eARCOMMANDS_ARDRONE3_MEDIARECORDSTATE_VIDEOSTATECHANGED_STATE state, uint8_t mass_storage_id, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaRecordState.VideoStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaRecordState.VideoStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordStateVideoStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaRecordStateVideoStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaRecordState.VideoStateChanged
 * @param callback new callback for the command ARDrone3.MediaRecordState.VideoStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordStateVideoStateChangedCallback (ARCOMMANDS_Decoder_ARDrone3MediaRecordStateVideoStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaRecordState.PictureStateChangedV2
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaRecordStatePictureStateChangedV2Callback_t) (eARCOMMANDS_ARDRONE3_MEDIARECORDSTATE_PICTURESTATECHANGEDV2_STATE state, eARCOMMANDS_ARDRONE3_MEDIARECORDSTATE_PICTURESTATECHANGEDV2_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaRecordState.PictureStateChangedV2
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaRecordState.PictureStateChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordStatePictureStateChangedV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaRecordStatePictureStateChangedV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaRecordState.PictureStateChangedV2
 * @param callback new callback for the command ARDrone3.MediaRecordState.PictureStateChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordStatePictureStateChangedV2Callback (ARCOMMANDS_Decoder_ARDrone3MediaRecordStatePictureStateChangedV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaRecordState.VideoStateChangedV2
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaRecordStateVideoStateChangedV2Callback_t) (eARCOMMANDS_ARDRONE3_MEDIARECORDSTATE_VIDEOSTATECHANGEDV2_STATE state, eARCOMMANDS_ARDRONE3_MEDIARECORDSTATE_VIDEOSTATECHANGEDV2_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaRecordState.VideoStateChangedV2
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaRecordState.VideoStateChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordStateVideoStateChangedV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaRecordStateVideoStateChangedV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaRecordState.VideoStateChangedV2
 * @param callback new callback for the command ARDrone3.MediaRecordState.VideoStateChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordStateVideoStateChangedV2Callback (ARCOMMANDS_Decoder_ARDrone3MediaRecordStateVideoStateChangedV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaRecordState.VideoResolutionState
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaRecordStateVideoResolutionStateCallback_t) (eARCOMMANDS_ARDRONE3_MEDIARECORDSTATE_VIDEORESOLUTIONSTATE_STREAMING streaming, eARCOMMANDS_ARDRONE3_MEDIARECORDSTATE_VIDEORESOLUTIONSTATE_RECORDING recording, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaRecordState.VideoResolutionState
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaRecordState.VideoResolutionState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordStateVideoResolutionStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaRecordStateVideoResolutionStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaRecordState.VideoResolutionState
 * @param callback new callback for the command ARDrone3.MediaRecordState.VideoResolutionState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordStateVideoResolutionStateCallback (ARCOMMANDS_Decoder_ARDrone3MediaRecordStateVideoResolutionStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaRecordEvent.PictureEventChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaRecordEventPictureEventChangedCallback_t) (eARCOMMANDS_ARDRONE3_MEDIARECORDEVENT_PICTUREEVENTCHANGED_EVENT event, eARCOMMANDS_ARDRONE3_MEDIARECORDEVENT_PICTUREEVENTCHANGED_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaRecordEvent.PictureEventChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaRecordEvent.PictureEventChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordEventPictureEventChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaRecordEventPictureEventChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaRecordEvent.PictureEventChanged
 * @param callback new callback for the command ARDrone3.MediaRecordEvent.PictureEventChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordEventPictureEventChangedCallback (ARCOMMANDS_Decoder_ARDrone3MediaRecordEventPictureEventChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaRecordEvent.VideoEventChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaRecordEventVideoEventChangedCallback_t) (eARCOMMANDS_ARDRONE3_MEDIARECORDEVENT_VIDEOEVENTCHANGED_EVENT event, eARCOMMANDS_ARDRONE3_MEDIARECORDEVENT_VIDEOEVENTCHANGED_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaRecordEvent.VideoEventChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaRecordEvent.VideoEventChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordEventVideoEventChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaRecordEventVideoEventChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaRecordEvent.VideoEventChanged
 * @param callback new callback for the command ARDrone3.MediaRecordEvent.VideoEventChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaRecordEventVideoEventChangedCallback (ARCOMMANDS_Decoder_ARDrone3MediaRecordEventVideoEventChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.FlatTrimChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStateFlatTrimChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.FlatTrimChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.FlatTrimChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateFlatTrimChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStateFlatTrimChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.FlatTrimChanged
 * @param callback new callback for the command ARDrone3.PilotingState.FlatTrimChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateFlatTrimChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStateFlatTrimChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.FlyingStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStateFlyingStateChangedCallback_t) (eARCOMMANDS_ARDRONE3_PILOTINGSTATE_FLYINGSTATECHANGED_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.FlyingStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.FlyingStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateFlyingStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStateFlyingStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.FlyingStateChanged
 * @param callback new callback for the command ARDrone3.PilotingState.FlyingStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateFlyingStateChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStateFlyingStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.AlertStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStateAlertStateChangedCallback_t) (eARCOMMANDS_ARDRONE3_PILOTINGSTATE_ALERTSTATECHANGED_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.AlertStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.AlertStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateAlertStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStateAlertStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.AlertStateChanged
 * @param callback new callback for the command ARDrone3.PilotingState.AlertStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateAlertStateChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStateAlertStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.NavigateHomeStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStateNavigateHomeStateChangedCallback_t) (eARCOMMANDS_ARDRONE3_PILOTINGSTATE_NAVIGATEHOMESTATECHANGED_STATE state, eARCOMMANDS_ARDRONE3_PILOTINGSTATE_NAVIGATEHOMESTATECHANGED_REASON reason, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.NavigateHomeStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.NavigateHomeStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateNavigateHomeStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStateNavigateHomeStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.NavigateHomeStateChanged
 * @param callback new callback for the command ARDrone3.PilotingState.NavigateHomeStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateNavigateHomeStateChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStateNavigateHomeStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.PositionChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStatePositionChangedCallback_t) (double latitude, double longitude, double altitude, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.PositionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.PositionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStatePositionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStatePositionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.PositionChanged
 * @param callback new callback for the command ARDrone3.PilotingState.PositionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStatePositionChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStatePositionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.SpeedChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStateSpeedChangedCallback_t) (float speedX, float speedY, float speedZ, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.SpeedChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.SpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateSpeedChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStateSpeedChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.SpeedChanged
 * @param callback new callback for the command ARDrone3.PilotingState.SpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateSpeedChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStateSpeedChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.AttitudeChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStateAttitudeChangedCallback_t) (float roll, float pitch, float yaw, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.AttitudeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.AttitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateAttitudeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStateAttitudeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.AttitudeChanged
 * @param callback new callback for the command ARDrone3.PilotingState.AttitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateAttitudeChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStateAttitudeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.AutoTakeOffModeChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStateAutoTakeOffModeChangedCallback_t) (uint8_t state, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.AutoTakeOffModeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.AutoTakeOffModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateAutoTakeOffModeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStateAutoTakeOffModeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.AutoTakeOffModeChanged
 * @param callback new callback for the command ARDrone3.PilotingState.AutoTakeOffModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateAutoTakeOffModeChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStateAutoTakeOffModeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.AltitudeChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStateAltitudeChangedCallback_t) (double altitude, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.AltitudeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.AltitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateAltitudeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStateAltitudeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.AltitudeChanged
 * @param callback new callback for the command ARDrone3.PilotingState.AltitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateAltitudeChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStateAltitudeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.GpsLocationChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStateGpsLocationChangedCallback_t) (double latitude, double longitude, double altitude, int8_t latitude_accuracy, int8_t longitude_accuracy, int8_t altitude_accuracy, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.GpsLocationChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.GpsLocationChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateGpsLocationChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStateGpsLocationChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.GpsLocationChanged
 * @param callback new callback for the command ARDrone3.PilotingState.GpsLocationChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateGpsLocationChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStateGpsLocationChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.LandingStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStateLandingStateChangedCallback_t) (eARCOMMANDS_ARDRONE3_PILOTINGSTATE_LANDINGSTATECHANGED_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.LandingStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.LandingStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateLandingStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStateLandingStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.LandingStateChanged
 * @param callback new callback for the command ARDrone3.PilotingState.LandingStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateLandingStateChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStateLandingStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.AirSpeedChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStateAirSpeedChangedCallback_t) (float airSpeed, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.AirSpeedChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.AirSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateAirSpeedChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStateAirSpeedChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.AirSpeedChanged
 * @param callback new callback for the command ARDrone3.PilotingState.AirSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateAirSpeedChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStateAirSpeedChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.moveToChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStateMoveToChangedCallback_t) (double latitude, double longitude, double altitude, eARCOMMANDS_ARDRONE3_PILOTINGSTATE_MOVETOCHANGED_ORIENTATION_MODE orientation_mode, float heading, eARCOMMANDS_ARDRONE3_PILOTINGSTATE_MOVETOCHANGED_STATUS status, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.moveToChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.moveToChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateMoveToChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStateMoveToChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.moveToChanged
 * @param callback new callback for the command ARDrone3.PilotingState.moveToChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateMoveToChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStateMoveToChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.MotionState
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStateMotionStateCallback_t) (eARCOMMANDS_ARDRONE3_PILOTINGSTATE_MOTIONSTATE_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.MotionState
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.MotionState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateMotionStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStateMotionStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.MotionState
 * @param callback new callback for the command ARDrone3.PilotingState.MotionState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateMotionStateCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStateMotionStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.PilotedPOI
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStatePilotedPOICallback_t) (double latitude, double longitude, double altitude, eARCOMMANDS_ARDRONE3_PILOTINGSTATE_PILOTEDPOI_STATUS status, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.PilotedPOI
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.PilotedPOI
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStatePilotedPOICb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStatePilotedPOICallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.PilotedPOI
 * @param callback new callback for the command ARDrone3.PilotingState.PilotedPOI
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStatePilotedPOICallback (ARCOMMANDS_Decoder_ARDrone3PilotingStatePilotedPOICallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingState.ReturnHomeBatteryCapacity
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingStateReturnHomeBatteryCapacityCallback_t) (eARCOMMANDS_ARDRONE3_PILOTINGSTATE_RETURNHOMEBATTERYCAPACITY_STATUS status, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingState.ReturnHomeBatteryCapacity
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingState.ReturnHomeBatteryCapacity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateReturnHomeBatteryCapacityCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingStateReturnHomeBatteryCapacityCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingState.ReturnHomeBatteryCapacity
 * @param callback new callback for the command ARDrone3.PilotingState.ReturnHomeBatteryCapacity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingStateReturnHomeBatteryCapacityCallback (ARCOMMANDS_Decoder_ARDrone3PilotingStateReturnHomeBatteryCapacityCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingEvent.moveByEnd
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingEventMoveByEndCallback_t) (float dX, float dY, float dZ, float dPsi, eARCOMMANDS_ARDRONE3_PILOTINGEVENT_MOVEBYEND_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingEvent.moveByEnd
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingEvent.moveByEnd
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingEventMoveByEndCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingEventMoveByEndCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingEvent.moveByEnd
 * @param callback new callback for the command ARDrone3.PilotingEvent.moveByEnd
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingEventMoveByEndCallback (ARCOMMANDS_Decoder_ARDrone3PilotingEventMoveByEndCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.NetworkState.WifiScanListChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3NetworkStateWifiScanListChangedCallback_t) (const char * ssid, int16_t rssi, eARCOMMANDS_ARDRONE3_NETWORKSTATE_WIFISCANLISTCHANGED_BAND band, uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.NetworkState.WifiScanListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.NetworkState.WifiScanListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkStateWifiScanListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3NetworkStateWifiScanListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.NetworkState.WifiScanListChanged
 * @param callback new callback for the command ARDrone3.NetworkState.WifiScanListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkStateWifiScanListChangedCallback (ARCOMMANDS_Decoder_ARDrone3NetworkStateWifiScanListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.NetworkState.AllWifiScanChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3NetworkStateAllWifiScanChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.NetworkState.AllWifiScanChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.NetworkState.AllWifiScanChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkStateAllWifiScanChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3NetworkStateAllWifiScanChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.NetworkState.AllWifiScanChanged
 * @param callback new callback for the command ARDrone3.NetworkState.AllWifiScanChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkStateAllWifiScanChangedCallback (ARCOMMANDS_Decoder_ARDrone3NetworkStateAllWifiScanChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.NetworkState.WifiAuthChannelListChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3NetworkStateWifiAuthChannelListChangedCallback_t) (eARCOMMANDS_ARDRONE3_NETWORKSTATE_WIFIAUTHCHANNELLISTCHANGED_BAND band, uint8_t channel, uint8_t in_or_out, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.NetworkState.WifiAuthChannelListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.NetworkState.WifiAuthChannelListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkStateWifiAuthChannelListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3NetworkStateWifiAuthChannelListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.NetworkState.WifiAuthChannelListChanged
 * @param callback new callback for the command ARDrone3.NetworkState.WifiAuthChannelListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkStateWifiAuthChannelListChangedCallback (ARCOMMANDS_Decoder_ARDrone3NetworkStateWifiAuthChannelListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.NetworkState.AllWifiAuthChannelChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3NetworkStateAllWifiAuthChannelChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.NetworkState.AllWifiAuthChannelChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.NetworkState.AllWifiAuthChannelChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkStateAllWifiAuthChannelChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3NetworkStateAllWifiAuthChannelChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.NetworkState.AllWifiAuthChannelChanged
 * @param callback new callback for the command ARDrone3.NetworkState.AllWifiAuthChannelChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkStateAllWifiAuthChannelChangedCallback (ARCOMMANDS_Decoder_ARDrone3NetworkStateAllWifiAuthChannelChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.MaxAltitudeChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMaxAltitudeChangedCallback_t) (float current, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.MaxAltitudeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.MaxAltitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateMaxAltitudeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMaxAltitudeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.MaxAltitudeChanged
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.MaxAltitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateMaxAltitudeChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMaxAltitudeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.MaxTiltChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMaxTiltChangedCallback_t) (float current, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.MaxTiltChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.MaxTiltChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateMaxTiltChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMaxTiltChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.MaxTiltChanged
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.MaxTiltChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateMaxTiltChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMaxTiltChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.AbsolutControlChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAbsolutControlChangedCallback_t) (uint8_t on, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.AbsolutControlChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.AbsolutControlChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateAbsolutControlChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAbsolutControlChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.AbsolutControlChanged
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.AbsolutControlChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateAbsolutControlChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAbsolutControlChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.MaxDistanceChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMaxDistanceChangedCallback_t) (float current, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.MaxDistanceChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.MaxDistanceChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateMaxDistanceChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMaxDistanceChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.MaxDistanceChanged
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.MaxDistanceChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateMaxDistanceChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMaxDistanceChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.NoFlyOverMaxDistanceChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateNoFlyOverMaxDistanceChangedCallback_t) (uint8_t shouldNotFlyOver, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.NoFlyOverMaxDistanceChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.NoFlyOverMaxDistanceChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateNoFlyOverMaxDistanceChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateNoFlyOverMaxDistanceChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.NoFlyOverMaxDistanceChanged
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.NoFlyOverMaxDistanceChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateNoFlyOverMaxDistanceChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateNoFlyOverMaxDistanceChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxHorizontalSpeed
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxHorizontalSpeedCallback_t) (float value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxHorizontalSpeed
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxHorizontalSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateAutonomousFlightMaxHorizontalSpeedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxHorizontalSpeedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxHorizontalSpeed
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxHorizontalSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateAutonomousFlightMaxHorizontalSpeedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxHorizontalSpeedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxVerticalSpeed
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxVerticalSpeedCallback_t) (float value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxVerticalSpeed
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxVerticalSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateAutonomousFlightMaxVerticalSpeedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxVerticalSpeedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxVerticalSpeed
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxVerticalSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateAutonomousFlightMaxVerticalSpeedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxVerticalSpeedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxHorizontalAcceleration
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxHorizontalAccelerationCallback_t) (float value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxHorizontalAcceleration
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxHorizontalAcceleration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateAutonomousFlightMaxHorizontalAccelerationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxHorizontalAccelerationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxHorizontalAcceleration
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxHorizontalAcceleration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateAutonomousFlightMaxHorizontalAccelerationCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxHorizontalAccelerationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxVerticalAcceleration
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxVerticalAccelerationCallback_t) (float value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxVerticalAcceleration
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxVerticalAcceleration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateAutonomousFlightMaxVerticalAccelerationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxVerticalAccelerationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxVerticalAcceleration
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxVerticalAcceleration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateAutonomousFlightMaxVerticalAccelerationCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxVerticalAccelerationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxRotationSpeed
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxRotationSpeedCallback_t) (float value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxRotationSpeed
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxRotationSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateAutonomousFlightMaxRotationSpeedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxRotationSpeedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxRotationSpeed
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.AutonomousFlightMaxRotationSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateAutonomousFlightMaxRotationSpeedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateAutonomousFlightMaxRotationSpeedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.BankedTurnChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateBankedTurnChangedCallback_t) (uint8_t state, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.BankedTurnChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.BankedTurnChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateBankedTurnChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateBankedTurnChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.BankedTurnChanged
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.BankedTurnChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateBankedTurnChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateBankedTurnChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.MinAltitudeChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMinAltitudeChangedCallback_t) (float current, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.MinAltitudeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.MinAltitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateMinAltitudeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMinAltitudeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.MinAltitudeChanged
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.MinAltitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateMinAltitudeChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMinAltitudeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.CirclingDirectionChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateCirclingDirectionChangedCallback_t) (eARCOMMANDS_ARDRONE3_PILOTINGSETTINGSSTATE_CIRCLINGDIRECTIONCHANGED_VALUE value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.CirclingDirectionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.CirclingDirectionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateCirclingDirectionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateCirclingDirectionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.CirclingDirectionChanged
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.CirclingDirectionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateCirclingDirectionChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateCirclingDirectionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.CirclingRadiusChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateCirclingRadiusChangedCallback_t) (uint16_t current, uint16_t min, uint16_t max, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.CirclingRadiusChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.CirclingRadiusChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateCirclingRadiusChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateCirclingRadiusChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.CirclingRadiusChanged
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.CirclingRadiusChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateCirclingRadiusChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateCirclingRadiusChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.CirclingAltitudeChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateCirclingAltitudeChangedCallback_t) (uint16_t current, uint16_t min, uint16_t max, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.CirclingAltitudeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.CirclingAltitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateCirclingAltitudeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateCirclingAltitudeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.CirclingAltitudeChanged
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.CirclingAltitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateCirclingAltitudeChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateCirclingAltitudeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.PitchModeChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStatePitchModeChangedCallback_t) (eARCOMMANDS_ARDRONE3_PILOTINGSETTINGSSTATE_PITCHMODECHANGED_VALUE value, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.PitchModeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.PitchModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStatePitchModeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStatePitchModeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.PitchModeChanged
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.PitchModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStatePitchModeChangedCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStatePitchModeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PilotingSettingsState.MotionDetection
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMotionDetectionCallback_t) (uint8_t enabled, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PilotingSettingsState.MotionDetection
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.MotionDetection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateMotionDetectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMotionDetectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PilotingSettingsState.MotionDetection
 * @param callback new callback for the command ARDrone3.PilotingSettingsState.MotionDetection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PilotingSettingsStateMotionDetectionCallback (ARCOMMANDS_Decoder_ARDrone3PilotingSettingsStateMotionDetectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SpeedSettingsState.MaxVerticalSpeedChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateMaxVerticalSpeedChangedCallback_t) (float current, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SpeedSettingsState.MaxVerticalSpeedChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SpeedSettingsState.MaxVerticalSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsStateMaxVerticalSpeedChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateMaxVerticalSpeedChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SpeedSettingsState.MaxVerticalSpeedChanged
 * @param callback new callback for the command ARDrone3.SpeedSettingsState.MaxVerticalSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsStateMaxVerticalSpeedChangedCallback (ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateMaxVerticalSpeedChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SpeedSettingsState.MaxRotationSpeedChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateMaxRotationSpeedChangedCallback_t) (float current, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SpeedSettingsState.MaxRotationSpeedChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SpeedSettingsState.MaxRotationSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsStateMaxRotationSpeedChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateMaxRotationSpeedChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SpeedSettingsState.MaxRotationSpeedChanged
 * @param callback new callback for the command ARDrone3.SpeedSettingsState.MaxRotationSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsStateMaxRotationSpeedChangedCallback (ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateMaxRotationSpeedChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SpeedSettingsState.HullProtectionChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateHullProtectionChangedCallback_t) (uint8_t present, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SpeedSettingsState.HullProtectionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SpeedSettingsState.HullProtectionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsStateHullProtectionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateHullProtectionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SpeedSettingsState.HullProtectionChanged
 * @param callback new callback for the command ARDrone3.SpeedSettingsState.HullProtectionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsStateHullProtectionChangedCallback (ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateHullProtectionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SpeedSettingsState.OutdoorChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateOutdoorChangedCallback_t) (uint8_t outdoor, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SpeedSettingsState.OutdoorChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SpeedSettingsState.OutdoorChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsStateOutdoorChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateOutdoorChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SpeedSettingsState.OutdoorChanged
 * @param callback new callback for the command ARDrone3.SpeedSettingsState.OutdoorChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsStateOutdoorChangedCallback (ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateOutdoorChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SpeedSettingsState.MaxPitchRollRotationSpeedChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateMaxPitchRollRotationSpeedChangedCallback_t) (float current, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SpeedSettingsState.MaxPitchRollRotationSpeedChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SpeedSettingsState.MaxPitchRollRotationSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsStateMaxPitchRollRotationSpeedChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateMaxPitchRollRotationSpeedChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SpeedSettingsState.MaxPitchRollRotationSpeedChanged
 * @param callback new callback for the command ARDrone3.SpeedSettingsState.MaxPitchRollRotationSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SpeedSettingsStateMaxPitchRollRotationSpeedChangedCallback (ARCOMMANDS_Decoder_ARDrone3SpeedSettingsStateMaxPitchRollRotationSpeedChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.NetworkSettingsState.WifiSelectionChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3NetworkSettingsStateWifiSelectionChangedCallback_t) (eARCOMMANDS_ARDRONE3_NETWORKSETTINGSSTATE_WIFISELECTIONCHANGED_TYPE type, eARCOMMANDS_ARDRONE3_NETWORKSETTINGSSTATE_WIFISELECTIONCHANGED_BAND band, uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.NetworkSettingsState.WifiSelectionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.NetworkSettingsState.WifiSelectionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkSettingsStateWifiSelectionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3NetworkSettingsStateWifiSelectionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.NetworkSettingsState.WifiSelectionChanged
 * @param callback new callback for the command ARDrone3.NetworkSettingsState.WifiSelectionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkSettingsStateWifiSelectionChangedCallback (ARCOMMANDS_Decoder_ARDrone3NetworkSettingsStateWifiSelectionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.NetworkSettingsState.wifiSecurityChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3NetworkSettingsStateWifiSecurityChangedCallback_t) (eARCOMMANDS_ARDRONE3_NETWORKSETTINGSSTATE_WIFISECURITYCHANGED_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.NetworkSettingsState.wifiSecurityChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.NetworkSettingsState.wifiSecurityChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkSettingsStateWifiSecurityChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3NetworkSettingsStateWifiSecurityChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.NetworkSettingsState.wifiSecurityChanged
 * @param callback new callback for the command ARDrone3.NetworkSettingsState.wifiSecurityChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkSettingsStateWifiSecurityChangedCallback (ARCOMMANDS_Decoder_ARDrone3NetworkSettingsStateWifiSecurityChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.NetworkSettingsState.wifiSecurity
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3NetworkSettingsStateWifiSecurityCallback_t) (eARCOMMANDS_ARDRONE3_NETWORKSETTINGSSTATE_WIFISECURITY_TYPE type, const char * key, eARCOMMANDS_ARDRONE3_NETWORKSETTINGSSTATE_WIFISECURITY_KEYTYPE keyType, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.NetworkSettingsState.wifiSecurity
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.NetworkSettingsState.wifiSecurity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkSettingsStateWifiSecurityCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3NetworkSettingsStateWifiSecurityCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.NetworkSettingsState.wifiSecurity
 * @param callback new callback for the command ARDrone3.NetworkSettingsState.wifiSecurity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3NetworkSettingsStateWifiSecurityCallback (ARCOMMANDS_Decoder_ARDrone3NetworkSettingsStateWifiSecurityCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SettingsState.ProductMotorVersionListChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SettingsStateProductMotorVersionListChangedCallback_t) (uint8_t motor_number, const char * type, const char * software, const char * hardware, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SettingsState.ProductMotorVersionListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SettingsState.ProductMotorVersionListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateProductMotorVersionListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SettingsStateProductMotorVersionListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SettingsState.ProductMotorVersionListChanged
 * @param callback new callback for the command ARDrone3.SettingsState.ProductMotorVersionListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateProductMotorVersionListChangedCallback (ARCOMMANDS_Decoder_ARDrone3SettingsStateProductMotorVersionListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SettingsState.ProductGPSVersionChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SettingsStateProductGPSVersionChangedCallback_t) (const char * software, const char * hardware, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SettingsState.ProductGPSVersionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SettingsState.ProductGPSVersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateProductGPSVersionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SettingsStateProductGPSVersionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SettingsState.ProductGPSVersionChanged
 * @param callback new callback for the command ARDrone3.SettingsState.ProductGPSVersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateProductGPSVersionChangedCallback (ARCOMMANDS_Decoder_ARDrone3SettingsStateProductGPSVersionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SettingsState.MotorErrorStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SettingsStateMotorErrorStateChangedCallback_t) (uint8_t motorIds, eARCOMMANDS_ARDRONE3_SETTINGSSTATE_MOTORERRORSTATECHANGED_MOTORERROR motorError, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SettingsState.MotorErrorStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SettingsState.MotorErrorStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateMotorErrorStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SettingsStateMotorErrorStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SettingsState.MotorErrorStateChanged
 * @param callback new callback for the command ARDrone3.SettingsState.MotorErrorStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateMotorErrorStateChangedCallback (ARCOMMANDS_Decoder_ARDrone3SettingsStateMotorErrorStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SettingsState.MotorSoftwareVersionChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SettingsStateMotorSoftwareVersionChangedCallback_t) (const char * version, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SettingsState.MotorSoftwareVersionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SettingsState.MotorSoftwareVersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateMotorSoftwareVersionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SettingsStateMotorSoftwareVersionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SettingsState.MotorSoftwareVersionChanged
 * @param callback new callback for the command ARDrone3.SettingsState.MotorSoftwareVersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateMotorSoftwareVersionChangedCallback (ARCOMMANDS_Decoder_ARDrone3SettingsStateMotorSoftwareVersionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SettingsState.MotorFlightsStatusChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SettingsStateMotorFlightsStatusChangedCallback_t) (uint16_t nbFlights, uint16_t lastFlightDuration, uint32_t totalFlightDuration, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SettingsState.MotorFlightsStatusChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SettingsState.MotorFlightsStatusChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateMotorFlightsStatusChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SettingsStateMotorFlightsStatusChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SettingsState.MotorFlightsStatusChanged
 * @param callback new callback for the command ARDrone3.SettingsState.MotorFlightsStatusChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateMotorFlightsStatusChangedCallback (ARCOMMANDS_Decoder_ARDrone3SettingsStateMotorFlightsStatusChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SettingsState.MotorErrorLastErrorChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SettingsStateMotorErrorLastErrorChangedCallback_t) (eARCOMMANDS_ARDRONE3_SETTINGSSTATE_MOTORERRORLASTERRORCHANGED_MOTORERROR motorError, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SettingsState.MotorErrorLastErrorChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SettingsState.MotorErrorLastErrorChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateMotorErrorLastErrorChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SettingsStateMotorErrorLastErrorChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SettingsState.MotorErrorLastErrorChanged
 * @param callback new callback for the command ARDrone3.SettingsState.MotorErrorLastErrorChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateMotorErrorLastErrorChangedCallback (ARCOMMANDS_Decoder_ARDrone3SettingsStateMotorErrorLastErrorChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SettingsState.P7ID
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SettingsStateP7IDCallback_t) (const char * serialID, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SettingsState.P7ID
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SettingsState.P7ID
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateP7IDCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SettingsStateP7IDCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SettingsState.P7ID
 * @param callback new callback for the command ARDrone3.SettingsState.P7ID
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateP7IDCallback (ARCOMMANDS_Decoder_ARDrone3SettingsStateP7IDCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SettingsState.CPUID
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SettingsStateCPUIDCallback_t) (const char * id, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SettingsState.CPUID
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SettingsState.CPUID
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateCPUIDCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SettingsStateCPUIDCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SettingsState.CPUID
 * @param callback new callback for the command ARDrone3.SettingsState.CPUID
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SettingsStateCPUIDCallback (ARCOMMANDS_Decoder_ARDrone3SettingsStateCPUIDCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettingsState.PictureFormatChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsStatePictureFormatChangedCallback_t) (eARCOMMANDS_ARDRONE3_PICTURESETTINGSSTATE_PICTUREFORMATCHANGED_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettingsState.PictureFormatChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettingsState.PictureFormatChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStatePictureFormatChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsStatePictureFormatChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettingsState.PictureFormatChanged
 * @param callback new callback for the command ARDrone3.PictureSettingsState.PictureFormatChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStatePictureFormatChangedCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsStatePictureFormatChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettingsState.AutoWhiteBalanceChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateAutoWhiteBalanceChangedCallback_t) (eARCOMMANDS_ARDRONE3_PICTURESETTINGSSTATE_AUTOWHITEBALANCECHANGED_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettingsState.AutoWhiteBalanceChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettingsState.AutoWhiteBalanceChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateAutoWhiteBalanceChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateAutoWhiteBalanceChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettingsState.AutoWhiteBalanceChanged
 * @param callback new callback for the command ARDrone3.PictureSettingsState.AutoWhiteBalanceChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateAutoWhiteBalanceChangedCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateAutoWhiteBalanceChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettingsState.ExpositionChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateExpositionChangedCallback_t) (float value, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettingsState.ExpositionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettingsState.ExpositionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateExpositionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateExpositionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettingsState.ExpositionChanged
 * @param callback new callback for the command ARDrone3.PictureSettingsState.ExpositionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateExpositionChangedCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateExpositionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettingsState.SaturationChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateSaturationChangedCallback_t) (float value, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettingsState.SaturationChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettingsState.SaturationChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateSaturationChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateSaturationChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettingsState.SaturationChanged
 * @param callback new callback for the command ARDrone3.PictureSettingsState.SaturationChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateSaturationChangedCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateSaturationChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettingsState.TimelapseChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateTimelapseChangedCallback_t) (uint8_t enabled, float interval, float minInterval, float maxInterval, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettingsState.TimelapseChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettingsState.TimelapseChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateTimelapseChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateTimelapseChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettingsState.TimelapseChanged
 * @param callback new callback for the command ARDrone3.PictureSettingsState.TimelapseChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateTimelapseChangedCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateTimelapseChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettingsState.VideoAutorecordChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoAutorecordChangedCallback_t) (uint8_t enabled, uint8_t mass_storage_id, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettingsState.VideoAutorecordChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettingsState.VideoAutorecordChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateVideoAutorecordChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoAutorecordChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettingsState.VideoAutorecordChanged
 * @param callback new callback for the command ARDrone3.PictureSettingsState.VideoAutorecordChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateVideoAutorecordChangedCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoAutorecordChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettingsState.VideoStabilizationModeChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoStabilizationModeChangedCallback_t) (eARCOMMANDS_ARDRONE3_PICTURESETTINGSSTATE_VIDEOSTABILIZATIONMODECHANGED_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettingsState.VideoStabilizationModeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettingsState.VideoStabilizationModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateVideoStabilizationModeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoStabilizationModeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettingsState.VideoStabilizationModeChanged
 * @param callback new callback for the command ARDrone3.PictureSettingsState.VideoStabilizationModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateVideoStabilizationModeChangedCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoStabilizationModeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettingsState.VideoRecordingModeChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoRecordingModeChangedCallback_t) (eARCOMMANDS_ARDRONE3_PICTURESETTINGSSTATE_VIDEORECORDINGMODECHANGED_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettingsState.VideoRecordingModeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettingsState.VideoRecordingModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateVideoRecordingModeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoRecordingModeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettingsState.VideoRecordingModeChanged
 * @param callback new callback for the command ARDrone3.PictureSettingsState.VideoRecordingModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateVideoRecordingModeChangedCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoRecordingModeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettingsState.VideoFramerateChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoFramerateChangedCallback_t) (eARCOMMANDS_ARDRONE3_PICTURESETTINGSSTATE_VIDEOFRAMERATECHANGED_FRAMERATE framerate, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettingsState.VideoFramerateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettingsState.VideoFramerateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateVideoFramerateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoFramerateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettingsState.VideoFramerateChanged
 * @param callback new callback for the command ARDrone3.PictureSettingsState.VideoFramerateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateVideoFramerateChangedCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoFramerateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PictureSettingsState.VideoResolutionsChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoResolutionsChangedCallback_t) (eARCOMMANDS_ARDRONE3_PICTURESETTINGSSTATE_VIDEORESOLUTIONSCHANGED_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PictureSettingsState.VideoResolutionsChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PictureSettingsState.VideoResolutionsChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateVideoResolutionsChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoResolutionsChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PictureSettingsState.VideoResolutionsChanged
 * @param callback new callback for the command ARDrone3.PictureSettingsState.VideoResolutionsChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PictureSettingsStateVideoResolutionsChangedCallback (ARCOMMANDS_Decoder_ARDrone3PictureSettingsStateVideoResolutionsChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaStreamingState.VideoEnableChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaStreamingStateVideoEnableChangedCallback_t) (eARCOMMANDS_ARDRONE3_MEDIASTREAMINGSTATE_VIDEOENABLECHANGED_ENABLED enabled, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaStreamingState.VideoEnableChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaStreamingState.VideoEnableChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaStreamingStateVideoEnableChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaStreamingStateVideoEnableChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaStreamingState.VideoEnableChanged
 * @param callback new callback for the command ARDrone3.MediaStreamingState.VideoEnableChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaStreamingStateVideoEnableChangedCallback (ARCOMMANDS_Decoder_ARDrone3MediaStreamingStateVideoEnableChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.MediaStreamingState.VideoStreamModeChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3MediaStreamingStateVideoStreamModeChangedCallback_t) (eARCOMMANDS_ARDRONE3_MEDIASTREAMINGSTATE_VIDEOSTREAMMODECHANGED_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.MediaStreamingState.VideoStreamModeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.MediaStreamingState.VideoStreamModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaStreamingStateVideoStreamModeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3MediaStreamingStateVideoStreamModeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.MediaStreamingState.VideoStreamModeChanged
 * @param callback new callback for the command ARDrone3.MediaStreamingState.VideoStreamModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3MediaStreamingStateVideoStreamModeChangedCallback (ARCOMMANDS_Decoder_ARDrone3MediaStreamingStateVideoStreamModeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSSettingsState.HomeChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateHomeChangedCallback_t) (double latitude, double longitude, double altitude, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSSettingsState.HomeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSSettingsState.HomeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsStateHomeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateHomeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSSettingsState.HomeChanged
 * @param callback new callback for the command ARDrone3.GPSSettingsState.HomeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsStateHomeChangedCallback (ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateHomeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSSettingsState.ResetHomeChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateResetHomeChangedCallback_t) (double latitude, double longitude, double altitude, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSSettingsState.ResetHomeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSSettingsState.ResetHomeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsStateResetHomeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateResetHomeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSSettingsState.ResetHomeChanged
 * @param callback new callback for the command ARDrone3.GPSSettingsState.ResetHomeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsStateResetHomeChangedCallback (ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateResetHomeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSSettingsState.GPSFixStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateGPSFixStateChangedCallback_t) (uint8_t fixed, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSSettingsState.GPSFixStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSSettingsState.GPSFixStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsStateGPSFixStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateGPSFixStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSSettingsState.GPSFixStateChanged
 * @param callback new callback for the command ARDrone3.GPSSettingsState.GPSFixStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsStateGPSFixStateChangedCallback (ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateGPSFixStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSSettingsState.GPSUpdateStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateGPSUpdateStateChangedCallback_t) (eARCOMMANDS_ARDRONE3_GPSSETTINGSSTATE_GPSUPDATESTATECHANGED_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSSettingsState.GPSUpdateStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSSettingsState.GPSUpdateStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsStateGPSUpdateStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateGPSUpdateStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSSettingsState.GPSUpdateStateChanged
 * @param callback new callback for the command ARDrone3.GPSSettingsState.GPSUpdateStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsStateGPSUpdateStateChangedCallback (ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateGPSUpdateStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSSettingsState.HomeTypeChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateHomeTypeChangedCallback_t) (eARCOMMANDS_ARDRONE3_GPSSETTINGSSTATE_HOMETYPECHANGED_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSSettingsState.HomeTypeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSSettingsState.HomeTypeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsStateHomeTypeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateHomeTypeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSSettingsState.HomeTypeChanged
 * @param callback new callback for the command ARDrone3.GPSSettingsState.HomeTypeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsStateHomeTypeChangedCallback (ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateHomeTypeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSSettingsState.ReturnHomeDelayChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateReturnHomeDelayChangedCallback_t) (uint16_t delay, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSSettingsState.ReturnHomeDelayChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSSettingsState.ReturnHomeDelayChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsStateReturnHomeDelayChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateReturnHomeDelayChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSSettingsState.ReturnHomeDelayChanged
 * @param callback new callback for the command ARDrone3.GPSSettingsState.ReturnHomeDelayChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsStateReturnHomeDelayChangedCallback (ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateReturnHomeDelayChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSSettingsState.GeofenceCenterChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateGeofenceCenterChangedCallback_t) (double latitude, double longitude, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSSettingsState.GeofenceCenterChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSSettingsState.GeofenceCenterChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsStateGeofenceCenterChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateGeofenceCenterChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSSettingsState.GeofenceCenterChanged
 * @param callback new callback for the command ARDrone3.GPSSettingsState.GeofenceCenterChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSSettingsStateGeofenceCenterChangedCallback (ARCOMMANDS_Decoder_ARDrone3GPSSettingsStateGeofenceCenterChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.CameraState.Orientation
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3CameraStateOrientationCallback_t) (int8_t tilt, int8_t pan, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.CameraState.Orientation
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.CameraState.Orientation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraStateOrientationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3CameraStateOrientationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.CameraState.Orientation
 * @param callback new callback for the command ARDrone3.CameraState.Orientation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraStateOrientationCallback (ARCOMMANDS_Decoder_ARDrone3CameraStateOrientationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.CameraState.defaultCameraOrientation
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3CameraStateDefaultCameraOrientationCallback_t) (int8_t tilt, int8_t pan, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.CameraState.defaultCameraOrientation
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.CameraState.defaultCameraOrientation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraStateDefaultCameraOrientationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3CameraStateDefaultCameraOrientationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.CameraState.defaultCameraOrientation
 * @param callback new callback for the command ARDrone3.CameraState.defaultCameraOrientation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraStateDefaultCameraOrientationCallback (ARCOMMANDS_Decoder_ARDrone3CameraStateDefaultCameraOrientationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.CameraState.OrientationV2
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3CameraStateOrientationV2Callback_t) (float tilt, float pan, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.CameraState.OrientationV2
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.CameraState.OrientationV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraStateOrientationV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3CameraStateOrientationV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.CameraState.OrientationV2
 * @param callback new callback for the command ARDrone3.CameraState.OrientationV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraStateOrientationV2Callback (ARCOMMANDS_Decoder_ARDrone3CameraStateOrientationV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.CameraState.defaultCameraOrientationV2
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3CameraStateDefaultCameraOrientationV2Callback_t) (float tilt, float pan, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.CameraState.defaultCameraOrientationV2
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.CameraState.defaultCameraOrientationV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraStateDefaultCameraOrientationV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3CameraStateDefaultCameraOrientationV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.CameraState.defaultCameraOrientationV2
 * @param callback new callback for the command ARDrone3.CameraState.defaultCameraOrientationV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraStateDefaultCameraOrientationV2Callback (ARCOMMANDS_Decoder_ARDrone3CameraStateDefaultCameraOrientationV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.CameraState.VelocityRange
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3CameraStateVelocityRangeCallback_t) (float max_tilt, float max_pan, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.CameraState.VelocityRange
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.CameraState.VelocityRange
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraStateVelocityRangeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3CameraStateVelocityRangeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.CameraState.VelocityRange
 * @param callback new callback for the command ARDrone3.CameraState.VelocityRange
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3CameraStateVelocityRangeCallback (ARCOMMANDS_Decoder_ARDrone3CameraStateVelocityRangeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.AntiflickeringState.electricFrequencyChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3AntiflickeringStateElectricFrequencyChangedCallback_t) (eARCOMMANDS_ARDRONE3_ANTIFLICKERINGSTATE_ELECTRICFREQUENCYCHANGED_FREQUENCY frequency, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.AntiflickeringState.electricFrequencyChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.AntiflickeringState.electricFrequencyChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3AntiflickeringStateElectricFrequencyChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3AntiflickeringStateElectricFrequencyChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.AntiflickeringState.electricFrequencyChanged
 * @param callback new callback for the command ARDrone3.AntiflickeringState.electricFrequencyChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3AntiflickeringStateElectricFrequencyChangedCallback (ARCOMMANDS_Decoder_ARDrone3AntiflickeringStateElectricFrequencyChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.AntiflickeringState.modeChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3AntiflickeringStateModeChangedCallback_t) (eARCOMMANDS_ARDRONE3_ANTIFLICKERINGSTATE_MODECHANGED_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.AntiflickeringState.modeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.AntiflickeringState.modeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3AntiflickeringStateModeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3AntiflickeringStateModeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.AntiflickeringState.modeChanged
 * @param callback new callback for the command ARDrone3.AntiflickeringState.modeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3AntiflickeringStateModeChangedCallback (ARCOMMANDS_Decoder_ARDrone3AntiflickeringStateModeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSState.NumberOfSatelliteChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSStateNumberOfSatelliteChangedCallback_t) (uint8_t numberOfSatellite, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSState.NumberOfSatelliteChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSState.NumberOfSatelliteChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSStateNumberOfSatelliteChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSStateNumberOfSatelliteChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSState.NumberOfSatelliteChanged
 * @param callback new callback for the command ARDrone3.GPSState.NumberOfSatelliteChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSStateNumberOfSatelliteChangedCallback (ARCOMMANDS_Decoder_ARDrone3GPSStateNumberOfSatelliteChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSState.HomeTypeAvailabilityChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSStateHomeTypeAvailabilityChangedCallback_t) (eARCOMMANDS_ARDRONE3_GPSSTATE_HOMETYPEAVAILABILITYCHANGED_TYPE type, uint8_t available, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSState.HomeTypeAvailabilityChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSState.HomeTypeAvailabilityChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSStateHomeTypeAvailabilityChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSStateHomeTypeAvailabilityChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSState.HomeTypeAvailabilityChanged
 * @param callback new callback for the command ARDrone3.GPSState.HomeTypeAvailabilityChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSStateHomeTypeAvailabilityChangedCallback (ARCOMMANDS_Decoder_ARDrone3GPSStateHomeTypeAvailabilityChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.GPSState.HomeTypeChosenChanged
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3GPSStateHomeTypeChosenChangedCallback_t) (eARCOMMANDS_ARDRONE3_GPSSTATE_HOMETYPECHOSENCHANGED_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.GPSState.HomeTypeChosenChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.GPSState.HomeTypeChosenChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSStateHomeTypeChosenChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3GPSStateHomeTypeChosenChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.GPSState.HomeTypeChosenChanged
 * @param callback new callback for the command ARDrone3.GPSState.HomeTypeChosenChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3GPSStateHomeTypeChosenChangedCallback (ARCOMMANDS_Decoder_ARDrone3GPSStateHomeTypeChosenChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.PROState.Features
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3PROStateFeaturesCallback_t) (uint64_t features, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.PROState.Features
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.PROState.Features
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PROStateFeaturesCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3PROStateFeaturesCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.PROState.Features
 * @param callback new callback for the command ARDrone3.PROState.Features
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3PROStateFeaturesCallback (ARCOMMANDS_Decoder_ARDrone3PROStateFeaturesCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.AccessoryState.ConnectedAccessories
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3AccessoryStateConnectedAccessoriesCallback_t) (uint8_t id, eARCOMMANDS_ARDRONE3_ACCESSORYSTATE_CONNECTEDACCESSORIES_ACCESSORY_TYPE accessory_type, const char * uid, const char * swVersion, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.AccessoryState.ConnectedAccessories
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.AccessoryState.ConnectedAccessories
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3AccessoryStateConnectedAccessoriesCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3AccessoryStateConnectedAccessoriesCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.AccessoryState.ConnectedAccessories
 * @param callback new callback for the command ARDrone3.AccessoryState.ConnectedAccessories
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3AccessoryStateConnectedAccessoriesCallback (ARCOMMANDS_Decoder_ARDrone3AccessoryStateConnectedAccessoriesCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.AccessoryState.Battery
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3AccessoryStateBatteryCallback_t) (uint8_t id, uint8_t batteryLevel, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.AccessoryState.Battery
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.AccessoryState.Battery
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3AccessoryStateBatteryCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3AccessoryStateBatteryCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.AccessoryState.Battery
 * @param callback new callback for the command ARDrone3.AccessoryState.Battery
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3AccessoryStateBatteryCallback (ARCOMMANDS_Decoder_ARDrone3AccessoryStateBatteryCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command ARDrone3.SoundState.AlertSound
 */
typedef void (*ARCOMMANDS_Decoder_ARDrone3SoundStateAlertSoundCallback_t) (eARCOMMANDS_ARDRONE3_SOUNDSTATE_ALERTSOUND_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command ARDrone3.SoundState.AlertSound
 * @param decoder the decoder instance
 * @param callback new callback for the command ARDrone3.SoundState.AlertSound
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SoundStateAlertSoundCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ARDrone3SoundStateAlertSoundCallback_t callback, void *custom);
/**
 * @brief callback setter for the command ARDrone3.SoundState.AlertSound
 * @param callback new callback for the command ARDrone3.SoundState.AlertSound
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetARDrone3SoundStateAlertSoundCallback (ARCOMMANDS_Decoder_ARDrone3SoundStateAlertSoundCallback_t callback, void *custom) DEPRECATED;

// Feature common


/**
 * @brief callback type for the command common.Network.Disconnect
 */
typedef void (*ARCOMMANDS_Decoder_CommonNetworkDisconnectCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Network.Disconnect
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Network.Disconnect
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonNetworkDisconnectCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonNetworkDisconnectCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Network.Disconnect
 * @param callback new callback for the command common.Network.Disconnect
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonNetworkDisconnectCallback (ARCOMMANDS_Decoder_CommonNetworkDisconnectCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Settings.AllSettings
 */
typedef void (*ARCOMMANDS_Decoder_CommonSettingsAllSettingsCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Settings.AllSettings
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Settings.AllSettings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsAllSettingsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonSettingsAllSettingsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Settings.AllSettings
 * @param callback new callback for the command common.Settings.AllSettings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsAllSettingsCallback (ARCOMMANDS_Decoder_CommonSettingsAllSettingsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Settings.Reset
 */
typedef void (*ARCOMMANDS_Decoder_CommonSettingsResetCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Settings.Reset
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Settings.Reset
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsResetCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonSettingsResetCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Settings.Reset
 * @param callback new callback for the command common.Settings.Reset
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsResetCallback (ARCOMMANDS_Decoder_CommonSettingsResetCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Settings.ProductName
 */
typedef void (*ARCOMMANDS_Decoder_CommonSettingsProductNameCallback_t) (const char * name, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Settings.ProductName
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Settings.ProductName
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsProductNameCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonSettingsProductNameCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Settings.ProductName
 * @param callback new callback for the command common.Settings.ProductName
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsProductNameCallback (ARCOMMANDS_Decoder_CommonSettingsProductNameCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Settings.Country
 */
typedef void (*ARCOMMANDS_Decoder_CommonSettingsCountryCallback_t) (const char * code, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Settings.Country
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Settings.Country
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsCountryCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonSettingsCountryCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Settings.Country
 * @param callback new callback for the command common.Settings.Country
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsCountryCallback (ARCOMMANDS_Decoder_CommonSettingsCountryCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Settings.AutoCountry
 */
typedef void (*ARCOMMANDS_Decoder_CommonSettingsAutoCountryCallback_t) (uint8_t automatic, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Settings.AutoCountry
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Settings.AutoCountry
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsAutoCountryCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonSettingsAutoCountryCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Settings.AutoCountry
 * @param callback new callback for the command common.Settings.AutoCountry
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsAutoCountryCallback (ARCOMMANDS_Decoder_CommonSettingsAutoCountryCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Common.AllStates
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonAllStatesCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Common.AllStates
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Common.AllStates
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonAllStatesCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonAllStatesCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Common.AllStates
 * @param callback new callback for the command common.Common.AllStates
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonAllStatesCallback (ARCOMMANDS_Decoder_CommonCommonAllStatesCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Common.CurrentDate
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonCurrentDateCallback_t) (const char * date, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Common.CurrentDate
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Common.CurrentDate
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonCurrentDateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonCurrentDateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Common.CurrentDate
 * @param callback new callback for the command common.Common.CurrentDate
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonCurrentDateCallback (ARCOMMANDS_Decoder_CommonCommonCurrentDateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Common.CurrentTime
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonCurrentTimeCallback_t) (const char * time, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Common.CurrentTime
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Common.CurrentTime
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonCurrentTimeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonCurrentTimeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Common.CurrentTime
 * @param callback new callback for the command common.Common.CurrentTime
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonCurrentTimeCallback (ARCOMMANDS_Decoder_CommonCommonCurrentTimeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Common.Reboot
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonRebootCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Common.Reboot
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Common.Reboot
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonRebootCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonRebootCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Common.Reboot
 * @param callback new callback for the command common.Common.Reboot
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonRebootCallback (ARCOMMANDS_Decoder_CommonCommonRebootCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.OverHeat.SwitchOff
 */
typedef void (*ARCOMMANDS_Decoder_CommonOverHeatSwitchOffCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.OverHeat.SwitchOff
 * @param decoder the decoder instance
 * @param callback new callback for the command common.OverHeat.SwitchOff
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonOverHeatSwitchOffCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonOverHeatSwitchOffCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.OverHeat.SwitchOff
 * @param callback new callback for the command common.OverHeat.SwitchOff
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonOverHeatSwitchOffCallback (ARCOMMANDS_Decoder_CommonOverHeatSwitchOffCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.OverHeat.Ventilate
 */
typedef void (*ARCOMMANDS_Decoder_CommonOverHeatVentilateCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.OverHeat.Ventilate
 * @param decoder the decoder instance
 * @param callback new callback for the command common.OverHeat.Ventilate
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonOverHeatVentilateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonOverHeatVentilateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.OverHeat.Ventilate
 * @param callback new callback for the command common.OverHeat.Ventilate
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonOverHeatVentilateCallback (ARCOMMANDS_Decoder_CommonOverHeatVentilateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Controller.isPiloting
 */
typedef void (*ARCOMMANDS_Decoder_CommonControllerIsPilotingCallback_t) (uint8_t piloting, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Controller.isPiloting
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Controller.isPiloting
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonControllerIsPilotingCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonControllerIsPilotingCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Controller.isPiloting
 * @param callback new callback for the command common.Controller.isPiloting
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonControllerIsPilotingCallback (ARCOMMANDS_Decoder_CommonControllerIsPilotingCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.WifiSettings.OutdoorSetting
 */
typedef void (*ARCOMMANDS_Decoder_CommonWifiSettingsOutdoorSettingCallback_t) (uint8_t outdoor, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.WifiSettings.OutdoorSetting
 * @param decoder the decoder instance
 * @param callback new callback for the command common.WifiSettings.OutdoorSetting
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonWifiSettingsOutdoorSettingCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonWifiSettingsOutdoorSettingCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.WifiSettings.OutdoorSetting
 * @param callback new callback for the command common.WifiSettings.OutdoorSetting
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonWifiSettingsOutdoorSettingCallback (ARCOMMANDS_Decoder_CommonWifiSettingsOutdoorSettingCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Mavlink.Start
 */
typedef void (*ARCOMMANDS_Decoder_CommonMavlinkStartCallback_t) (const char * filepath, eARCOMMANDS_COMMON_MAVLINK_START_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Mavlink.Start
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Mavlink.Start
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonMavlinkStartCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonMavlinkStartCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Mavlink.Start
 * @param callback new callback for the command common.Mavlink.Start
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonMavlinkStartCallback (ARCOMMANDS_Decoder_CommonMavlinkStartCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Mavlink.Pause
 */
typedef void (*ARCOMMANDS_Decoder_CommonMavlinkPauseCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Mavlink.Pause
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Mavlink.Pause
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonMavlinkPauseCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonMavlinkPauseCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Mavlink.Pause
 * @param callback new callback for the command common.Mavlink.Pause
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonMavlinkPauseCallback (ARCOMMANDS_Decoder_CommonMavlinkPauseCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Mavlink.Stop
 */
typedef void (*ARCOMMANDS_Decoder_CommonMavlinkStopCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Mavlink.Stop
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Mavlink.Stop
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonMavlinkStopCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonMavlinkStopCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Mavlink.Stop
 * @param callback new callback for the command common.Mavlink.Stop
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonMavlinkStopCallback (ARCOMMANDS_Decoder_CommonMavlinkStopCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.FlightPlanSettings.ReturnHomeOnDisconnect
 */
typedef void (*ARCOMMANDS_Decoder_CommonFlightPlanSettingsReturnHomeOnDisconnectCallback_t) (uint8_t value, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.FlightPlanSettings.ReturnHomeOnDisconnect
 * @param decoder the decoder instance
 * @param callback new callback for the command common.FlightPlanSettings.ReturnHomeOnDisconnect
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFlightPlanSettingsReturnHomeOnDisconnectCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonFlightPlanSettingsReturnHomeOnDisconnectCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.FlightPlanSettings.ReturnHomeOnDisconnect
 * @param callback new callback for the command common.FlightPlanSettings.ReturnHomeOnDisconnect
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFlightPlanSettingsReturnHomeOnDisconnectCallback (ARCOMMANDS_Decoder_CommonFlightPlanSettingsReturnHomeOnDisconnectCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Calibration.MagnetoCalibration
 */
typedef void (*ARCOMMANDS_Decoder_CommonCalibrationMagnetoCalibrationCallback_t) (uint8_t calibrate, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Calibration.MagnetoCalibration
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Calibration.MagnetoCalibration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCalibrationMagnetoCalibrationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCalibrationMagnetoCalibrationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Calibration.MagnetoCalibration
 * @param callback new callback for the command common.Calibration.MagnetoCalibration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCalibrationMagnetoCalibrationCallback (ARCOMMANDS_Decoder_CommonCalibrationMagnetoCalibrationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Calibration.PitotCalibration
 */
typedef void (*ARCOMMANDS_Decoder_CommonCalibrationPitotCalibrationCallback_t) (uint8_t calibrate, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Calibration.PitotCalibration
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Calibration.PitotCalibration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCalibrationPitotCalibrationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCalibrationPitotCalibrationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Calibration.PitotCalibration
 * @param callback new callback for the command common.Calibration.PitotCalibration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCalibrationPitotCalibrationCallback (ARCOMMANDS_Decoder_CommonCalibrationPitotCalibrationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.GPS.ControllerPositionForRun
 */
typedef void (*ARCOMMANDS_Decoder_CommonGPSControllerPositionForRunCallback_t) (double latitude, double longitude, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.GPS.ControllerPositionForRun
 * @param decoder the decoder instance
 * @param callback new callback for the command common.GPS.ControllerPositionForRun
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonGPSControllerPositionForRunCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonGPSControllerPositionForRunCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.GPS.ControllerPositionForRun
 * @param callback new callback for the command common.GPS.ControllerPositionForRun
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonGPSControllerPositionForRunCallback (ARCOMMANDS_Decoder_CommonGPSControllerPositionForRunCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Audio.ControllerReadyForStreaming
 */
typedef void (*ARCOMMANDS_Decoder_CommonAudioControllerReadyForStreamingCallback_t) (uint8_t ready, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Audio.ControllerReadyForStreaming
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Audio.ControllerReadyForStreaming
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAudioControllerReadyForStreamingCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonAudioControllerReadyForStreamingCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Audio.ControllerReadyForStreaming
 * @param callback new callback for the command common.Audio.ControllerReadyForStreaming
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAudioControllerReadyForStreamingCallback (ARCOMMANDS_Decoder_CommonAudioControllerReadyForStreamingCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Headlights.intensity
 */
typedef void (*ARCOMMANDS_Decoder_CommonHeadlightsIntensityCallback_t) (uint8_t left, uint8_t right, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Headlights.intensity
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Headlights.intensity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonHeadlightsIntensityCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonHeadlightsIntensityCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Headlights.intensity
 * @param callback new callback for the command common.Headlights.intensity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonHeadlightsIntensityCallback (ARCOMMANDS_Decoder_CommonHeadlightsIntensityCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Animations.StartAnimation
 */
typedef void (*ARCOMMANDS_Decoder_CommonAnimationsStartAnimationCallback_t) (eARCOMMANDS_COMMON_ANIMATIONS_STARTANIMATION_ANIM anim, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Animations.StartAnimation
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Animations.StartAnimation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAnimationsStartAnimationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonAnimationsStartAnimationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Animations.StartAnimation
 * @param callback new callback for the command common.Animations.StartAnimation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAnimationsStartAnimationCallback (ARCOMMANDS_Decoder_CommonAnimationsStartAnimationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Animations.StopAnimation
 */
typedef void (*ARCOMMANDS_Decoder_CommonAnimationsStopAnimationCallback_t) (eARCOMMANDS_COMMON_ANIMATIONS_STOPANIMATION_ANIM anim, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Animations.StopAnimation
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Animations.StopAnimation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAnimationsStopAnimationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonAnimationsStopAnimationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Animations.StopAnimation
 * @param callback new callback for the command common.Animations.StopAnimation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAnimationsStopAnimationCallback (ARCOMMANDS_Decoder_CommonAnimationsStopAnimationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Animations.StopAllAnimations
 */
typedef void (*ARCOMMANDS_Decoder_CommonAnimationsStopAllAnimationsCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Animations.StopAllAnimations
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Animations.StopAllAnimations
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAnimationsStopAllAnimationsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonAnimationsStopAllAnimationsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Animations.StopAllAnimations
 * @param callback new callback for the command common.Animations.StopAllAnimations
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAnimationsStopAllAnimationsCallback (ARCOMMANDS_Decoder_CommonAnimationsStopAllAnimationsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Accessory.Config
 */
typedef void (*ARCOMMANDS_Decoder_CommonAccessoryConfigCallback_t) (eARCOMMANDS_COMMON_ACCESSORY_CONFIG_ACCESSORY accessory, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Accessory.Config
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Accessory.Config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAccessoryConfigCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonAccessoryConfigCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Accessory.Config
 * @param callback new callback for the command common.Accessory.Config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAccessoryConfigCallback (ARCOMMANDS_Decoder_CommonAccessoryConfigCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Charger.SetMaxChargeRate
 */
typedef void (*ARCOMMANDS_Decoder_CommonChargerSetMaxChargeRateCallback_t) (eARCOMMANDS_COMMON_CHARGER_SETMAXCHARGERATE_RATE rate, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Charger.SetMaxChargeRate
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Charger.SetMaxChargeRate
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonChargerSetMaxChargeRateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonChargerSetMaxChargeRateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Charger.SetMaxChargeRate
 * @param callback new callback for the command common.Charger.SetMaxChargeRate
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonChargerSetMaxChargeRateCallback (ARCOMMANDS_Decoder_CommonChargerSetMaxChargeRateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.Factory.Reset
 */
typedef void (*ARCOMMANDS_Decoder_CommonFactoryResetCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.Factory.Reset
 * @param decoder the decoder instance
 * @param callback new callback for the command common.Factory.Reset
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFactoryResetCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonFactoryResetCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.Factory.Reset
 * @param callback new callback for the command common.Factory.Reset
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFactoryResetCallback (ARCOMMANDS_Decoder_CommonFactoryResetCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.NetworkEvent.Disconnection
 */
typedef void (*ARCOMMANDS_Decoder_CommonNetworkEventDisconnectionCallback_t) (eARCOMMANDS_COMMON_NETWORKEVENT_DISCONNECTION_CAUSE cause, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.NetworkEvent.Disconnection
 * @param decoder the decoder instance
 * @param callback new callback for the command common.NetworkEvent.Disconnection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonNetworkEventDisconnectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonNetworkEventDisconnectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.NetworkEvent.Disconnection
 * @param callback new callback for the command common.NetworkEvent.Disconnection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonNetworkEventDisconnectionCallback (ARCOMMANDS_Decoder_CommonNetworkEventDisconnectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.SettingsState.AllSettingsChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonSettingsStateAllSettingsChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.SettingsState.AllSettingsChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.SettingsState.AllSettingsChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateAllSettingsChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonSettingsStateAllSettingsChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.SettingsState.AllSettingsChanged
 * @param callback new callback for the command common.SettingsState.AllSettingsChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateAllSettingsChangedCallback (ARCOMMANDS_Decoder_CommonSettingsStateAllSettingsChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.SettingsState.ResetChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonSettingsStateResetChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.SettingsState.ResetChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.SettingsState.ResetChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateResetChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonSettingsStateResetChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.SettingsState.ResetChanged
 * @param callback new callback for the command common.SettingsState.ResetChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateResetChangedCallback (ARCOMMANDS_Decoder_CommonSettingsStateResetChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.SettingsState.ProductNameChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonSettingsStateProductNameChangedCallback_t) (const char * name, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.SettingsState.ProductNameChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.SettingsState.ProductNameChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateProductNameChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonSettingsStateProductNameChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.SettingsState.ProductNameChanged
 * @param callback new callback for the command common.SettingsState.ProductNameChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateProductNameChangedCallback (ARCOMMANDS_Decoder_CommonSettingsStateProductNameChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.SettingsState.ProductVersionChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonSettingsStateProductVersionChangedCallback_t) (const char * software, const char * hardware, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.SettingsState.ProductVersionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.SettingsState.ProductVersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateProductVersionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonSettingsStateProductVersionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.SettingsState.ProductVersionChanged
 * @param callback new callback for the command common.SettingsState.ProductVersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateProductVersionChangedCallback (ARCOMMANDS_Decoder_CommonSettingsStateProductVersionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.SettingsState.ProductSerialHighChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonSettingsStateProductSerialHighChangedCallback_t) (const char * high, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.SettingsState.ProductSerialHighChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.SettingsState.ProductSerialHighChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateProductSerialHighChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonSettingsStateProductSerialHighChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.SettingsState.ProductSerialHighChanged
 * @param callback new callback for the command common.SettingsState.ProductSerialHighChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateProductSerialHighChangedCallback (ARCOMMANDS_Decoder_CommonSettingsStateProductSerialHighChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.SettingsState.ProductSerialLowChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonSettingsStateProductSerialLowChangedCallback_t) (const char * low, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.SettingsState.ProductSerialLowChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.SettingsState.ProductSerialLowChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateProductSerialLowChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonSettingsStateProductSerialLowChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.SettingsState.ProductSerialLowChanged
 * @param callback new callback for the command common.SettingsState.ProductSerialLowChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateProductSerialLowChangedCallback (ARCOMMANDS_Decoder_CommonSettingsStateProductSerialLowChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.SettingsState.CountryChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonSettingsStateCountryChangedCallback_t) (const char * code, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.SettingsState.CountryChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.SettingsState.CountryChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateCountryChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonSettingsStateCountryChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.SettingsState.CountryChanged
 * @param callback new callback for the command common.SettingsState.CountryChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateCountryChangedCallback (ARCOMMANDS_Decoder_CommonSettingsStateCountryChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.SettingsState.AutoCountryChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonSettingsStateAutoCountryChangedCallback_t) (uint8_t automatic, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.SettingsState.AutoCountryChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.SettingsState.AutoCountryChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateAutoCountryChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonSettingsStateAutoCountryChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.SettingsState.AutoCountryChanged
 * @param callback new callback for the command common.SettingsState.AutoCountryChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonSettingsStateAutoCountryChangedCallback (ARCOMMANDS_Decoder_CommonSettingsStateAutoCountryChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.AllStatesChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateAllStatesChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.AllStatesChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.AllStatesChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateAllStatesChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateAllStatesChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.AllStatesChanged
 * @param callback new callback for the command common.CommonState.AllStatesChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateAllStatesChangedCallback (ARCOMMANDS_Decoder_CommonCommonStateAllStatesChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.BatteryStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateBatteryStateChangedCallback_t) (uint8_t percent, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.BatteryStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.BatteryStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateBatteryStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateBatteryStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.BatteryStateChanged
 * @param callback new callback for the command common.CommonState.BatteryStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateBatteryStateChangedCallback (ARCOMMANDS_Decoder_CommonCommonStateBatteryStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.MassStorageStateListChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateMassStorageStateListChangedCallback_t) (uint8_t mass_storage_id, const char * name, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.MassStorageStateListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.MassStorageStateListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateMassStorageStateListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateMassStorageStateListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.MassStorageStateListChanged
 * @param callback new callback for the command common.CommonState.MassStorageStateListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateMassStorageStateListChangedCallback (ARCOMMANDS_Decoder_CommonCommonStateMassStorageStateListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.MassStorageInfoStateListChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateMassStorageInfoStateListChangedCallback_t) (uint8_t mass_storage_id, uint32_t size, uint32_t used_size, uint8_t plugged, uint8_t full, uint8_t internal, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.MassStorageInfoStateListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.MassStorageInfoStateListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateMassStorageInfoStateListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateMassStorageInfoStateListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.MassStorageInfoStateListChanged
 * @param callback new callback for the command common.CommonState.MassStorageInfoStateListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateMassStorageInfoStateListChangedCallback (ARCOMMANDS_Decoder_CommonCommonStateMassStorageInfoStateListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.CurrentDateChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateCurrentDateChangedCallback_t) (const char * date, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.CurrentDateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.CurrentDateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateCurrentDateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateCurrentDateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.CurrentDateChanged
 * @param callback new callback for the command common.CommonState.CurrentDateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateCurrentDateChangedCallback (ARCOMMANDS_Decoder_CommonCommonStateCurrentDateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.CurrentTimeChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateCurrentTimeChangedCallback_t) (const char * time, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.CurrentTimeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.CurrentTimeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateCurrentTimeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateCurrentTimeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.CurrentTimeChanged
 * @param callback new callback for the command common.CommonState.CurrentTimeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateCurrentTimeChangedCallback (ARCOMMANDS_Decoder_CommonCommonStateCurrentTimeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.MassStorageInfoRemainingListChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateMassStorageInfoRemainingListChangedCallback_t) (uint32_t free_space, uint16_t rec_time, uint32_t photo_remaining, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.MassStorageInfoRemainingListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.MassStorageInfoRemainingListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateMassStorageInfoRemainingListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateMassStorageInfoRemainingListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.MassStorageInfoRemainingListChanged
 * @param callback new callback for the command common.CommonState.MassStorageInfoRemainingListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateMassStorageInfoRemainingListChangedCallback (ARCOMMANDS_Decoder_CommonCommonStateMassStorageInfoRemainingListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.WifiSignalChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateWifiSignalChangedCallback_t) (int16_t rssi, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.WifiSignalChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.WifiSignalChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateWifiSignalChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateWifiSignalChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.WifiSignalChanged
 * @param callback new callback for the command common.CommonState.WifiSignalChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateWifiSignalChangedCallback (ARCOMMANDS_Decoder_CommonCommonStateWifiSignalChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.SensorsStatesListChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateSensorsStatesListChangedCallback_t) (eARCOMMANDS_COMMON_COMMONSTATE_SENSORSSTATESLISTCHANGED_SENSORNAME sensorName, uint8_t sensorState, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.SensorsStatesListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.SensorsStatesListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateSensorsStatesListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateSensorsStatesListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.SensorsStatesListChanged
 * @param callback new callback for the command common.CommonState.SensorsStatesListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateSensorsStatesListChangedCallback (ARCOMMANDS_Decoder_CommonCommonStateSensorsStatesListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.ProductModel
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateProductModelCallback_t) (eARCOMMANDS_COMMON_COMMONSTATE_PRODUCTMODEL_MODEL model, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.ProductModel
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.ProductModel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateProductModelCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateProductModelCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.ProductModel
 * @param callback new callback for the command common.CommonState.ProductModel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateProductModelCallback (ARCOMMANDS_Decoder_CommonCommonStateProductModelCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.CountryListKnown
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateCountryListKnownCallback_t) (uint8_t listFlags, const char * countryCodes, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.CountryListKnown
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.CountryListKnown
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateCountryListKnownCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateCountryListKnownCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.CountryListKnown
 * @param callback new callback for the command common.CommonState.CountryListKnown
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateCountryListKnownCallback (ARCOMMANDS_Decoder_CommonCommonStateCountryListKnownCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.DeprecatedMassStorageContentChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateDeprecatedMassStorageContentChangedCallback_t) (uint8_t mass_storage_id, uint16_t nbPhotos, uint16_t nbVideos, uint16_t nbPuds, uint16_t nbCrashLogs, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.DeprecatedMassStorageContentChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.DeprecatedMassStorageContentChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateDeprecatedMassStorageContentChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateDeprecatedMassStorageContentChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.DeprecatedMassStorageContentChanged
 * @param callback new callback for the command common.CommonState.DeprecatedMassStorageContentChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateDeprecatedMassStorageContentChangedCallback (ARCOMMANDS_Decoder_CommonCommonStateDeprecatedMassStorageContentChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.MassStorageContent
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateMassStorageContentCallback_t) (uint8_t mass_storage_id, uint16_t nbPhotos, uint16_t nbVideos, uint16_t nbPuds, uint16_t nbCrashLogs, uint16_t nbRawPhotos, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.MassStorageContent
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.MassStorageContent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateMassStorageContentCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateMassStorageContentCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.MassStorageContent
 * @param callback new callback for the command common.CommonState.MassStorageContent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateMassStorageContentCallback (ARCOMMANDS_Decoder_CommonCommonStateMassStorageContentCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.MassStorageContentForCurrentRun
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateMassStorageContentForCurrentRunCallback_t) (uint8_t mass_storage_id, uint16_t nbPhotos, uint16_t nbVideos, uint16_t nbRawPhotos, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.MassStorageContentForCurrentRun
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.MassStorageContentForCurrentRun
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateMassStorageContentForCurrentRunCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateMassStorageContentForCurrentRunCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.MassStorageContentForCurrentRun
 * @param callback new callback for the command common.CommonState.MassStorageContentForCurrentRun
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateMassStorageContentForCurrentRunCallback (ARCOMMANDS_Decoder_CommonCommonStateMassStorageContentForCurrentRunCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CommonState.VideoRecordingTimestamp
 */
typedef void (*ARCOMMANDS_Decoder_CommonCommonStateVideoRecordingTimestampCallback_t) (uint64_t startTimestamp, uint64_t stopTimestamp, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CommonState.VideoRecordingTimestamp
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CommonState.VideoRecordingTimestamp
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateVideoRecordingTimestampCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCommonStateVideoRecordingTimestampCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CommonState.VideoRecordingTimestamp
 * @param callback new callback for the command common.CommonState.VideoRecordingTimestamp
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCommonStateVideoRecordingTimestampCallback (ARCOMMANDS_Decoder_CommonCommonStateVideoRecordingTimestampCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.OverHeatState.OverHeatChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonOverHeatStateOverHeatChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.OverHeatState.OverHeatChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.OverHeatState.OverHeatChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonOverHeatStateOverHeatChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonOverHeatStateOverHeatChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.OverHeatState.OverHeatChanged
 * @param callback new callback for the command common.OverHeatState.OverHeatChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonOverHeatStateOverHeatChangedCallback (ARCOMMANDS_Decoder_CommonOverHeatStateOverHeatChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.OverHeatState.OverHeatRegulationChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonOverHeatStateOverHeatRegulationChangedCallback_t) (uint8_t regulationType, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.OverHeatState.OverHeatRegulationChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.OverHeatState.OverHeatRegulationChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonOverHeatStateOverHeatRegulationChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonOverHeatStateOverHeatRegulationChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.OverHeatState.OverHeatRegulationChanged
 * @param callback new callback for the command common.OverHeatState.OverHeatRegulationChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonOverHeatStateOverHeatRegulationChangedCallback (ARCOMMANDS_Decoder_CommonOverHeatStateOverHeatRegulationChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.WifiSettingsState.outdoorSettingsChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonWifiSettingsStateOutdoorSettingsChangedCallback_t) (uint8_t outdoor, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.WifiSettingsState.outdoorSettingsChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.WifiSettingsState.outdoorSettingsChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonWifiSettingsStateOutdoorSettingsChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonWifiSettingsStateOutdoorSettingsChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.WifiSettingsState.outdoorSettingsChanged
 * @param callback new callback for the command common.WifiSettingsState.outdoorSettingsChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonWifiSettingsStateOutdoorSettingsChangedCallback (ARCOMMANDS_Decoder_CommonWifiSettingsStateOutdoorSettingsChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.MavlinkState.MavlinkFilePlayingStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonMavlinkStateMavlinkFilePlayingStateChangedCallback_t) (eARCOMMANDS_COMMON_MAVLINKSTATE_MAVLINKFILEPLAYINGSTATECHANGED_STATE state, const char * filepath, eARCOMMANDS_COMMON_MAVLINKSTATE_MAVLINKFILEPLAYINGSTATECHANGED_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.MavlinkState.MavlinkFilePlayingStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.MavlinkState.MavlinkFilePlayingStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonMavlinkStateMavlinkFilePlayingStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonMavlinkStateMavlinkFilePlayingStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.MavlinkState.MavlinkFilePlayingStateChanged
 * @param callback new callback for the command common.MavlinkState.MavlinkFilePlayingStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonMavlinkStateMavlinkFilePlayingStateChangedCallback (ARCOMMANDS_Decoder_CommonMavlinkStateMavlinkFilePlayingStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.MavlinkState.MavlinkPlayErrorStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonMavlinkStateMavlinkPlayErrorStateChangedCallback_t) (eARCOMMANDS_COMMON_MAVLINKSTATE_MAVLINKPLAYERRORSTATECHANGED_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.MavlinkState.MavlinkPlayErrorStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.MavlinkState.MavlinkPlayErrorStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonMavlinkStateMavlinkPlayErrorStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonMavlinkStateMavlinkPlayErrorStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.MavlinkState.MavlinkPlayErrorStateChanged
 * @param callback new callback for the command common.MavlinkState.MavlinkPlayErrorStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonMavlinkStateMavlinkPlayErrorStateChangedCallback (ARCOMMANDS_Decoder_CommonMavlinkStateMavlinkPlayErrorStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.MavlinkState.MissionItemExecuted
 */
typedef void (*ARCOMMANDS_Decoder_CommonMavlinkStateMissionItemExecutedCallback_t) (uint32_t idx, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.MavlinkState.MissionItemExecuted
 * @param decoder the decoder instance
 * @param callback new callback for the command common.MavlinkState.MissionItemExecuted
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonMavlinkStateMissionItemExecutedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonMavlinkStateMissionItemExecutedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.MavlinkState.MissionItemExecuted
 * @param callback new callback for the command common.MavlinkState.MissionItemExecuted
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonMavlinkStateMissionItemExecutedCallback (ARCOMMANDS_Decoder_CommonMavlinkStateMissionItemExecutedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.FlightPlanSettingsState.ReturnHomeOnDisconnectChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonFlightPlanSettingsStateReturnHomeOnDisconnectChangedCallback_t) (uint8_t state, uint8_t isReadOnly, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.FlightPlanSettingsState.ReturnHomeOnDisconnectChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.FlightPlanSettingsState.ReturnHomeOnDisconnectChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFlightPlanSettingsStateReturnHomeOnDisconnectChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonFlightPlanSettingsStateReturnHomeOnDisconnectChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.FlightPlanSettingsState.ReturnHomeOnDisconnectChanged
 * @param callback new callback for the command common.FlightPlanSettingsState.ReturnHomeOnDisconnectChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFlightPlanSettingsStateReturnHomeOnDisconnectChangedCallback (ARCOMMANDS_Decoder_CommonFlightPlanSettingsStateReturnHomeOnDisconnectChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CalibrationState.MagnetoCalibrationStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCalibrationStateMagnetoCalibrationStateChangedCallback_t) (uint8_t xAxisCalibration, uint8_t yAxisCalibration, uint8_t zAxisCalibration, uint8_t calibrationFailed, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CalibrationState.MagnetoCalibrationStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CalibrationState.MagnetoCalibrationStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCalibrationStateMagnetoCalibrationStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCalibrationStateMagnetoCalibrationStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CalibrationState.MagnetoCalibrationStateChanged
 * @param callback new callback for the command common.CalibrationState.MagnetoCalibrationStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCalibrationStateMagnetoCalibrationStateChangedCallback (ARCOMMANDS_Decoder_CommonCalibrationStateMagnetoCalibrationStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CalibrationState.MagnetoCalibrationRequiredState
 */
typedef void (*ARCOMMANDS_Decoder_CommonCalibrationStateMagnetoCalibrationRequiredStateCallback_t) (uint8_t required, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CalibrationState.MagnetoCalibrationRequiredState
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CalibrationState.MagnetoCalibrationRequiredState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCalibrationStateMagnetoCalibrationRequiredStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCalibrationStateMagnetoCalibrationRequiredStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CalibrationState.MagnetoCalibrationRequiredState
 * @param callback new callback for the command common.CalibrationState.MagnetoCalibrationRequiredState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCalibrationStateMagnetoCalibrationRequiredStateCallback (ARCOMMANDS_Decoder_CommonCalibrationStateMagnetoCalibrationRequiredStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CalibrationState.MagnetoCalibrationAxisToCalibrateChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCalibrationStateMagnetoCalibrationAxisToCalibrateChangedCallback_t) (eARCOMMANDS_COMMON_CALIBRATIONSTATE_MAGNETOCALIBRATIONAXISTOCALIBRATECHANGED_AXIS axis, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CalibrationState.MagnetoCalibrationAxisToCalibrateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CalibrationState.MagnetoCalibrationAxisToCalibrateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCalibrationStateMagnetoCalibrationAxisToCalibrateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCalibrationStateMagnetoCalibrationAxisToCalibrateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CalibrationState.MagnetoCalibrationAxisToCalibrateChanged
 * @param callback new callback for the command common.CalibrationState.MagnetoCalibrationAxisToCalibrateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCalibrationStateMagnetoCalibrationAxisToCalibrateChangedCallback (ARCOMMANDS_Decoder_CommonCalibrationStateMagnetoCalibrationAxisToCalibrateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CalibrationState.MagnetoCalibrationStartedChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCalibrationStateMagnetoCalibrationStartedChangedCallback_t) (uint8_t started, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CalibrationState.MagnetoCalibrationStartedChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CalibrationState.MagnetoCalibrationStartedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCalibrationStateMagnetoCalibrationStartedChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCalibrationStateMagnetoCalibrationStartedChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CalibrationState.MagnetoCalibrationStartedChanged
 * @param callback new callback for the command common.CalibrationState.MagnetoCalibrationStartedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCalibrationStateMagnetoCalibrationStartedChangedCallback (ARCOMMANDS_Decoder_CommonCalibrationStateMagnetoCalibrationStartedChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CalibrationState.PitotCalibrationStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCalibrationStatePitotCalibrationStateChangedCallback_t) (eARCOMMANDS_COMMON_CALIBRATIONSTATE_PITOTCALIBRATIONSTATECHANGED_STATE state, uint8_t lastError, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CalibrationState.PitotCalibrationStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CalibrationState.PitotCalibrationStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCalibrationStatePitotCalibrationStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCalibrationStatePitotCalibrationStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CalibrationState.PitotCalibrationStateChanged
 * @param callback new callback for the command common.CalibrationState.PitotCalibrationStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCalibrationStatePitotCalibrationStateChangedCallback (ARCOMMANDS_Decoder_CommonCalibrationStatePitotCalibrationStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.CameraSettingsState.CameraSettingsChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonCameraSettingsStateCameraSettingsChangedCallback_t) (float fov, float panMax, float panMin, float tiltMax, float tiltMin, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.CameraSettingsState.CameraSettingsChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.CameraSettingsState.CameraSettingsChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCameraSettingsStateCameraSettingsChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonCameraSettingsStateCameraSettingsChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.CameraSettingsState.CameraSettingsChanged
 * @param callback new callback for the command common.CameraSettingsState.CameraSettingsChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonCameraSettingsStateCameraSettingsChangedCallback (ARCOMMANDS_Decoder_CommonCameraSettingsStateCameraSettingsChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.FlightPlanState.AvailabilityStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonFlightPlanStateAvailabilityStateChangedCallback_t) (uint8_t AvailabilityState, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.FlightPlanState.AvailabilityStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.FlightPlanState.AvailabilityStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFlightPlanStateAvailabilityStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonFlightPlanStateAvailabilityStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.FlightPlanState.AvailabilityStateChanged
 * @param callback new callback for the command common.FlightPlanState.AvailabilityStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFlightPlanStateAvailabilityStateChangedCallback (ARCOMMANDS_Decoder_CommonFlightPlanStateAvailabilityStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.FlightPlanState.ComponentStateListChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonFlightPlanStateComponentStateListChangedCallback_t) (eARCOMMANDS_COMMON_FLIGHTPLANSTATE_COMPONENTSTATELISTCHANGED_COMPONENT component, uint8_t State, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.FlightPlanState.ComponentStateListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.FlightPlanState.ComponentStateListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFlightPlanStateComponentStateListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonFlightPlanStateComponentStateListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.FlightPlanState.ComponentStateListChanged
 * @param callback new callback for the command common.FlightPlanState.ComponentStateListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFlightPlanStateComponentStateListChangedCallback (ARCOMMANDS_Decoder_CommonFlightPlanStateComponentStateListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.FlightPlanState.LockStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonFlightPlanStateLockStateChangedCallback_t) (uint8_t LockState, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.FlightPlanState.LockStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.FlightPlanState.LockStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFlightPlanStateLockStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonFlightPlanStateLockStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.FlightPlanState.LockStateChanged
 * @param callback new callback for the command common.FlightPlanState.LockStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFlightPlanStateLockStateChangedCallback (ARCOMMANDS_Decoder_CommonFlightPlanStateLockStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.FlightPlanEvent.StartingErrorEvent
 */
typedef void (*ARCOMMANDS_Decoder_CommonFlightPlanEventStartingErrorEventCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.FlightPlanEvent.StartingErrorEvent
 * @param decoder the decoder instance
 * @param callback new callback for the command common.FlightPlanEvent.StartingErrorEvent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFlightPlanEventStartingErrorEventCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonFlightPlanEventStartingErrorEventCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.FlightPlanEvent.StartingErrorEvent
 * @param callback new callback for the command common.FlightPlanEvent.StartingErrorEvent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFlightPlanEventStartingErrorEventCallback (ARCOMMANDS_Decoder_CommonFlightPlanEventStartingErrorEventCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.FlightPlanEvent.SpeedBridleEvent
 */
typedef void (*ARCOMMANDS_Decoder_CommonFlightPlanEventSpeedBridleEventCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command common.FlightPlanEvent.SpeedBridleEvent
 * @param decoder the decoder instance
 * @param callback new callback for the command common.FlightPlanEvent.SpeedBridleEvent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFlightPlanEventSpeedBridleEventCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonFlightPlanEventSpeedBridleEventCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.FlightPlanEvent.SpeedBridleEvent
 * @param callback new callback for the command common.FlightPlanEvent.SpeedBridleEvent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonFlightPlanEventSpeedBridleEventCallback (ARCOMMANDS_Decoder_CommonFlightPlanEventSpeedBridleEventCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.ARLibsVersionsState.ControllerLibARCommandsVersion
 */
typedef void (*ARCOMMANDS_Decoder_CommonARLibsVersionsStateControllerLibARCommandsVersionCallback_t) (const char * version, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.ARLibsVersionsState.ControllerLibARCommandsVersion
 * @param decoder the decoder instance
 * @param callback new callback for the command common.ARLibsVersionsState.ControllerLibARCommandsVersion
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonARLibsVersionsStateControllerLibARCommandsVersionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonARLibsVersionsStateControllerLibARCommandsVersionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.ARLibsVersionsState.ControllerLibARCommandsVersion
 * @param callback new callback for the command common.ARLibsVersionsState.ControllerLibARCommandsVersion
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonARLibsVersionsStateControllerLibARCommandsVersionCallback (ARCOMMANDS_Decoder_CommonARLibsVersionsStateControllerLibARCommandsVersionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.ARLibsVersionsState.SkyControllerLibARCommandsVersion
 */
typedef void (*ARCOMMANDS_Decoder_CommonARLibsVersionsStateSkyControllerLibARCommandsVersionCallback_t) (const char * version, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.ARLibsVersionsState.SkyControllerLibARCommandsVersion
 * @param decoder the decoder instance
 * @param callback new callback for the command common.ARLibsVersionsState.SkyControllerLibARCommandsVersion
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonARLibsVersionsStateSkyControllerLibARCommandsVersionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonARLibsVersionsStateSkyControllerLibARCommandsVersionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.ARLibsVersionsState.SkyControllerLibARCommandsVersion
 * @param callback new callback for the command common.ARLibsVersionsState.SkyControllerLibARCommandsVersion
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonARLibsVersionsStateSkyControllerLibARCommandsVersionCallback (ARCOMMANDS_Decoder_CommonARLibsVersionsStateSkyControllerLibARCommandsVersionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.ARLibsVersionsState.DeviceLibARCommandsVersion
 */
typedef void (*ARCOMMANDS_Decoder_CommonARLibsVersionsStateDeviceLibARCommandsVersionCallback_t) (const char * version, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.ARLibsVersionsState.DeviceLibARCommandsVersion
 * @param decoder the decoder instance
 * @param callback new callback for the command common.ARLibsVersionsState.DeviceLibARCommandsVersion
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonARLibsVersionsStateDeviceLibARCommandsVersionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonARLibsVersionsStateDeviceLibARCommandsVersionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.ARLibsVersionsState.DeviceLibARCommandsVersion
 * @param callback new callback for the command common.ARLibsVersionsState.DeviceLibARCommandsVersion
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonARLibsVersionsStateDeviceLibARCommandsVersionCallback (ARCOMMANDS_Decoder_CommonARLibsVersionsStateDeviceLibARCommandsVersionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.AudioState.AudioStreamingRunning
 */
typedef void (*ARCOMMANDS_Decoder_CommonAudioStateAudioStreamingRunningCallback_t) (uint8_t running, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.AudioState.AudioStreamingRunning
 * @param decoder the decoder instance
 * @param callback new callback for the command common.AudioState.AudioStreamingRunning
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAudioStateAudioStreamingRunningCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonAudioStateAudioStreamingRunningCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.AudioState.AudioStreamingRunning
 * @param callback new callback for the command common.AudioState.AudioStreamingRunning
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAudioStateAudioStreamingRunningCallback (ARCOMMANDS_Decoder_CommonAudioStateAudioStreamingRunningCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.HeadlightsState.intensityChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonHeadlightsStateIntensityChangedCallback_t) (uint8_t left, uint8_t right, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.HeadlightsState.intensityChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.HeadlightsState.intensityChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonHeadlightsStateIntensityChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonHeadlightsStateIntensityChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.HeadlightsState.intensityChanged
 * @param callback new callback for the command common.HeadlightsState.intensityChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonHeadlightsStateIntensityChangedCallback (ARCOMMANDS_Decoder_CommonHeadlightsStateIntensityChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.AnimationsState.List
 */
typedef void (*ARCOMMANDS_Decoder_CommonAnimationsStateListCallback_t) (eARCOMMANDS_COMMON_ANIMATIONSSTATE_LIST_ANIM anim, eARCOMMANDS_COMMON_ANIMATIONSSTATE_LIST_STATE state, eARCOMMANDS_COMMON_ANIMATIONSSTATE_LIST_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.AnimationsState.List
 * @param decoder the decoder instance
 * @param callback new callback for the command common.AnimationsState.List
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAnimationsStateListCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonAnimationsStateListCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.AnimationsState.List
 * @param callback new callback for the command common.AnimationsState.List
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAnimationsStateListCallback (ARCOMMANDS_Decoder_CommonAnimationsStateListCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.AccessoryState.SupportedAccessoriesListChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonAccessoryStateSupportedAccessoriesListChangedCallback_t) (eARCOMMANDS_COMMON_ACCESSORYSTATE_SUPPORTEDACCESSORIESLISTCHANGED_ACCESSORY accessory, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.AccessoryState.SupportedAccessoriesListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.AccessoryState.SupportedAccessoriesListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAccessoryStateSupportedAccessoriesListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonAccessoryStateSupportedAccessoriesListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.AccessoryState.SupportedAccessoriesListChanged
 * @param callback new callback for the command common.AccessoryState.SupportedAccessoriesListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAccessoryStateSupportedAccessoriesListChangedCallback (ARCOMMANDS_Decoder_CommonAccessoryStateSupportedAccessoriesListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.AccessoryState.AccessoryConfigChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonAccessoryStateAccessoryConfigChangedCallback_t) (eARCOMMANDS_COMMON_ACCESSORYSTATE_ACCESSORYCONFIGCHANGED_NEWACCESSORY newAccessory, eARCOMMANDS_COMMON_ACCESSORYSTATE_ACCESSORYCONFIGCHANGED_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.AccessoryState.AccessoryConfigChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.AccessoryState.AccessoryConfigChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAccessoryStateAccessoryConfigChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonAccessoryStateAccessoryConfigChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.AccessoryState.AccessoryConfigChanged
 * @param callback new callback for the command common.AccessoryState.AccessoryConfigChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAccessoryStateAccessoryConfigChangedCallback (ARCOMMANDS_Decoder_CommonAccessoryStateAccessoryConfigChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.AccessoryState.AccessoryConfigModificationEnabled
 */
typedef void (*ARCOMMANDS_Decoder_CommonAccessoryStateAccessoryConfigModificationEnabledCallback_t) (uint8_t enabled, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.AccessoryState.AccessoryConfigModificationEnabled
 * @param decoder the decoder instance
 * @param callback new callback for the command common.AccessoryState.AccessoryConfigModificationEnabled
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAccessoryStateAccessoryConfigModificationEnabledCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonAccessoryStateAccessoryConfigModificationEnabledCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.AccessoryState.AccessoryConfigModificationEnabled
 * @param callback new callback for the command common.AccessoryState.AccessoryConfigModificationEnabled
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonAccessoryStateAccessoryConfigModificationEnabledCallback (ARCOMMANDS_Decoder_CommonAccessoryStateAccessoryConfigModificationEnabledCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.ChargerState.MaxChargeRateChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonChargerStateMaxChargeRateChangedCallback_t) (eARCOMMANDS_COMMON_CHARGERSTATE_MAXCHARGERATECHANGED_RATE rate, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.ChargerState.MaxChargeRateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.ChargerState.MaxChargeRateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonChargerStateMaxChargeRateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonChargerStateMaxChargeRateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.ChargerState.MaxChargeRateChanged
 * @param callback new callback for the command common.ChargerState.MaxChargeRateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonChargerStateMaxChargeRateChangedCallback (ARCOMMANDS_Decoder_CommonChargerStateMaxChargeRateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.ChargerState.CurrentChargeStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonChargerStateCurrentChargeStateChangedCallback_t) (eARCOMMANDS_COMMON_CHARGERSTATE_CURRENTCHARGESTATECHANGED_STATUS status, eARCOMMANDS_COMMON_CHARGERSTATE_CURRENTCHARGESTATECHANGED_PHASE phase, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.ChargerState.CurrentChargeStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.ChargerState.CurrentChargeStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonChargerStateCurrentChargeStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonChargerStateCurrentChargeStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.ChargerState.CurrentChargeStateChanged
 * @param callback new callback for the command common.ChargerState.CurrentChargeStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonChargerStateCurrentChargeStateChangedCallback (ARCOMMANDS_Decoder_CommonChargerStateCurrentChargeStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.ChargerState.LastChargeRateChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonChargerStateLastChargeRateChangedCallback_t) (eARCOMMANDS_COMMON_CHARGERSTATE_LASTCHARGERATECHANGED_RATE rate, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.ChargerState.LastChargeRateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.ChargerState.LastChargeRateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonChargerStateLastChargeRateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonChargerStateLastChargeRateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.ChargerState.LastChargeRateChanged
 * @param callback new callback for the command common.ChargerState.LastChargeRateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonChargerStateLastChargeRateChangedCallback (ARCOMMANDS_Decoder_CommonChargerStateLastChargeRateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.ChargerState.ChargingInfo
 */
typedef void (*ARCOMMANDS_Decoder_CommonChargerStateChargingInfoCallback_t) (eARCOMMANDS_COMMON_CHARGERSTATE_CHARGINGINFO_PHASE phase, eARCOMMANDS_COMMON_CHARGERSTATE_CHARGINGINFO_RATE rate, uint8_t intensity, uint8_t fullChargingTime, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.ChargerState.ChargingInfo
 * @param decoder the decoder instance
 * @param callback new callback for the command common.ChargerState.ChargingInfo
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonChargerStateChargingInfoCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonChargerStateChargingInfoCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.ChargerState.ChargingInfo
 * @param callback new callback for the command common.ChargerState.ChargingInfo
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonChargerStateChargingInfoCallback (ARCOMMANDS_Decoder_CommonChargerStateChargingInfoCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command common.RunState.RunIdChanged
 */
typedef void (*ARCOMMANDS_Decoder_CommonRunStateRunIdChangedCallback_t) (const char * runId, void *custom);
/**
 * @brief Set Decoder callback setter for the command common.RunState.RunIdChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command common.RunState.RunIdChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonRunStateRunIdChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_CommonRunStateRunIdChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command common.RunState.RunIdChanged
 * @param callback new callback for the command common.RunState.RunIdChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetCommonRunStateRunIdChangedCallback (ARCOMMANDS_Decoder_CommonRunStateRunIdChangedCallback_t callback, void *custom) DEPRECATED;

// Feature controller_info


/**
 * @brief callback type for the command controller_info.gps
 */
typedef void (*ARCOMMANDS_Decoder_ControllerInfoGpsCallback_t) (double latitude, double longitude, float altitude, float horizontal_accuracy, float vertical_accuracy, float north_speed, float east_speed, float down_speed, double timestamp, void *custom);
/**
 * @brief Set Decoder callback setter for the command controller_info.gps
 * @param decoder the decoder instance
 * @param callback new callback for the command controller_info.gps
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetControllerInfoGpsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ControllerInfoGpsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command controller_info.gps
 * @param callback new callback for the command controller_info.gps
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetControllerInfoGpsCallback (ARCOMMANDS_Decoder_ControllerInfoGpsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command controller_info.barometer
 */
typedef void (*ARCOMMANDS_Decoder_ControllerInfoBarometerCallback_t) (float pressure, double timestamp, void *custom);
/**
 * @brief Set Decoder callback setter for the command controller_info.barometer
 * @param decoder the decoder instance
 * @param callback new callback for the command controller_info.barometer
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetControllerInfoBarometerCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ControllerInfoBarometerCallback_t callback, void *custom);
/**
 * @brief callback setter for the command controller_info.barometer
 * @param callback new callback for the command controller_info.barometer
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetControllerInfoBarometerCallback (ARCOMMANDS_Decoder_ControllerInfoBarometerCallback_t callback, void *custom) DEPRECATED;

// Feature debug


/**
 * @brief callback type for the command debug.get_all_settings
 */
typedef void (*ARCOMMANDS_Decoder_DebugGetAllSettingsCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command debug.get_all_settings
 * @param decoder the decoder instance
 * @param callback new callback for the command debug.get_all_settings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDebugGetAllSettingsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_DebugGetAllSettingsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command debug.get_all_settings
 * @param callback new callback for the command debug.get_all_settings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDebugGetAllSettingsCallback (ARCOMMANDS_Decoder_DebugGetAllSettingsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command debug.set_setting
 */
typedef void (*ARCOMMANDS_Decoder_DebugSetSettingCallback_t) (uint16_t id, const char * value, void *custom);
/**
 * @brief Set Decoder callback setter for the command debug.set_setting
 * @param decoder the decoder instance
 * @param callback new callback for the command debug.set_setting
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDebugSetSettingCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_DebugSetSettingCallback_t callback, void *custom);
/**
 * @brief callback setter for the command debug.set_setting
 * @param callback new callback for the command debug.set_setting
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDebugSetSettingCallback (ARCOMMANDS_Decoder_DebugSetSettingCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command debug.settings_info
 */
typedef void (*ARCOMMANDS_Decoder_DebugSettingsInfoCallback_t) (uint8_t list_flags, uint16_t id, const char * label, eARCOMMANDS_DEBUG_SETTING_TYPE type, eARCOMMANDS_DEBUG_SETTING_MODE mode, const char * range_min, const char * range_max, const char * range_step, const char * value, void *custom);
/**
 * @brief Set Decoder callback setter for the command debug.settings_info
 * @param decoder the decoder instance
 * @param callback new callback for the command debug.settings_info
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDebugSettingsInfoCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_DebugSettingsInfoCallback_t callback, void *custom);
/**
 * @brief callback setter for the command debug.settings_info
 * @param callback new callback for the command debug.settings_info
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDebugSettingsInfoCallback (ARCOMMANDS_Decoder_DebugSettingsInfoCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command debug.settings_list
 */
typedef void (*ARCOMMANDS_Decoder_DebugSettingsListCallback_t) (uint16_t id, const char * value, void *custom);
/**
 * @brief Set Decoder callback setter for the command debug.settings_list
 * @param decoder the decoder instance
 * @param callback new callback for the command debug.settings_list
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDebugSettingsListCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_DebugSettingsListCallback_t callback, void *custom);
/**
 * @brief callback setter for the command debug.settings_list
 * @param callback new callback for the command debug.settings_list
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDebugSettingsListCallback (ARCOMMANDS_Decoder_DebugSettingsListCallback_t callback, void *custom) DEPRECATED;

// Feature drone_manager


/**
 * @brief callback type for the command drone_manager.discover_drones
 */
typedef void (*ARCOMMANDS_Decoder_DroneManagerDiscoverDronesCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command drone_manager.discover_drones
 * @param decoder the decoder instance
 * @param callback new callback for the command drone_manager.discover_drones
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerDiscoverDronesCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_DroneManagerDiscoverDronesCallback_t callback, void *custom);
/**
 * @brief callback setter for the command drone_manager.discover_drones
 * @param callback new callback for the command drone_manager.discover_drones
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerDiscoverDronesCallback (ARCOMMANDS_Decoder_DroneManagerDiscoverDronesCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command drone_manager.connect
 */
typedef void (*ARCOMMANDS_Decoder_DroneManagerConnectCallback_t) (const char * serial, const char * key, void *custom);
/**
 * @brief Set Decoder callback setter for the command drone_manager.connect
 * @param decoder the decoder instance
 * @param callback new callback for the command drone_manager.connect
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerConnectCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_DroneManagerConnectCallback_t callback, void *custom);
/**
 * @brief callback setter for the command drone_manager.connect
 * @param callback new callback for the command drone_manager.connect
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerConnectCallback (ARCOMMANDS_Decoder_DroneManagerConnectCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command drone_manager.forget
 */
typedef void (*ARCOMMANDS_Decoder_DroneManagerForgetCallback_t) (const char * serial, void *custom);
/**
 * @brief Set Decoder callback setter for the command drone_manager.forget
 * @param decoder the decoder instance
 * @param callback new callback for the command drone_manager.forget
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerForgetCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_DroneManagerForgetCallback_t callback, void *custom);
/**
 * @brief callback setter for the command drone_manager.forget
 * @param callback new callback for the command drone_manager.forget
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerForgetCallback (ARCOMMANDS_Decoder_DroneManagerForgetCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command drone_manager.drone_list_item
 */
typedef void (*ARCOMMANDS_Decoder_DroneManagerDroneListItemCallback_t) (const char * serial, uint16_t model, const char * name, uint8_t connection_order, uint8_t active, uint8_t visible, eARCOMMANDS_DRONE_MANAGER_SECURITY security, uint8_t has_saved_key, int8_t rssi, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command drone_manager.drone_list_item
 * @param decoder the decoder instance
 * @param callback new callback for the command drone_manager.drone_list_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerDroneListItemCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_DroneManagerDroneListItemCallback_t callback, void *custom);
/**
 * @brief callback setter for the command drone_manager.drone_list_item
 * @param callback new callback for the command drone_manager.drone_list_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerDroneListItemCallback (ARCOMMANDS_Decoder_DroneManagerDroneListItemCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command drone_manager.connection_state
 */
typedef void (*ARCOMMANDS_Decoder_DroneManagerConnectionStateCallback_t) (eARCOMMANDS_DRONE_MANAGER_CONNECTION_STATE state, const char * serial, uint16_t model, const char * name, void *custom);
/**
 * @brief Set Decoder callback setter for the command drone_manager.connection_state
 * @param decoder the decoder instance
 * @param callback new callback for the command drone_manager.connection_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerConnectionStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_DroneManagerConnectionStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command drone_manager.connection_state
 * @param callback new callback for the command drone_manager.connection_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerConnectionStateCallback (ARCOMMANDS_Decoder_DroneManagerConnectionStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command drone_manager.authentication_failed
 */
typedef void (*ARCOMMANDS_Decoder_DroneManagerAuthenticationFailedCallback_t) (const char * serial, uint16_t model, const char * name, void *custom);
/**
 * @brief Set Decoder callback setter for the command drone_manager.authentication_failed
 * @param decoder the decoder instance
 * @param callback new callback for the command drone_manager.authentication_failed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerAuthenticationFailedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_DroneManagerAuthenticationFailedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command drone_manager.authentication_failed
 * @param callback new callback for the command drone_manager.authentication_failed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerAuthenticationFailedCallback (ARCOMMANDS_Decoder_DroneManagerAuthenticationFailedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command drone_manager.connection_refused
 */
typedef void (*ARCOMMANDS_Decoder_DroneManagerConnectionRefusedCallback_t) (const char * serial, uint16_t model, const char * name, void *custom);
/**
 * @brief Set Decoder callback setter for the command drone_manager.connection_refused
 * @param decoder the decoder instance
 * @param callback new callback for the command drone_manager.connection_refused
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerConnectionRefusedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_DroneManagerConnectionRefusedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command drone_manager.connection_refused
 * @param callback new callback for the command drone_manager.connection_refused
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerConnectionRefusedCallback (ARCOMMANDS_Decoder_DroneManagerConnectionRefusedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command drone_manager.known_drone_item
 */
typedef void (*ARCOMMANDS_Decoder_DroneManagerKnownDroneItemCallback_t) (const char * serial, uint16_t model, const char * name, eARCOMMANDS_DRONE_MANAGER_SECURITY security, uint8_t has_saved_key, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command drone_manager.known_drone_item
 * @param decoder the decoder instance
 * @param callback new callback for the command drone_manager.known_drone_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerKnownDroneItemCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_DroneManagerKnownDroneItemCallback_t callback, void *custom);
/**
 * @brief callback setter for the command drone_manager.known_drone_item
 * @param callback new callback for the command drone_manager.known_drone_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetDroneManagerKnownDroneItemCallback (ARCOMMANDS_Decoder_DroneManagerKnownDroneItemCallback_t callback, void *custom) DEPRECATED;

// Feature follow_me


/**
 * @brief callback type for the command follow_me.start
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeStartCallback_t) (eARCOMMANDS_FOLLOW_ME_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.start
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.start
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStartCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeStartCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.start
 * @param callback new callback for the command follow_me.start
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStartCallback (ARCOMMANDS_Decoder_FollowMeStartCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.stop
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeStopCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.stop
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.stop
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStopCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeStopCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.stop
 * @param callback new callback for the command follow_me.stop
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStopCallback (ARCOMMANDS_Decoder_FollowMeStopCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.configure_geographic
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeConfigureGeographicCallback_t) (uint8_t use_default, float distance, float elevation, float azimuth, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.configure_geographic
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.configure_geographic
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeConfigureGeographicCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeConfigureGeographicCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.configure_geographic
 * @param callback new callback for the command follow_me.configure_geographic
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeConfigureGeographicCallback (ARCOMMANDS_Decoder_FollowMeConfigureGeographicCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.configure_relative
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeConfigureRelativeCallback_t) (uint8_t use_default, float distance, float elevation, float azimuth, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.configure_relative
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.configure_relative
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeConfigureRelativeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeConfigureRelativeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.configure_relative
 * @param callback new callback for the command follow_me.configure_relative
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeConfigureRelativeCallback (ARCOMMANDS_Decoder_FollowMeConfigureRelativeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.stop_animation
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeStopAnimationCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.stop_animation
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.stop_animation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStopAnimationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeStopAnimationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.stop_animation
 * @param callback new callback for the command follow_me.stop_animation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStopAnimationCallback (ARCOMMANDS_Decoder_FollowMeStopAnimationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.start_helicoid_anim
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeStartHelicoidAnimCallback_t) (uint8_t use_default, float speed, float revolution_number, float vertical_distance, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.start_helicoid_anim
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.start_helicoid_anim
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStartHelicoidAnimCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeStartHelicoidAnimCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.start_helicoid_anim
 * @param callback new callback for the command follow_me.start_helicoid_anim
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStartHelicoidAnimCallback (ARCOMMANDS_Decoder_FollowMeStartHelicoidAnimCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.start_swing_anim
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeStartSwingAnimCallback_t) (uint8_t use_default, float speed, float vertical_distance, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.start_swing_anim
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.start_swing_anim
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStartSwingAnimCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeStartSwingAnimCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.start_swing_anim
 * @param callback new callback for the command follow_me.start_swing_anim
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStartSwingAnimCallback (ARCOMMANDS_Decoder_FollowMeStartSwingAnimCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.start_boomerang_anim
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeStartBoomerangAnimCallback_t) (uint8_t use_default, float speed, float distance, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.start_boomerang_anim
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.start_boomerang_anim
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStartBoomerangAnimCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeStartBoomerangAnimCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.start_boomerang_anim
 * @param callback new callback for the command follow_me.start_boomerang_anim
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStartBoomerangAnimCallback (ARCOMMANDS_Decoder_FollowMeStartBoomerangAnimCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.start_candle_anim
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeStartCandleAnimCallback_t) (uint8_t use_default, float speed, float vertical_distance, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.start_candle_anim
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.start_candle_anim
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStartCandleAnimCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeStartCandleAnimCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.start_candle_anim
 * @param callback new callback for the command follow_me.start_candle_anim
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStartCandleAnimCallback (ARCOMMANDS_Decoder_FollowMeStartCandleAnimCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.start_dolly_slide_anim
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeStartDollySlideAnimCallback_t) (uint8_t use_default, float speed, float angle, float horizontal_distance, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.start_dolly_slide_anim
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.start_dolly_slide_anim
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStartDollySlideAnimCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeStartDollySlideAnimCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.start_dolly_slide_anim
 * @param callback new callback for the command follow_me.start_dolly_slide_anim
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStartDollySlideAnimCallback (ARCOMMANDS_Decoder_FollowMeStartDollySlideAnimCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.target_framing_position
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeTargetFramingPositionCallback_t) (int8_t horizontal, int8_t vertical, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.target_framing_position
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.target_framing_position
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeTargetFramingPositionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeTargetFramingPositionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.target_framing_position
 * @param callback new callback for the command follow_me.target_framing_position
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeTargetFramingPositionCallback (ARCOMMANDS_Decoder_FollowMeTargetFramingPositionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.target_image_detection
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeTargetImageDetectionCallback_t) (float target_azimuth, float target_elevation, float change_of_scale, uint8_t confidence_index, uint8_t is_new_selection, uint64_t timestamp, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.target_image_detection
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.target_image_detection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeTargetImageDetectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeTargetImageDetectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.target_image_detection
 * @param callback new callback for the command follow_me.target_image_detection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeTargetImageDetectionCallback (ARCOMMANDS_Decoder_FollowMeTargetImageDetectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.state
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeStateCallback_t) (eARCOMMANDS_FOLLOW_ME_MODE mode, eARCOMMANDS_FOLLOW_ME_BEHAVIOR behavior, eARCOMMANDS_FOLLOW_ME_ANIMATION animation, uint16_t animation_available, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.state
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.state
 * @param callback new callback for the command follow_me.state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeStateCallback (ARCOMMANDS_Decoder_FollowMeStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.mode_info
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeModeInfoCallback_t) (eARCOMMANDS_FOLLOW_ME_MODE mode, uint16_t missing_requirements, uint16_t improvements, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.mode_info
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.mode_info
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeModeInfoCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeModeInfoCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.mode_info
 * @param callback new callback for the command follow_me.mode_info
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeModeInfoCallback (ARCOMMANDS_Decoder_FollowMeModeInfoCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.geographic_config
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeGeographicConfigCallback_t) (uint8_t use_default, float distance, float elevation, float azimuth, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.geographic_config
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.geographic_config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeGeographicConfigCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeGeographicConfigCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.geographic_config
 * @param callback new callback for the command follow_me.geographic_config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeGeographicConfigCallback (ARCOMMANDS_Decoder_FollowMeGeographicConfigCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.relative_config
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeRelativeConfigCallback_t) (uint8_t use_default, float distance, float elevation, float azimuth, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.relative_config
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.relative_config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeRelativeConfigCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeRelativeConfigCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.relative_config
 * @param callback new callback for the command follow_me.relative_config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeRelativeConfigCallback (ARCOMMANDS_Decoder_FollowMeRelativeConfigCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.target_trajectory
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeTargetTrajectoryCallback_t) (double latitude, double longitude, float altitude, float north_speed, float east_speed, float down_speed, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.target_trajectory
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.target_trajectory
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeTargetTrajectoryCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeTargetTrajectoryCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.target_trajectory
 * @param callback new callback for the command follow_me.target_trajectory
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeTargetTrajectoryCallback (ARCOMMANDS_Decoder_FollowMeTargetTrajectoryCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.helicoid_anim_config
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeHelicoidAnimConfigCallback_t) (uint8_t use_default, float speed, float revolution_nb, float vertical_distance, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.helicoid_anim_config
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.helicoid_anim_config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeHelicoidAnimConfigCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeHelicoidAnimConfigCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.helicoid_anim_config
 * @param callback new callback for the command follow_me.helicoid_anim_config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeHelicoidAnimConfigCallback (ARCOMMANDS_Decoder_FollowMeHelicoidAnimConfigCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.swing_anim_config
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeSwingAnimConfigCallback_t) (uint8_t use_default, float speed, float vertical_distance, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.swing_anim_config
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.swing_anim_config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeSwingAnimConfigCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeSwingAnimConfigCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.swing_anim_config
 * @param callback new callback for the command follow_me.swing_anim_config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeSwingAnimConfigCallback (ARCOMMANDS_Decoder_FollowMeSwingAnimConfigCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.boomerang_anim_config
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeBoomerangAnimConfigCallback_t) (uint8_t use_default, float speed, float distance, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.boomerang_anim_config
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.boomerang_anim_config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeBoomerangAnimConfigCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeBoomerangAnimConfigCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.boomerang_anim_config
 * @param callback new callback for the command follow_me.boomerang_anim_config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeBoomerangAnimConfigCallback (ARCOMMANDS_Decoder_FollowMeBoomerangAnimConfigCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.candle_anim_config
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeCandleAnimConfigCallback_t) (uint8_t use_default, float speed, float vertical_distance, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.candle_anim_config
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.candle_anim_config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeCandleAnimConfigCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeCandleAnimConfigCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.candle_anim_config
 * @param callback new callback for the command follow_me.candle_anim_config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeCandleAnimConfigCallback (ARCOMMANDS_Decoder_FollowMeCandleAnimConfigCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.dolly_slide_anim_config
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeDollySlideAnimConfigCallback_t) (uint8_t use_default, float speed, float angle, float horizontal_distance, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.dolly_slide_anim_config
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.dolly_slide_anim_config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeDollySlideAnimConfigCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeDollySlideAnimConfigCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.dolly_slide_anim_config
 * @param callback new callback for the command follow_me.dolly_slide_anim_config
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeDollySlideAnimConfigCallback (ARCOMMANDS_Decoder_FollowMeDollySlideAnimConfigCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.target_framing_position_changed
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeTargetFramingPositionChangedCallback_t) (int8_t horizontal, int8_t vertical, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.target_framing_position_changed
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.target_framing_position_changed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeTargetFramingPositionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeTargetFramingPositionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.target_framing_position_changed
 * @param callback new callback for the command follow_me.target_framing_position_changed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeTargetFramingPositionChangedCallback (ARCOMMANDS_Decoder_FollowMeTargetFramingPositionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command follow_me.target_image_detection_state
 */
typedef void (*ARCOMMANDS_Decoder_FollowMeTargetImageDetectionStateCallback_t) (eARCOMMANDS_FOLLOW_ME_IMAGE_DETECTION_STATUS state, void *custom);
/**
 * @brief Set Decoder callback setter for the command follow_me.target_image_detection_state
 * @param decoder the decoder instance
 * @param callback new callback for the command follow_me.target_image_detection_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeTargetImageDetectionStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_FollowMeTargetImageDetectionStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command follow_me.target_image_detection_state
 * @param callback new callback for the command follow_me.target_image_detection_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetFollowMeTargetImageDetectionStateCallback (ARCOMMANDS_Decoder_FollowMeTargetImageDetectionStateCallback_t callback, void *custom) DEPRECATED;

// Feature JumpingSumo


/**
 * @brief callback type for the command JumpingSumo.Piloting.PCMD
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoPilotingPCMDCallback_t) (uint8_t flag, int8_t speed, int8_t turn, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.Piloting.PCMD
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.Piloting.PCMD
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoPilotingPCMDCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoPilotingPCMDCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.Piloting.PCMD
 * @param callback new callback for the command JumpingSumo.Piloting.PCMD
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoPilotingPCMDCallback (ARCOMMANDS_Decoder_JumpingSumoPilotingPCMDCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.Piloting.Posture
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoPilotingPostureCallback_t) (eARCOMMANDS_JUMPINGSUMO_PILOTING_POSTURE_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.Piloting.Posture
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.Piloting.Posture
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoPilotingPostureCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoPilotingPostureCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.Piloting.Posture
 * @param callback new callback for the command JumpingSumo.Piloting.Posture
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoPilotingPostureCallback (ARCOMMANDS_Decoder_JumpingSumoPilotingPostureCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.Piloting.addCapOffset
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoPilotingAddCapOffsetCallback_t) (float offset, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.Piloting.addCapOffset
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.Piloting.addCapOffset
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoPilotingAddCapOffsetCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoPilotingAddCapOffsetCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.Piloting.addCapOffset
 * @param callback new callback for the command JumpingSumo.Piloting.addCapOffset
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoPilotingAddCapOffsetCallback (ARCOMMANDS_Decoder_JumpingSumoPilotingAddCapOffsetCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.Animations.JumpStop
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoAnimationsJumpStopCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.Animations.JumpStop
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.Animations.JumpStop
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsJumpStopCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoAnimationsJumpStopCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.Animations.JumpStop
 * @param callback new callback for the command JumpingSumo.Animations.JumpStop
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsJumpStopCallback (ARCOMMANDS_Decoder_JumpingSumoAnimationsJumpStopCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.Animations.JumpCancel
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoAnimationsJumpCancelCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.Animations.JumpCancel
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.Animations.JumpCancel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsJumpCancelCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoAnimationsJumpCancelCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.Animations.JumpCancel
 * @param callback new callback for the command JumpingSumo.Animations.JumpCancel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsJumpCancelCallback (ARCOMMANDS_Decoder_JumpingSumoAnimationsJumpCancelCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.Animations.JumpLoad
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoAnimationsJumpLoadCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.Animations.JumpLoad
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.Animations.JumpLoad
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsJumpLoadCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoAnimationsJumpLoadCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.Animations.JumpLoad
 * @param callback new callback for the command JumpingSumo.Animations.JumpLoad
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsJumpLoadCallback (ARCOMMANDS_Decoder_JumpingSumoAnimationsJumpLoadCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.Animations.Jump
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoAnimationsJumpCallback_t) (eARCOMMANDS_JUMPINGSUMO_ANIMATIONS_JUMP_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.Animations.Jump
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.Animations.Jump
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsJumpCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoAnimationsJumpCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.Animations.Jump
 * @param callback new callback for the command JumpingSumo.Animations.Jump
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsJumpCallback (ARCOMMANDS_Decoder_JumpingSumoAnimationsJumpCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.Animations.SimpleAnimation
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoAnimationsSimpleAnimationCallback_t) (eARCOMMANDS_JUMPINGSUMO_ANIMATIONS_SIMPLEANIMATION_ID id, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.Animations.SimpleAnimation
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.Animations.SimpleAnimation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsSimpleAnimationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoAnimationsSimpleAnimationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.Animations.SimpleAnimation
 * @param callback new callback for the command JumpingSumo.Animations.SimpleAnimation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsSimpleAnimationCallback (ARCOMMANDS_Decoder_JumpingSumoAnimationsSimpleAnimationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.MediaRecord.Picture
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoMediaRecordPictureCallback_t) (uint8_t mass_storage_id, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.MediaRecord.Picture
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.MediaRecord.Picture
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordPictureCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoMediaRecordPictureCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.MediaRecord.Picture
 * @param callback new callback for the command JumpingSumo.MediaRecord.Picture
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordPictureCallback (ARCOMMANDS_Decoder_JumpingSumoMediaRecordPictureCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.MediaRecord.Video
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoMediaRecordVideoCallback_t) (eARCOMMANDS_JUMPINGSUMO_MEDIARECORD_VIDEO_RECORD record, uint8_t mass_storage_id, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.MediaRecord.Video
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.MediaRecord.Video
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordVideoCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoMediaRecordVideoCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.MediaRecord.Video
 * @param callback new callback for the command JumpingSumo.MediaRecord.Video
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordVideoCallback (ARCOMMANDS_Decoder_JumpingSumoMediaRecordVideoCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.MediaRecord.PictureV2
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoMediaRecordPictureV2Callback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.MediaRecord.PictureV2
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.MediaRecord.PictureV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordPictureV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoMediaRecordPictureV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.MediaRecord.PictureV2
 * @param callback new callback for the command JumpingSumo.MediaRecord.PictureV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordPictureV2Callback (ARCOMMANDS_Decoder_JumpingSumoMediaRecordPictureV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.MediaRecord.VideoV2
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoMediaRecordVideoV2Callback_t) (eARCOMMANDS_JUMPINGSUMO_MEDIARECORD_VIDEOV2_RECORD record, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.MediaRecord.VideoV2
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.MediaRecord.VideoV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordVideoV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoMediaRecordVideoV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.MediaRecord.VideoV2
 * @param callback new callback for the command JumpingSumo.MediaRecord.VideoV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordVideoV2Callback (ARCOMMANDS_Decoder_JumpingSumoMediaRecordVideoV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.NetworkSettings.WifiSelection
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoNetworkSettingsWifiSelectionCallback_t) (eARCOMMANDS_JUMPINGSUMO_NETWORKSETTINGS_WIFISELECTION_TYPE type, eARCOMMANDS_JUMPINGSUMO_NETWORKSETTINGS_WIFISELECTION_BAND band, uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.NetworkSettings.WifiSelection
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.NetworkSettings.WifiSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkSettingsWifiSelectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoNetworkSettingsWifiSelectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.NetworkSettings.WifiSelection
 * @param callback new callback for the command JumpingSumo.NetworkSettings.WifiSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkSettingsWifiSelectionCallback (ARCOMMANDS_Decoder_JumpingSumoNetworkSettingsWifiSelectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.Network.WifiScan
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoNetworkWifiScanCallback_t) (eARCOMMANDS_JUMPINGSUMO_NETWORK_WIFISCAN_BAND band, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.Network.WifiScan
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.Network.WifiScan
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkWifiScanCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoNetworkWifiScanCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.Network.WifiScan
 * @param callback new callback for the command JumpingSumo.Network.WifiScan
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkWifiScanCallback (ARCOMMANDS_Decoder_JumpingSumoNetworkWifiScanCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.Network.WifiAuthChannel
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoNetworkWifiAuthChannelCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.Network.WifiAuthChannel
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.Network.WifiAuthChannel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkWifiAuthChannelCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoNetworkWifiAuthChannelCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.Network.WifiAuthChannel
 * @param callback new callback for the command JumpingSumo.Network.WifiAuthChannel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkWifiAuthChannelCallback (ARCOMMANDS_Decoder_JumpingSumoNetworkWifiAuthChannelCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.AudioSettings.MasterVolume
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoAudioSettingsMasterVolumeCallback_t) (uint8_t volume, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.AudioSettings.MasterVolume
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.AudioSettings.MasterVolume
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAudioSettingsMasterVolumeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoAudioSettingsMasterVolumeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.AudioSettings.MasterVolume
 * @param callback new callback for the command JumpingSumo.AudioSettings.MasterVolume
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAudioSettingsMasterVolumeCallback (ARCOMMANDS_Decoder_JumpingSumoAudioSettingsMasterVolumeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.AudioSettings.Theme
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoAudioSettingsThemeCallback_t) (eARCOMMANDS_JUMPINGSUMO_AUDIOSETTINGS_THEME_THEME theme, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.AudioSettings.Theme
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.AudioSettings.Theme
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAudioSettingsThemeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoAudioSettingsThemeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.AudioSettings.Theme
 * @param callback new callback for the command JumpingSumo.AudioSettings.Theme
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAudioSettingsThemeCallback (ARCOMMANDS_Decoder_JumpingSumoAudioSettingsThemeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.RoadPlan.AllScriptsMetadata
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoRoadPlanAllScriptsMetadataCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.RoadPlan.AllScriptsMetadata
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.RoadPlan.AllScriptsMetadata
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanAllScriptsMetadataCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoRoadPlanAllScriptsMetadataCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.RoadPlan.AllScriptsMetadata
 * @param callback new callback for the command JumpingSumo.RoadPlan.AllScriptsMetadata
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanAllScriptsMetadataCallback (ARCOMMANDS_Decoder_JumpingSumoRoadPlanAllScriptsMetadataCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.RoadPlan.ScriptUploaded
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoRoadPlanScriptUploadedCallback_t) (const char * uuid, const char * md5Hash, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.RoadPlan.ScriptUploaded
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.RoadPlan.ScriptUploaded
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanScriptUploadedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoRoadPlanScriptUploadedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.RoadPlan.ScriptUploaded
 * @param callback new callback for the command JumpingSumo.RoadPlan.ScriptUploaded
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanScriptUploadedCallback (ARCOMMANDS_Decoder_JumpingSumoRoadPlanScriptUploadedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.RoadPlan.ScriptDelete
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoRoadPlanScriptDeleteCallback_t) (const char * uuid, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.RoadPlan.ScriptDelete
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.RoadPlan.ScriptDelete
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanScriptDeleteCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoRoadPlanScriptDeleteCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.RoadPlan.ScriptDelete
 * @param callback new callback for the command JumpingSumo.RoadPlan.ScriptDelete
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanScriptDeleteCallback (ARCOMMANDS_Decoder_JumpingSumoRoadPlanScriptDeleteCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.RoadPlan.PlayScript
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoRoadPlanPlayScriptCallback_t) (const char * uuid, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.RoadPlan.PlayScript
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.RoadPlan.PlayScript
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanPlayScriptCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoRoadPlanPlayScriptCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.RoadPlan.PlayScript
 * @param callback new callback for the command JumpingSumo.RoadPlan.PlayScript
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanPlayScriptCallback (ARCOMMANDS_Decoder_JumpingSumoRoadPlanPlayScriptCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.SpeedSettings.Outdoor
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoSpeedSettingsOutdoorCallback_t) (uint8_t outdoor, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.SpeedSettings.Outdoor
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.SpeedSettings.Outdoor
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoSpeedSettingsOutdoorCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoSpeedSettingsOutdoorCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.SpeedSettings.Outdoor
 * @param callback new callback for the command JumpingSumo.SpeedSettings.Outdoor
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoSpeedSettingsOutdoorCallback (ARCOMMANDS_Decoder_JumpingSumoSpeedSettingsOutdoorCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.MediaStreaming.VideoEnable
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoMediaStreamingVideoEnableCallback_t) (uint8_t enable, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.MediaStreaming.VideoEnable
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.MediaStreaming.VideoEnable
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaStreamingVideoEnableCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoMediaStreamingVideoEnableCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.MediaStreaming.VideoEnable
 * @param callback new callback for the command JumpingSumo.MediaStreaming.VideoEnable
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaStreamingVideoEnableCallback (ARCOMMANDS_Decoder_JumpingSumoMediaStreamingVideoEnableCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.VideoSettings.Autorecord
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoVideoSettingsAutorecordCallback_t) (uint8_t enabled, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.VideoSettings.Autorecord
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.VideoSettings.Autorecord
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoVideoSettingsAutorecordCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoVideoSettingsAutorecordCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.VideoSettings.Autorecord
 * @param callback new callback for the command JumpingSumo.VideoSettings.Autorecord
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoVideoSettingsAutorecordCallback (ARCOMMANDS_Decoder_JumpingSumoVideoSettingsAutorecordCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.PilotingState.PostureChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoPilotingStatePostureChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_PILOTINGSTATE_POSTURECHANGED_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.PilotingState.PostureChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.PilotingState.PostureChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoPilotingStatePostureChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoPilotingStatePostureChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.PilotingState.PostureChanged
 * @param callback new callback for the command JumpingSumo.PilotingState.PostureChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoPilotingStatePostureChangedCallback (ARCOMMANDS_Decoder_JumpingSumoPilotingStatePostureChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.PilotingState.AlertStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoPilotingStateAlertStateChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_PILOTINGSTATE_ALERTSTATECHANGED_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.PilotingState.AlertStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.PilotingState.AlertStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoPilotingStateAlertStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoPilotingStateAlertStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.PilotingState.AlertStateChanged
 * @param callback new callback for the command JumpingSumo.PilotingState.AlertStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoPilotingStateAlertStateChangedCallback (ARCOMMANDS_Decoder_JumpingSumoPilotingStateAlertStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.PilotingState.SpeedChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoPilotingStateSpeedChangedCallback_t) (int8_t speed, int16_t realSpeed, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.PilotingState.SpeedChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.PilotingState.SpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoPilotingStateSpeedChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoPilotingStateSpeedChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.PilotingState.SpeedChanged
 * @param callback new callback for the command JumpingSumo.PilotingState.SpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoPilotingStateSpeedChangedCallback (ARCOMMANDS_Decoder_JumpingSumoPilotingStateSpeedChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.AnimationsState.JumpLoadChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoAnimationsStateJumpLoadChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_ANIMATIONSSTATE_JUMPLOADCHANGED_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.AnimationsState.JumpLoadChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.AnimationsState.JumpLoadChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsStateJumpLoadChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoAnimationsStateJumpLoadChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.AnimationsState.JumpLoadChanged
 * @param callback new callback for the command JumpingSumo.AnimationsState.JumpLoadChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsStateJumpLoadChangedCallback (ARCOMMANDS_Decoder_JumpingSumoAnimationsStateJumpLoadChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.AnimationsState.JumpTypeChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoAnimationsStateJumpTypeChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_ANIMATIONSSTATE_JUMPTYPECHANGED_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.AnimationsState.JumpTypeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.AnimationsState.JumpTypeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsStateJumpTypeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoAnimationsStateJumpTypeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.AnimationsState.JumpTypeChanged
 * @param callback new callback for the command JumpingSumo.AnimationsState.JumpTypeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsStateJumpTypeChangedCallback (ARCOMMANDS_Decoder_JumpingSumoAnimationsStateJumpTypeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.AnimationsState.JumpMotorProblemChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoAnimationsStateJumpMotorProblemChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_ANIMATIONSSTATE_JUMPMOTORPROBLEMCHANGED_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.AnimationsState.JumpMotorProblemChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.AnimationsState.JumpMotorProblemChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsStateJumpMotorProblemChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoAnimationsStateJumpMotorProblemChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.AnimationsState.JumpMotorProblemChanged
 * @param callback new callback for the command JumpingSumo.AnimationsState.JumpMotorProblemChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAnimationsStateJumpMotorProblemChangedCallback (ARCOMMANDS_Decoder_JumpingSumoAnimationsStateJumpMotorProblemChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.SettingsState.ProductGPSVersionChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoSettingsStateProductGPSVersionChangedCallback_t) (const char * software, const char * hardware, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.SettingsState.ProductGPSVersionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.SettingsState.ProductGPSVersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoSettingsStateProductGPSVersionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoSettingsStateProductGPSVersionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.SettingsState.ProductGPSVersionChanged
 * @param callback new callback for the command JumpingSumo.SettingsState.ProductGPSVersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoSettingsStateProductGPSVersionChangedCallback (ARCOMMANDS_Decoder_JumpingSumoSettingsStateProductGPSVersionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.MediaRecordState.PictureStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoMediaRecordStatePictureStateChangedCallback_t) (uint8_t state, uint8_t mass_storage_id, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.MediaRecordState.PictureStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.MediaRecordState.PictureStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordStatePictureStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoMediaRecordStatePictureStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.MediaRecordState.PictureStateChanged
 * @param callback new callback for the command JumpingSumo.MediaRecordState.PictureStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordStatePictureStateChangedCallback (ARCOMMANDS_Decoder_JumpingSumoMediaRecordStatePictureStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.MediaRecordState.VideoStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoMediaRecordStateVideoStateChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_MEDIARECORDSTATE_VIDEOSTATECHANGED_STATE state, uint8_t mass_storage_id, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.MediaRecordState.VideoStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.MediaRecordState.VideoStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordStateVideoStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoMediaRecordStateVideoStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.MediaRecordState.VideoStateChanged
 * @param callback new callback for the command JumpingSumo.MediaRecordState.VideoStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordStateVideoStateChangedCallback (ARCOMMANDS_Decoder_JumpingSumoMediaRecordStateVideoStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.MediaRecordState.PictureStateChangedV2
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoMediaRecordStatePictureStateChangedV2Callback_t) (eARCOMMANDS_JUMPINGSUMO_MEDIARECORDSTATE_PICTURESTATECHANGEDV2_STATE state, eARCOMMANDS_JUMPINGSUMO_MEDIARECORDSTATE_PICTURESTATECHANGEDV2_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.MediaRecordState.PictureStateChangedV2
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.MediaRecordState.PictureStateChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordStatePictureStateChangedV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoMediaRecordStatePictureStateChangedV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.MediaRecordState.PictureStateChangedV2
 * @param callback new callback for the command JumpingSumo.MediaRecordState.PictureStateChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordStatePictureStateChangedV2Callback (ARCOMMANDS_Decoder_JumpingSumoMediaRecordStatePictureStateChangedV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.MediaRecordState.VideoStateChangedV2
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoMediaRecordStateVideoStateChangedV2Callback_t) (eARCOMMANDS_JUMPINGSUMO_MEDIARECORDSTATE_VIDEOSTATECHANGEDV2_STATE state, eARCOMMANDS_JUMPINGSUMO_MEDIARECORDSTATE_VIDEOSTATECHANGEDV2_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.MediaRecordState.VideoStateChangedV2
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.MediaRecordState.VideoStateChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordStateVideoStateChangedV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoMediaRecordStateVideoStateChangedV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.MediaRecordState.VideoStateChangedV2
 * @param callback new callback for the command JumpingSumo.MediaRecordState.VideoStateChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordStateVideoStateChangedV2Callback (ARCOMMANDS_Decoder_JumpingSumoMediaRecordStateVideoStateChangedV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.MediaRecordEvent.PictureEventChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoMediaRecordEventPictureEventChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_MEDIARECORDEVENT_PICTUREEVENTCHANGED_EVENT event, eARCOMMANDS_JUMPINGSUMO_MEDIARECORDEVENT_PICTUREEVENTCHANGED_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.MediaRecordEvent.PictureEventChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.MediaRecordEvent.PictureEventChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordEventPictureEventChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoMediaRecordEventPictureEventChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.MediaRecordEvent.PictureEventChanged
 * @param callback new callback for the command JumpingSumo.MediaRecordEvent.PictureEventChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordEventPictureEventChangedCallback (ARCOMMANDS_Decoder_JumpingSumoMediaRecordEventPictureEventChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.MediaRecordEvent.VideoEventChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoMediaRecordEventVideoEventChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_MEDIARECORDEVENT_VIDEOEVENTCHANGED_EVENT event, eARCOMMANDS_JUMPINGSUMO_MEDIARECORDEVENT_VIDEOEVENTCHANGED_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.MediaRecordEvent.VideoEventChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.MediaRecordEvent.VideoEventChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordEventVideoEventChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoMediaRecordEventVideoEventChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.MediaRecordEvent.VideoEventChanged
 * @param callback new callback for the command JumpingSumo.MediaRecordEvent.VideoEventChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaRecordEventVideoEventChangedCallback (ARCOMMANDS_Decoder_JumpingSumoMediaRecordEventVideoEventChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.NetworkSettingsState.WifiSelectionChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoNetworkSettingsStateWifiSelectionChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_NETWORKSETTINGSSTATE_WIFISELECTIONCHANGED_TYPE type, eARCOMMANDS_JUMPINGSUMO_NETWORKSETTINGSSTATE_WIFISELECTIONCHANGED_BAND band, uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.NetworkSettingsState.WifiSelectionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.NetworkSettingsState.WifiSelectionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkSettingsStateWifiSelectionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoNetworkSettingsStateWifiSelectionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.NetworkSettingsState.WifiSelectionChanged
 * @param callback new callback for the command JumpingSumo.NetworkSettingsState.WifiSelectionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkSettingsStateWifiSelectionChangedCallback (ARCOMMANDS_Decoder_JumpingSumoNetworkSettingsStateWifiSelectionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.NetworkState.WifiScanListChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoNetworkStateWifiScanListChangedCallback_t) (const char * ssid, int16_t rssi, eARCOMMANDS_JUMPINGSUMO_NETWORKSTATE_WIFISCANLISTCHANGED_BAND band, uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.NetworkState.WifiScanListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.NetworkState.WifiScanListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkStateWifiScanListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoNetworkStateWifiScanListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.NetworkState.WifiScanListChanged
 * @param callback new callback for the command JumpingSumo.NetworkState.WifiScanListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkStateWifiScanListChangedCallback (ARCOMMANDS_Decoder_JumpingSumoNetworkStateWifiScanListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.NetworkState.AllWifiScanChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoNetworkStateAllWifiScanChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.NetworkState.AllWifiScanChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.NetworkState.AllWifiScanChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkStateAllWifiScanChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoNetworkStateAllWifiScanChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.NetworkState.AllWifiScanChanged
 * @param callback new callback for the command JumpingSumo.NetworkState.AllWifiScanChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkStateAllWifiScanChangedCallback (ARCOMMANDS_Decoder_JumpingSumoNetworkStateAllWifiScanChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.NetworkState.WifiAuthChannelListChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoNetworkStateWifiAuthChannelListChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_NETWORKSTATE_WIFIAUTHCHANNELLISTCHANGED_BAND band, uint8_t channel, uint8_t in_or_out, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.NetworkState.WifiAuthChannelListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.NetworkState.WifiAuthChannelListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkStateWifiAuthChannelListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoNetworkStateWifiAuthChannelListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.NetworkState.WifiAuthChannelListChanged
 * @param callback new callback for the command JumpingSumo.NetworkState.WifiAuthChannelListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkStateWifiAuthChannelListChangedCallback (ARCOMMANDS_Decoder_JumpingSumoNetworkStateWifiAuthChannelListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.NetworkState.AllWifiAuthChannelChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoNetworkStateAllWifiAuthChannelChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.NetworkState.AllWifiAuthChannelChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.NetworkState.AllWifiAuthChannelChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkStateAllWifiAuthChannelChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoNetworkStateAllWifiAuthChannelChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.NetworkState.AllWifiAuthChannelChanged
 * @param callback new callback for the command JumpingSumo.NetworkState.AllWifiAuthChannelChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkStateAllWifiAuthChannelChangedCallback (ARCOMMANDS_Decoder_JumpingSumoNetworkStateAllWifiAuthChannelChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.NetworkState.LinkQualityChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoNetworkStateLinkQualityChangedCallback_t) (uint8_t quality, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.NetworkState.LinkQualityChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.NetworkState.LinkQualityChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkStateLinkQualityChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoNetworkStateLinkQualityChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.NetworkState.LinkQualityChanged
 * @param callback new callback for the command JumpingSumo.NetworkState.LinkQualityChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoNetworkStateLinkQualityChangedCallback (ARCOMMANDS_Decoder_JumpingSumoNetworkStateLinkQualityChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.AudioSettingsState.MasterVolumeChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoAudioSettingsStateMasterVolumeChangedCallback_t) (uint8_t volume, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.AudioSettingsState.MasterVolumeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.AudioSettingsState.MasterVolumeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAudioSettingsStateMasterVolumeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoAudioSettingsStateMasterVolumeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.AudioSettingsState.MasterVolumeChanged
 * @param callback new callback for the command JumpingSumo.AudioSettingsState.MasterVolumeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAudioSettingsStateMasterVolumeChangedCallback (ARCOMMANDS_Decoder_JumpingSumoAudioSettingsStateMasterVolumeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.AudioSettingsState.ThemeChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoAudioSettingsStateThemeChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_AUDIOSETTINGSSTATE_THEMECHANGED_THEME theme, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.AudioSettingsState.ThemeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.AudioSettingsState.ThemeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAudioSettingsStateThemeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoAudioSettingsStateThemeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.AudioSettingsState.ThemeChanged
 * @param callback new callback for the command JumpingSumo.AudioSettingsState.ThemeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoAudioSettingsStateThemeChangedCallback (ARCOMMANDS_Decoder_JumpingSumoAudioSettingsStateThemeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.RoadPlanState.ScriptMetadataListChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoRoadPlanStateScriptMetadataListChangedCallback_t) (const char * uuid, uint8_t version, const char * product, const char * name, uint64_t lastModified, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.RoadPlanState.ScriptMetadataListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.RoadPlanState.ScriptMetadataListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanStateScriptMetadataListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoRoadPlanStateScriptMetadataListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.RoadPlanState.ScriptMetadataListChanged
 * @param callback new callback for the command JumpingSumo.RoadPlanState.ScriptMetadataListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanStateScriptMetadataListChangedCallback (ARCOMMANDS_Decoder_JumpingSumoRoadPlanStateScriptMetadataListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.RoadPlanState.AllScriptsMetadataChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoRoadPlanStateAllScriptsMetadataChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.RoadPlanState.AllScriptsMetadataChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.RoadPlanState.AllScriptsMetadataChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanStateAllScriptsMetadataChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoRoadPlanStateAllScriptsMetadataChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.RoadPlanState.AllScriptsMetadataChanged
 * @param callback new callback for the command JumpingSumo.RoadPlanState.AllScriptsMetadataChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanStateAllScriptsMetadataChangedCallback (ARCOMMANDS_Decoder_JumpingSumoRoadPlanStateAllScriptsMetadataChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.RoadPlanState.ScriptUploadChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoRoadPlanStateScriptUploadChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_ROADPLANSTATE_SCRIPTUPLOADCHANGED_RESULTCODE resultCode, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.RoadPlanState.ScriptUploadChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.RoadPlanState.ScriptUploadChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanStateScriptUploadChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoRoadPlanStateScriptUploadChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.RoadPlanState.ScriptUploadChanged
 * @param callback new callback for the command JumpingSumo.RoadPlanState.ScriptUploadChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanStateScriptUploadChangedCallback (ARCOMMANDS_Decoder_JumpingSumoRoadPlanStateScriptUploadChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.RoadPlanState.ScriptDeleteChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoRoadPlanStateScriptDeleteChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_ROADPLANSTATE_SCRIPTDELETECHANGED_RESULTCODE resultCode, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.RoadPlanState.ScriptDeleteChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.RoadPlanState.ScriptDeleteChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanStateScriptDeleteChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoRoadPlanStateScriptDeleteChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.RoadPlanState.ScriptDeleteChanged
 * @param callback new callback for the command JumpingSumo.RoadPlanState.ScriptDeleteChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanStateScriptDeleteChangedCallback (ARCOMMANDS_Decoder_JumpingSumoRoadPlanStateScriptDeleteChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.RoadPlanState.PlayScriptChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoRoadPlanStatePlayScriptChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_ROADPLANSTATE_PLAYSCRIPTCHANGED_RESULTCODE resultCode, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.RoadPlanState.PlayScriptChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.RoadPlanState.PlayScriptChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanStatePlayScriptChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoRoadPlanStatePlayScriptChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.RoadPlanState.PlayScriptChanged
 * @param callback new callback for the command JumpingSumo.RoadPlanState.PlayScriptChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoRoadPlanStatePlayScriptChangedCallback (ARCOMMANDS_Decoder_JumpingSumoRoadPlanStatePlayScriptChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.SpeedSettingsState.OutdoorChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoSpeedSettingsStateOutdoorChangedCallback_t) (uint8_t outdoor, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.SpeedSettingsState.OutdoorChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.SpeedSettingsState.OutdoorChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoSpeedSettingsStateOutdoorChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoSpeedSettingsStateOutdoorChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.SpeedSettingsState.OutdoorChanged
 * @param callback new callback for the command JumpingSumo.SpeedSettingsState.OutdoorChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoSpeedSettingsStateOutdoorChangedCallback (ARCOMMANDS_Decoder_JumpingSumoSpeedSettingsStateOutdoorChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.MediaStreamingState.VideoEnableChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoMediaStreamingStateVideoEnableChangedCallback_t) (eARCOMMANDS_JUMPINGSUMO_MEDIASTREAMINGSTATE_VIDEOENABLECHANGED_ENABLED enabled, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.MediaStreamingState.VideoEnableChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.MediaStreamingState.VideoEnableChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaStreamingStateVideoEnableChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoMediaStreamingStateVideoEnableChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.MediaStreamingState.VideoEnableChanged
 * @param callback new callback for the command JumpingSumo.MediaStreamingState.VideoEnableChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoMediaStreamingStateVideoEnableChangedCallback (ARCOMMANDS_Decoder_JumpingSumoMediaStreamingStateVideoEnableChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command JumpingSumo.VideoSettingsState.AutorecordChanged
 */
typedef void (*ARCOMMANDS_Decoder_JumpingSumoVideoSettingsStateAutorecordChangedCallback_t) (uint8_t enabled, void *custom);
/**
 * @brief Set Decoder callback setter for the command JumpingSumo.VideoSettingsState.AutorecordChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command JumpingSumo.VideoSettingsState.AutorecordChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoVideoSettingsStateAutorecordChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_JumpingSumoVideoSettingsStateAutorecordChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command JumpingSumo.VideoSettingsState.AutorecordChanged
 * @param callback new callback for the command JumpingSumo.VideoSettingsState.AutorecordChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetJumpingSumoVideoSettingsStateAutorecordChangedCallback (ARCOMMANDS_Decoder_JumpingSumoVideoSettingsStateAutorecordChangedCallback_t callback, void *custom) DEPRECATED;

// Feature mapper


/**
 * @brief callback type for the command mapper.grab
 */
typedef void (*ARCOMMANDS_Decoder_MapperGrabCallback_t) (uint32_t buttons, uint32_t axes, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.grab
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.grab
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperGrabCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperGrabCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.grab
 * @param callback new callback for the command mapper.grab
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperGrabCallback (ARCOMMANDS_Decoder_MapperGrabCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.map_button_action
 */
typedef void (*ARCOMMANDS_Decoder_MapperMapButtonActionCallback_t) (uint16_t product, eARCOMMANDS_MAPPER_BUTTON_ACTION action, uint32_t buttons, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.map_button_action
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.map_button_action
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperMapButtonActionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperMapButtonActionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.map_button_action
 * @param callback new callback for the command mapper.map_button_action
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperMapButtonActionCallback (ARCOMMANDS_Decoder_MapperMapButtonActionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.map_axis_action
 */
typedef void (*ARCOMMANDS_Decoder_MapperMapAxisActionCallback_t) (uint16_t product, eARCOMMANDS_MAPPER_AXIS_ACTION action, int32_t axis, uint32_t buttons, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.map_axis_action
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.map_axis_action
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperMapAxisActionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperMapAxisActionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.map_axis_action
 * @param callback new callback for the command mapper.map_axis_action
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperMapAxisActionCallback (ARCOMMANDS_Decoder_MapperMapAxisActionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.reset_mapping
 */
typedef void (*ARCOMMANDS_Decoder_MapperResetMappingCallback_t) (uint16_t product, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.reset_mapping
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.reset_mapping
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperResetMappingCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperResetMappingCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.reset_mapping
 * @param callback new callback for the command mapper.reset_mapping
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperResetMappingCallback (ARCOMMANDS_Decoder_MapperResetMappingCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.set_expo
 */
typedef void (*ARCOMMANDS_Decoder_MapperSetExpoCallback_t) (uint16_t product, int32_t axis, eARCOMMANDS_MAPPER_EXPO_TYPE expo, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.set_expo
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.set_expo
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperSetExpoCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperSetExpoCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.set_expo
 * @param callback new callback for the command mapper.set_expo
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperSetExpoCallback (ARCOMMANDS_Decoder_MapperSetExpoCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.set_inverted
 */
typedef void (*ARCOMMANDS_Decoder_MapperSetInvertedCallback_t) (uint16_t product, int32_t axis, uint8_t inverted, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.set_inverted
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.set_inverted
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperSetInvertedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperSetInvertedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.set_inverted
 * @param callback new callback for the command mapper.set_inverted
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperSetInvertedCallback (ARCOMMANDS_Decoder_MapperSetInvertedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.grab_state
 */
typedef void (*ARCOMMANDS_Decoder_MapperGrabStateCallback_t) (uint32_t buttons, uint32_t axes, uint32_t buttons_state, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.grab_state
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.grab_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperGrabStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperGrabStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.grab_state
 * @param callback new callback for the command mapper.grab_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperGrabStateCallback (ARCOMMANDS_Decoder_MapperGrabStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.grab_button_event
 */
typedef void (*ARCOMMANDS_Decoder_MapperGrabButtonEventCallback_t) (uint32_t button, eARCOMMANDS_MAPPER_BUTTON_EVENT event, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.grab_button_event
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.grab_button_event
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperGrabButtonEventCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperGrabButtonEventCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.grab_button_event
 * @param callback new callback for the command mapper.grab_button_event
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperGrabButtonEventCallback (ARCOMMANDS_Decoder_MapperGrabButtonEventCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.grab_axis_event
 */
typedef void (*ARCOMMANDS_Decoder_MapperGrabAxisEventCallback_t) (uint32_t axis, int8_t value, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.grab_axis_event
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.grab_axis_event
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperGrabAxisEventCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperGrabAxisEventCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.grab_axis_event
 * @param callback new callback for the command mapper.grab_axis_event
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperGrabAxisEventCallback (ARCOMMANDS_Decoder_MapperGrabAxisEventCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.button_mapping_item
 */
typedef void (*ARCOMMANDS_Decoder_MapperButtonMappingItemCallback_t) (uint32_t uid, uint16_t product, eARCOMMANDS_MAPPER_BUTTON_ACTION action, uint32_t buttons, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.button_mapping_item
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.button_mapping_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperButtonMappingItemCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperButtonMappingItemCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.button_mapping_item
 * @param callback new callback for the command mapper.button_mapping_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperButtonMappingItemCallback (ARCOMMANDS_Decoder_MapperButtonMappingItemCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.axis_mapping_item
 */
typedef void (*ARCOMMANDS_Decoder_MapperAxisMappingItemCallback_t) (uint32_t uid, uint16_t product, eARCOMMANDS_MAPPER_AXIS_ACTION action, int32_t axis, uint32_t buttons, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.axis_mapping_item
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.axis_mapping_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperAxisMappingItemCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperAxisMappingItemCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.axis_mapping_item
 * @param callback new callback for the command mapper.axis_mapping_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperAxisMappingItemCallback (ARCOMMANDS_Decoder_MapperAxisMappingItemCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.application_axis_event
 */
typedef void (*ARCOMMANDS_Decoder_MapperApplicationAxisEventCallback_t) (eARCOMMANDS_MAPPER_AXIS_ACTION action, int8_t value, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.application_axis_event
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.application_axis_event
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperApplicationAxisEventCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperApplicationAxisEventCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.application_axis_event
 * @param callback new callback for the command mapper.application_axis_event
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperApplicationAxisEventCallback (ARCOMMANDS_Decoder_MapperApplicationAxisEventCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.application_button_event
 */
typedef void (*ARCOMMANDS_Decoder_MapperApplicationButtonEventCallback_t) (eARCOMMANDS_MAPPER_BUTTON_ACTION action, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.application_button_event
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.application_button_event
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperApplicationButtonEventCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperApplicationButtonEventCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.application_button_event
 * @param callback new callback for the command mapper.application_button_event
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperApplicationButtonEventCallback (ARCOMMANDS_Decoder_MapperApplicationButtonEventCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.expo_map_item
 */
typedef void (*ARCOMMANDS_Decoder_MapperExpoMapItemCallback_t) (uint32_t uid, uint16_t product, int32_t axis, eARCOMMANDS_MAPPER_EXPO_TYPE expo, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.expo_map_item
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.expo_map_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperExpoMapItemCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperExpoMapItemCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.expo_map_item
 * @param callback new callback for the command mapper.expo_map_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperExpoMapItemCallback (ARCOMMANDS_Decoder_MapperExpoMapItemCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.inverted_map_item
 */
typedef void (*ARCOMMANDS_Decoder_MapperInvertedMapItemCallback_t) (uint32_t uid, uint16_t product, int32_t axis, uint8_t inverted, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.inverted_map_item
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.inverted_map_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperInvertedMapItemCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperInvertedMapItemCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.inverted_map_item
 * @param callback new callback for the command mapper.inverted_map_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperInvertedMapItemCallback (ARCOMMANDS_Decoder_MapperInvertedMapItemCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper.active_product
 */
typedef void (*ARCOMMANDS_Decoder_MapperActiveProductCallback_t) (uint16_t product, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper.active_product
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper.active_product
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperActiveProductCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperActiveProductCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper.active_product
 * @param callback new callback for the command mapper.active_product
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperActiveProductCallback (ARCOMMANDS_Decoder_MapperActiveProductCallback_t callback, void *custom) DEPRECATED;

// Feature mapper_mini


/**
 * @brief callback type for the command mapper_mini.map_button_action
 */
typedef void (*ARCOMMANDS_Decoder_MapperMiniMapButtonActionCallback_t) (uint8_t modes, eARCOMMANDS_MAPPER_MINI_BUTTON_ACTION action, uint32_t buttons, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper_mini.map_button_action
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper_mini.map_button_action
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperMiniMapButtonActionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperMiniMapButtonActionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper_mini.map_button_action
 * @param callback new callback for the command mapper_mini.map_button_action
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperMiniMapButtonActionCallback (ARCOMMANDS_Decoder_MapperMiniMapButtonActionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper_mini.map_axis_action
 */
typedef void (*ARCOMMANDS_Decoder_MapperMiniMapAxisActionCallback_t) (uint8_t modes, eARCOMMANDS_MAPPER_MINI_AXIS_ACTION action, int8_t axis, uint32_t buttons, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper_mini.map_axis_action
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper_mini.map_axis_action
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperMiniMapAxisActionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperMiniMapAxisActionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper_mini.map_axis_action
 * @param callback new callback for the command mapper_mini.map_axis_action
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperMiniMapAxisActionCallback (ARCOMMANDS_Decoder_MapperMiniMapAxisActionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper_mini.reset_mapping
 */
typedef void (*ARCOMMANDS_Decoder_MapperMiniResetMappingCallback_t) (uint8_t modes, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper_mini.reset_mapping
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper_mini.reset_mapping
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperMiniResetMappingCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperMiniResetMappingCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper_mini.reset_mapping
 * @param callback new callback for the command mapper_mini.reset_mapping
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperMiniResetMappingCallback (ARCOMMANDS_Decoder_MapperMiniResetMappingCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper_mini.button_mapping_item
 */
typedef void (*ARCOMMANDS_Decoder_MapperMiniButtonMappingItemCallback_t) (uint16_t uid, uint8_t modes, eARCOMMANDS_MAPPER_MINI_BUTTON_ACTION action, uint32_t buttons, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper_mini.button_mapping_item
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper_mini.button_mapping_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperMiniButtonMappingItemCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperMiniButtonMappingItemCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper_mini.button_mapping_item
 * @param callback new callback for the command mapper_mini.button_mapping_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperMiniButtonMappingItemCallback (ARCOMMANDS_Decoder_MapperMiniButtonMappingItemCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command mapper_mini.axis_mapping_item
 */
typedef void (*ARCOMMANDS_Decoder_MapperMiniAxisMappingItemCallback_t) (uint16_t uid, uint8_t modes, eARCOMMANDS_MAPPER_MINI_AXIS_ACTION action, int8_t axis, uint32_t buttons, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command mapper_mini.axis_mapping_item
 * @param decoder the decoder instance
 * @param callback new callback for the command mapper_mini.axis_mapping_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperMiniAxisMappingItemCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MapperMiniAxisMappingItemCallback_t callback, void *custom);
/**
 * @brief callback setter for the command mapper_mini.axis_mapping_item
 * @param callback new callback for the command mapper_mini.axis_mapping_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMapperMiniAxisMappingItemCallback (ARCOMMANDS_Decoder_MapperMiniAxisMappingItemCallback_t callback, void *custom) DEPRECATED;

// Feature MiniDrone


/**
 * @brief callback type for the command MiniDrone.Piloting.FlatTrim
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingFlatTrimCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Piloting.FlatTrim
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Piloting.FlatTrim
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingFlatTrimCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingFlatTrimCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Piloting.FlatTrim
 * @param callback new callback for the command MiniDrone.Piloting.FlatTrim
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingFlatTrimCallback (ARCOMMANDS_Decoder_MiniDronePilotingFlatTrimCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Piloting.TakeOff
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingTakeOffCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Piloting.TakeOff
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Piloting.TakeOff
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingTakeOffCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingTakeOffCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Piloting.TakeOff
 * @param callback new callback for the command MiniDrone.Piloting.TakeOff
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingTakeOffCallback (ARCOMMANDS_Decoder_MiniDronePilotingTakeOffCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Piloting.PCMD
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingPCMDCallback_t) (uint8_t flag, int8_t roll, int8_t pitch, int8_t yaw, int8_t gaz, uint32_t timestamp, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Piloting.PCMD
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Piloting.PCMD
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingPCMDCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingPCMDCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Piloting.PCMD
 * @param callback new callback for the command MiniDrone.Piloting.PCMD
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingPCMDCallback (ARCOMMANDS_Decoder_MiniDronePilotingPCMDCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Piloting.Landing
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingLandingCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Piloting.Landing
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Piloting.Landing
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingLandingCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingLandingCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Piloting.Landing
 * @param callback new callback for the command MiniDrone.Piloting.Landing
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingLandingCallback (ARCOMMANDS_Decoder_MiniDronePilotingLandingCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Piloting.Emergency
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingEmergencyCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Piloting.Emergency
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Piloting.Emergency
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingEmergencyCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingEmergencyCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Piloting.Emergency
 * @param callback new callback for the command MiniDrone.Piloting.Emergency
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingEmergencyCallback (ARCOMMANDS_Decoder_MiniDronePilotingEmergencyCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Piloting.AutoTakeOffMode
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingAutoTakeOffModeCallback_t) (uint8_t state, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Piloting.AutoTakeOffMode
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Piloting.AutoTakeOffMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingAutoTakeOffModeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingAutoTakeOffModeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Piloting.AutoTakeOffMode
 * @param callback new callback for the command MiniDrone.Piloting.AutoTakeOffMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingAutoTakeOffModeCallback (ARCOMMANDS_Decoder_MiniDronePilotingAutoTakeOffModeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Piloting.FlyingMode
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingFlyingModeCallback_t) (eARCOMMANDS_MINIDRONE_PILOTING_FLYINGMODE_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Piloting.FlyingMode
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Piloting.FlyingMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingFlyingModeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingFlyingModeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Piloting.FlyingMode
 * @param callback new callback for the command MiniDrone.Piloting.FlyingMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingFlyingModeCallback (ARCOMMANDS_Decoder_MiniDronePilotingFlyingModeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Piloting.PlaneGearBox
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingPlaneGearBoxCallback_t) (eARCOMMANDS_MINIDRONE_PILOTING_PLANEGEARBOX_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Piloting.PlaneGearBox
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Piloting.PlaneGearBox
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingPlaneGearBoxCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingPlaneGearBoxCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Piloting.PlaneGearBox
 * @param callback new callback for the command MiniDrone.Piloting.PlaneGearBox
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingPlaneGearBoxCallback (ARCOMMANDS_Decoder_MiniDronePilotingPlaneGearBoxCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Piloting.TogglePilotingMode
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingTogglePilotingModeCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Piloting.TogglePilotingMode
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Piloting.TogglePilotingMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingTogglePilotingModeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingTogglePilotingModeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Piloting.TogglePilotingMode
 * @param callback new callback for the command MiniDrone.Piloting.TogglePilotingMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingTogglePilotingModeCallback (ARCOMMANDS_Decoder_MiniDronePilotingTogglePilotingModeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Animations.Flip
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneAnimationsFlipCallback_t) (eARCOMMANDS_MINIDRONE_ANIMATIONS_FLIP_DIRECTION direction, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Animations.Flip
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Animations.Flip
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneAnimationsFlipCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneAnimationsFlipCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Animations.Flip
 * @param callback new callback for the command MiniDrone.Animations.Flip
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneAnimationsFlipCallback (ARCOMMANDS_Decoder_MiniDroneAnimationsFlipCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Animations.Cap
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneAnimationsCapCallback_t) (int16_t offset, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Animations.Cap
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Animations.Cap
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneAnimationsCapCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneAnimationsCapCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Animations.Cap
 * @param callback new callback for the command MiniDrone.Animations.Cap
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneAnimationsCapCallback (ARCOMMANDS_Decoder_MiniDroneAnimationsCapCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.MediaRecord.Picture
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMediaRecordPictureCallback_t) (uint8_t mass_storage_id, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.MediaRecord.Picture
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.MediaRecord.Picture
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMediaRecordPictureCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMediaRecordPictureCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.MediaRecord.Picture
 * @param callback new callback for the command MiniDrone.MediaRecord.Picture
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMediaRecordPictureCallback (ARCOMMANDS_Decoder_MiniDroneMediaRecordPictureCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.MediaRecord.PictureV2
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMediaRecordPictureV2Callback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.MediaRecord.PictureV2
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.MediaRecord.PictureV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMediaRecordPictureV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMediaRecordPictureV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.MediaRecord.PictureV2
 * @param callback new callback for the command MiniDrone.MediaRecord.PictureV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMediaRecordPictureV2Callback (ARCOMMANDS_Decoder_MiniDroneMediaRecordPictureV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingSettings.MaxAltitude
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingSettingsMaxAltitudeCallback_t) (float current, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingSettings.MaxAltitude
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingSettings.MaxAltitude
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsMaxAltitudeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingSettingsMaxAltitudeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingSettings.MaxAltitude
 * @param callback new callback for the command MiniDrone.PilotingSettings.MaxAltitude
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsMaxAltitudeCallback (ARCOMMANDS_Decoder_MiniDronePilotingSettingsMaxAltitudeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingSettings.MaxTilt
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingSettingsMaxTiltCallback_t) (float current, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingSettings.MaxTilt
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingSettings.MaxTilt
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsMaxTiltCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingSettingsMaxTiltCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingSettings.MaxTilt
 * @param callback new callback for the command MiniDrone.PilotingSettings.MaxTilt
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsMaxTiltCallback (ARCOMMANDS_Decoder_MiniDronePilotingSettingsMaxTiltCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingSettings.BankedTurn
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingSettingsBankedTurnCallback_t) (uint8_t value, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingSettings.BankedTurn
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingSettings.BankedTurn
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsBankedTurnCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingSettingsBankedTurnCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingSettings.BankedTurn
 * @param callback new callback for the command MiniDrone.PilotingSettings.BankedTurn
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsBankedTurnCallback (ARCOMMANDS_Decoder_MiniDronePilotingSettingsBankedTurnCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingSettings.MaxThrottle
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingSettingsMaxThrottleCallback_t) (float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingSettings.MaxThrottle
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingSettings.MaxThrottle
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsMaxThrottleCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingSettingsMaxThrottleCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingSettings.MaxThrottle
 * @param callback new callback for the command MiniDrone.PilotingSettings.MaxThrottle
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsMaxThrottleCallback (ARCOMMANDS_Decoder_MiniDronePilotingSettingsMaxThrottleCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingSettings.PreferredPilotingMode
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingSettingsPreferredPilotingModeCallback_t) (eARCOMMANDS_MINIDRONE_PILOTINGSETTINGS_PREFERREDPILOTINGMODE_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingSettings.PreferredPilotingMode
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingSettings.PreferredPilotingMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsPreferredPilotingModeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingSettingsPreferredPilotingModeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingSettings.PreferredPilotingMode
 * @param callback new callback for the command MiniDrone.PilotingSettings.PreferredPilotingMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsPreferredPilotingModeCallback (ARCOMMANDS_Decoder_MiniDronePilotingSettingsPreferredPilotingModeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.SpeedSettings.MaxVerticalSpeed
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneSpeedSettingsMaxVerticalSpeedCallback_t) (float current, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.SpeedSettings.MaxVerticalSpeed
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.SpeedSettings.MaxVerticalSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsMaxVerticalSpeedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneSpeedSettingsMaxVerticalSpeedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.SpeedSettings.MaxVerticalSpeed
 * @param callback new callback for the command MiniDrone.SpeedSettings.MaxVerticalSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsMaxVerticalSpeedCallback (ARCOMMANDS_Decoder_MiniDroneSpeedSettingsMaxVerticalSpeedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.SpeedSettings.MaxRotationSpeed
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneSpeedSettingsMaxRotationSpeedCallback_t) (float current, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.SpeedSettings.MaxRotationSpeed
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.SpeedSettings.MaxRotationSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsMaxRotationSpeedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneSpeedSettingsMaxRotationSpeedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.SpeedSettings.MaxRotationSpeed
 * @param callback new callback for the command MiniDrone.SpeedSettings.MaxRotationSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsMaxRotationSpeedCallback (ARCOMMANDS_Decoder_MiniDroneSpeedSettingsMaxRotationSpeedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.SpeedSettings.Wheels
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneSpeedSettingsWheelsCallback_t) (uint8_t present, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.SpeedSettings.Wheels
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.SpeedSettings.Wheels
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsWheelsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneSpeedSettingsWheelsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.SpeedSettings.Wheels
 * @param callback new callback for the command MiniDrone.SpeedSettings.Wheels
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsWheelsCallback (ARCOMMANDS_Decoder_MiniDroneSpeedSettingsWheelsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.SpeedSettings.MaxHorizontalSpeed
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneSpeedSettingsMaxHorizontalSpeedCallback_t) (float current, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.SpeedSettings.MaxHorizontalSpeed
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.SpeedSettings.MaxHorizontalSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsMaxHorizontalSpeedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneSpeedSettingsMaxHorizontalSpeedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.SpeedSettings.MaxHorizontalSpeed
 * @param callback new callback for the command MiniDrone.SpeedSettings.MaxHorizontalSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsMaxHorizontalSpeedCallback (ARCOMMANDS_Decoder_MiniDroneSpeedSettingsMaxHorizontalSpeedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.SpeedSettings.MaxPlaneModeRotationSpeed
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneSpeedSettingsMaxPlaneModeRotationSpeedCallback_t) (float current, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.SpeedSettings.MaxPlaneModeRotationSpeed
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.SpeedSettings.MaxPlaneModeRotationSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsMaxPlaneModeRotationSpeedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneSpeedSettingsMaxPlaneModeRotationSpeedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.SpeedSettings.MaxPlaneModeRotationSpeed
 * @param callback new callback for the command MiniDrone.SpeedSettings.MaxPlaneModeRotationSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsMaxPlaneModeRotationSpeedCallback (ARCOMMANDS_Decoder_MiniDroneSpeedSettingsMaxPlaneModeRotationSpeedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Settings.CutOutMode
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneSettingsCutOutModeCallback_t) (uint8_t enable, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Settings.CutOutMode
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Settings.CutOutMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSettingsCutOutModeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneSettingsCutOutModeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Settings.CutOutMode
 * @param callback new callback for the command MiniDrone.Settings.CutOutMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSettingsCutOutModeCallback (ARCOMMANDS_Decoder_MiniDroneSettingsCutOutModeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.GPS.ControllerLatitudeForRun
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneGPSControllerLatitudeForRunCallback_t) (double latitude, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.GPS.ControllerLatitudeForRun
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.GPS.ControllerLatitudeForRun
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneGPSControllerLatitudeForRunCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneGPSControllerLatitudeForRunCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.GPS.ControllerLatitudeForRun
 * @param callback new callback for the command MiniDrone.GPS.ControllerLatitudeForRun
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneGPSControllerLatitudeForRunCallback (ARCOMMANDS_Decoder_MiniDroneGPSControllerLatitudeForRunCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.GPS.ControllerLongitudeForRun
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneGPSControllerLongitudeForRunCallback_t) (double longitude, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.GPS.ControllerLongitudeForRun
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.GPS.ControllerLongitudeForRun
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneGPSControllerLongitudeForRunCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneGPSControllerLongitudeForRunCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.GPS.ControllerLongitudeForRun
 * @param callback new callback for the command MiniDrone.GPS.ControllerLongitudeForRun
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneGPSControllerLongitudeForRunCallback (ARCOMMANDS_Decoder_MiniDroneGPSControllerLongitudeForRunCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Configuration.ControllerType
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneConfigurationControllerTypeCallback_t) (const char * type, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Configuration.ControllerType
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Configuration.ControllerType
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneConfigurationControllerTypeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneConfigurationControllerTypeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Configuration.ControllerType
 * @param callback new callback for the command MiniDrone.Configuration.ControllerType
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneConfigurationControllerTypeCallback (ARCOMMANDS_Decoder_MiniDroneConfigurationControllerTypeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Configuration.ControllerName
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneConfigurationControllerNameCallback_t) (const char * name, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Configuration.ControllerName
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Configuration.ControllerName
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneConfigurationControllerNameCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneConfigurationControllerNameCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Configuration.ControllerName
 * @param callback new callback for the command MiniDrone.Configuration.ControllerName
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneConfigurationControllerNameCallback (ARCOMMANDS_Decoder_MiniDroneConfigurationControllerNameCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.UsbAccessory.LightControl
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneUsbAccessoryLightControlCallback_t) (uint8_t id, eARCOMMANDS_MINIDRONE_USBACCESSORY_LIGHTCONTROL_MODE mode, uint8_t intensity, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.UsbAccessory.LightControl
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.UsbAccessory.LightControl
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneUsbAccessoryLightControlCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneUsbAccessoryLightControlCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.UsbAccessory.LightControl
 * @param callback new callback for the command MiniDrone.UsbAccessory.LightControl
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneUsbAccessoryLightControlCallback (ARCOMMANDS_Decoder_MiniDroneUsbAccessoryLightControlCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.UsbAccessory.ClawControl
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneUsbAccessoryClawControlCallback_t) (uint8_t id, eARCOMMANDS_MINIDRONE_USBACCESSORY_CLAWCONTROL_ACTION action, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.UsbAccessory.ClawControl
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.UsbAccessory.ClawControl
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneUsbAccessoryClawControlCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneUsbAccessoryClawControlCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.UsbAccessory.ClawControl
 * @param callback new callback for the command MiniDrone.UsbAccessory.ClawControl
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneUsbAccessoryClawControlCallback (ARCOMMANDS_Decoder_MiniDroneUsbAccessoryClawControlCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.UsbAccessory.GunControl
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneUsbAccessoryGunControlCallback_t) (uint8_t id, eARCOMMANDS_MINIDRONE_USBACCESSORY_GUNCONTROL_ACTION action, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.UsbAccessory.GunControl
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.UsbAccessory.GunControl
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneUsbAccessoryGunControlCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneUsbAccessoryGunControlCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.UsbAccessory.GunControl
 * @param callback new callback for the command MiniDrone.UsbAccessory.GunControl
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneUsbAccessoryGunControlCallback (ARCOMMANDS_Decoder_MiniDroneUsbAccessoryGunControlCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.RemoteController.SetPairedRemote
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneRemoteControllerSetPairedRemoteCallback_t) (uint16_t msb_mac, uint16_t mid_mac, uint16_t lsb_mac, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.RemoteController.SetPairedRemote
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.RemoteController.SetPairedRemote
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneRemoteControllerSetPairedRemoteCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneRemoteControllerSetPairedRemoteCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.RemoteController.SetPairedRemote
 * @param callback new callback for the command MiniDrone.RemoteController.SetPairedRemote
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneRemoteControllerSetPairedRemoteCallback (ARCOMMANDS_Decoder_MiniDroneRemoteControllerSetPairedRemoteCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.VideoSettings.Autorecord
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneVideoSettingsAutorecordCallback_t) (uint8_t enable, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.VideoSettings.Autorecord
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.VideoSettings.Autorecord
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneVideoSettingsAutorecordCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneVideoSettingsAutorecordCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.VideoSettings.Autorecord
 * @param callback new callback for the command MiniDrone.VideoSettings.Autorecord
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneVideoSettingsAutorecordCallback (ARCOMMANDS_Decoder_MiniDroneVideoSettingsAutorecordCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.VideoSettings.ElectricFrequency
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneVideoSettingsElectricFrequencyCallback_t) (eARCOMMANDS_MINIDRONE_VIDEOSETTINGS_ELECTRICFREQUENCY_FREQUENCY frequency, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.VideoSettings.ElectricFrequency
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.VideoSettings.ElectricFrequency
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneVideoSettingsElectricFrequencyCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneVideoSettingsElectricFrequencyCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.VideoSettings.ElectricFrequency
 * @param callback new callback for the command MiniDrone.VideoSettings.ElectricFrequency
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneVideoSettingsElectricFrequencyCallback (ARCOMMANDS_Decoder_MiniDroneVideoSettingsElectricFrequencyCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.VideoSettings.VideoResolution
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneVideoSettingsVideoResolutionCallback_t) (eARCOMMANDS_MINIDRONE_VIDEOSETTINGS_VIDEORESOLUTION_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.VideoSettings.VideoResolution
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.VideoSettings.VideoResolution
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneVideoSettingsVideoResolutionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneVideoSettingsVideoResolutionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.VideoSettings.VideoResolution
 * @param callback new callback for the command MiniDrone.VideoSettings.VideoResolution
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneVideoSettingsVideoResolutionCallback (ARCOMMANDS_Decoder_MiniDroneVideoSettingsVideoResolutionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Minicam.Picture
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMinicamPictureCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Minicam.Picture
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Minicam.Picture
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamPictureCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMinicamPictureCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Minicam.Picture
 * @param callback new callback for the command MiniDrone.Minicam.Picture
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamPictureCallback (ARCOMMANDS_Decoder_MiniDroneMinicamPictureCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Minicam.Video
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMinicamVideoCallback_t) (eARCOMMANDS_MINIDRONE_MINICAM_VIDEO_RECORD record, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Minicam.Video
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Minicam.Video
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamVideoCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMinicamVideoCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Minicam.Video
 * @param callback new callback for the command MiniDrone.Minicam.Video
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamVideoCallback (ARCOMMANDS_Decoder_MiniDroneMinicamVideoCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.Minicam.MassStorageFormat
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMinicamMassStorageFormatCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.Minicam.MassStorageFormat
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.Minicam.MassStorageFormat
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamMassStorageFormatCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMinicamMassStorageFormatCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.Minicam.MassStorageFormat
 * @param callback new callback for the command MiniDrone.Minicam.MassStorageFormat
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamMassStorageFormatCallback (ARCOMMANDS_Decoder_MiniDroneMinicamMassStorageFormatCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingState.FlatTrimChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingStateFlatTrimChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingState.FlatTrimChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingState.FlatTrimChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingStateFlatTrimChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingStateFlatTrimChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingState.FlatTrimChanged
 * @param callback new callback for the command MiniDrone.PilotingState.FlatTrimChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingStateFlatTrimChangedCallback (ARCOMMANDS_Decoder_MiniDronePilotingStateFlatTrimChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingState.FlyingStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingStateFlyingStateChangedCallback_t) (eARCOMMANDS_MINIDRONE_PILOTINGSTATE_FLYINGSTATECHANGED_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingState.FlyingStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingState.FlyingStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingStateFlyingStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingStateFlyingStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingState.FlyingStateChanged
 * @param callback new callback for the command MiniDrone.PilotingState.FlyingStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingStateFlyingStateChangedCallback (ARCOMMANDS_Decoder_MiniDronePilotingStateFlyingStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingState.AlertStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingStateAlertStateChangedCallback_t) (eARCOMMANDS_MINIDRONE_PILOTINGSTATE_ALERTSTATECHANGED_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingState.AlertStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingState.AlertStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingStateAlertStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingStateAlertStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingState.AlertStateChanged
 * @param callback new callback for the command MiniDrone.PilotingState.AlertStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingStateAlertStateChangedCallback (ARCOMMANDS_Decoder_MiniDronePilotingStateAlertStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingState.AutoTakeOffModeChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingStateAutoTakeOffModeChangedCallback_t) (uint8_t state, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingState.AutoTakeOffModeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingState.AutoTakeOffModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingStateAutoTakeOffModeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingStateAutoTakeOffModeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingState.AutoTakeOffModeChanged
 * @param callback new callback for the command MiniDrone.PilotingState.AutoTakeOffModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingStateAutoTakeOffModeChangedCallback (ARCOMMANDS_Decoder_MiniDronePilotingStateAutoTakeOffModeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingState.FlyingModeChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingStateFlyingModeChangedCallback_t) (eARCOMMANDS_MINIDRONE_PILOTINGSTATE_FLYINGMODECHANGED_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingState.FlyingModeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingState.FlyingModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingStateFlyingModeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingStateFlyingModeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingState.FlyingModeChanged
 * @param callback new callback for the command MiniDrone.PilotingState.FlyingModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingStateFlyingModeChangedCallback (ARCOMMANDS_Decoder_MiniDronePilotingStateFlyingModeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingState.PlaneGearBoxChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingStatePlaneGearBoxChangedCallback_t) (eARCOMMANDS_MINIDRONE_PILOTINGSTATE_PLANEGEARBOXCHANGED_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingState.PlaneGearBoxChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingState.PlaneGearBoxChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingStatePlaneGearBoxChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingStatePlaneGearBoxChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingState.PlaneGearBoxChanged
 * @param callback new callback for the command MiniDrone.PilotingState.PlaneGearBoxChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingStatePlaneGearBoxChangedCallback (ARCOMMANDS_Decoder_MiniDronePilotingStatePlaneGearBoxChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingState.PilotingModeChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingStatePilotingModeChangedCallback_t) (eARCOMMANDS_MINIDRONE_PILOTINGSTATE_PILOTINGMODECHANGED_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingState.PilotingModeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingState.PilotingModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingStatePilotingModeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingStatePilotingModeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingState.PilotingModeChanged
 * @param callback new callback for the command MiniDrone.PilotingState.PilotingModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingStatePilotingModeChangedCallback (ARCOMMANDS_Decoder_MiniDronePilotingStatePilotingModeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.MediaRecordState.PictureStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMediaRecordStatePictureStateChangedCallback_t) (uint8_t state, uint8_t mass_storage_id, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.MediaRecordState.PictureStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.MediaRecordState.PictureStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMediaRecordStatePictureStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMediaRecordStatePictureStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.MediaRecordState.PictureStateChanged
 * @param callback new callback for the command MiniDrone.MediaRecordState.PictureStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMediaRecordStatePictureStateChangedCallback (ARCOMMANDS_Decoder_MiniDroneMediaRecordStatePictureStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.MediaRecordState.PictureStateChangedV2
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMediaRecordStatePictureStateChangedV2Callback_t) (eARCOMMANDS_MINIDRONE_MEDIARECORDSTATE_PICTURESTATECHANGEDV2_STATE state, eARCOMMANDS_MINIDRONE_MEDIARECORDSTATE_PICTURESTATECHANGEDV2_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.MediaRecordState.PictureStateChangedV2
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.MediaRecordState.PictureStateChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMediaRecordStatePictureStateChangedV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMediaRecordStatePictureStateChangedV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.MediaRecordState.PictureStateChangedV2
 * @param callback new callback for the command MiniDrone.MediaRecordState.PictureStateChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMediaRecordStatePictureStateChangedV2Callback (ARCOMMANDS_Decoder_MiniDroneMediaRecordStatePictureStateChangedV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.MediaRecordEvent.PictureEventChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMediaRecordEventPictureEventChangedCallback_t) (eARCOMMANDS_MINIDRONE_MEDIARECORDEVENT_PICTUREEVENTCHANGED_EVENT event, eARCOMMANDS_MINIDRONE_MEDIARECORDEVENT_PICTUREEVENTCHANGED_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.MediaRecordEvent.PictureEventChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.MediaRecordEvent.PictureEventChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMediaRecordEventPictureEventChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMediaRecordEventPictureEventChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.MediaRecordEvent.PictureEventChanged
 * @param callback new callback for the command MiniDrone.MediaRecordEvent.PictureEventChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMediaRecordEventPictureEventChangedCallback (ARCOMMANDS_Decoder_MiniDroneMediaRecordEventPictureEventChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingSettingsState.MaxAltitudeChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingSettingsStateMaxAltitudeChangedCallback_t) (float current, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingSettingsState.MaxAltitudeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingSettingsState.MaxAltitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsStateMaxAltitudeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingSettingsStateMaxAltitudeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingSettingsState.MaxAltitudeChanged
 * @param callback new callback for the command MiniDrone.PilotingSettingsState.MaxAltitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsStateMaxAltitudeChangedCallback (ARCOMMANDS_Decoder_MiniDronePilotingSettingsStateMaxAltitudeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingSettingsState.MaxTiltChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingSettingsStateMaxTiltChangedCallback_t) (float current, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingSettingsState.MaxTiltChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingSettingsState.MaxTiltChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsStateMaxTiltChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingSettingsStateMaxTiltChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingSettingsState.MaxTiltChanged
 * @param callback new callback for the command MiniDrone.PilotingSettingsState.MaxTiltChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsStateMaxTiltChangedCallback (ARCOMMANDS_Decoder_MiniDronePilotingSettingsStateMaxTiltChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingSettingsState.BankedTurnChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingSettingsStateBankedTurnChangedCallback_t) (uint8_t state, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingSettingsState.BankedTurnChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingSettingsState.BankedTurnChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsStateBankedTurnChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingSettingsStateBankedTurnChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingSettingsState.BankedTurnChanged
 * @param callback new callback for the command MiniDrone.PilotingSettingsState.BankedTurnChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsStateBankedTurnChangedCallback (ARCOMMANDS_Decoder_MiniDronePilotingSettingsStateBankedTurnChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingSettingsState.MaxThrottleChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingSettingsStateMaxThrottleChangedCallback_t) (float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingSettingsState.MaxThrottleChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingSettingsState.MaxThrottleChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsStateMaxThrottleChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingSettingsStateMaxThrottleChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingSettingsState.MaxThrottleChanged
 * @param callback new callback for the command MiniDrone.PilotingSettingsState.MaxThrottleChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsStateMaxThrottleChangedCallback (ARCOMMANDS_Decoder_MiniDronePilotingSettingsStateMaxThrottleChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.PilotingSettingsState.PreferredPilotingModeChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDronePilotingSettingsStatePreferredPilotingModeChangedCallback_t) (eARCOMMANDS_MINIDRONE_PILOTINGSETTINGSSTATE_PREFERREDPILOTINGMODECHANGED_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.PilotingSettingsState.PreferredPilotingModeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.PilotingSettingsState.PreferredPilotingModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsStatePreferredPilotingModeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDronePilotingSettingsStatePreferredPilotingModeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.PilotingSettingsState.PreferredPilotingModeChanged
 * @param callback new callback for the command MiniDrone.PilotingSettingsState.PreferredPilotingModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDronePilotingSettingsStatePreferredPilotingModeChangedCallback (ARCOMMANDS_Decoder_MiniDronePilotingSettingsStatePreferredPilotingModeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.SpeedSettingsState.MaxVerticalSpeedChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateMaxVerticalSpeedChangedCallback_t) (float current, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.SpeedSettingsState.MaxVerticalSpeedChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.SpeedSettingsState.MaxVerticalSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsStateMaxVerticalSpeedChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateMaxVerticalSpeedChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.SpeedSettingsState.MaxVerticalSpeedChanged
 * @param callback new callback for the command MiniDrone.SpeedSettingsState.MaxVerticalSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsStateMaxVerticalSpeedChangedCallback (ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateMaxVerticalSpeedChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.SpeedSettingsState.MaxRotationSpeedChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateMaxRotationSpeedChangedCallback_t) (float current, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.SpeedSettingsState.MaxRotationSpeedChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.SpeedSettingsState.MaxRotationSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsStateMaxRotationSpeedChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateMaxRotationSpeedChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.SpeedSettingsState.MaxRotationSpeedChanged
 * @param callback new callback for the command MiniDrone.SpeedSettingsState.MaxRotationSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsStateMaxRotationSpeedChangedCallback (ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateMaxRotationSpeedChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.SpeedSettingsState.WheelsChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateWheelsChangedCallback_t) (uint8_t present, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.SpeedSettingsState.WheelsChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.SpeedSettingsState.WheelsChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsStateWheelsChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateWheelsChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.SpeedSettingsState.WheelsChanged
 * @param callback new callback for the command MiniDrone.SpeedSettingsState.WheelsChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsStateWheelsChangedCallback (ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateWheelsChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.SpeedSettingsState.MaxHorizontalSpeedChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateMaxHorizontalSpeedChangedCallback_t) (float current, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.SpeedSettingsState.MaxHorizontalSpeedChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.SpeedSettingsState.MaxHorizontalSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsStateMaxHorizontalSpeedChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateMaxHorizontalSpeedChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.SpeedSettingsState.MaxHorizontalSpeedChanged
 * @param callback new callback for the command MiniDrone.SpeedSettingsState.MaxHorizontalSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsStateMaxHorizontalSpeedChangedCallback (ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateMaxHorizontalSpeedChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.SpeedSettingsState.MaxPlaneModeRotationSpeedChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateMaxPlaneModeRotationSpeedChangedCallback_t) (float current, float min, float max, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.SpeedSettingsState.MaxPlaneModeRotationSpeedChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.SpeedSettingsState.MaxPlaneModeRotationSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsStateMaxPlaneModeRotationSpeedChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateMaxPlaneModeRotationSpeedChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.SpeedSettingsState.MaxPlaneModeRotationSpeedChanged
 * @param callback new callback for the command MiniDrone.SpeedSettingsState.MaxPlaneModeRotationSpeedChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSpeedSettingsStateMaxPlaneModeRotationSpeedChangedCallback (ARCOMMANDS_Decoder_MiniDroneSpeedSettingsStateMaxPlaneModeRotationSpeedChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.SettingsState.ProductMotorsVersionChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneSettingsStateProductMotorsVersionChangedCallback_t) (uint8_t motor, const char * type, const char * software, const char * hardware, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.SettingsState.ProductMotorsVersionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.SettingsState.ProductMotorsVersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSettingsStateProductMotorsVersionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneSettingsStateProductMotorsVersionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.SettingsState.ProductMotorsVersionChanged
 * @param callback new callback for the command MiniDrone.SettingsState.ProductMotorsVersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSettingsStateProductMotorsVersionChangedCallback (ARCOMMANDS_Decoder_MiniDroneSettingsStateProductMotorsVersionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.SettingsState.ProductInertialVersionChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneSettingsStateProductInertialVersionChangedCallback_t) (const char * software, const char * hardware, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.SettingsState.ProductInertialVersionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.SettingsState.ProductInertialVersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSettingsStateProductInertialVersionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneSettingsStateProductInertialVersionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.SettingsState.ProductInertialVersionChanged
 * @param callback new callback for the command MiniDrone.SettingsState.ProductInertialVersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSettingsStateProductInertialVersionChangedCallback (ARCOMMANDS_Decoder_MiniDroneSettingsStateProductInertialVersionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.SettingsState.CutOutModeChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneSettingsStateCutOutModeChangedCallback_t) (uint8_t enable, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.SettingsState.CutOutModeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.SettingsState.CutOutModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSettingsStateCutOutModeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneSettingsStateCutOutModeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.SettingsState.CutOutModeChanged
 * @param callback new callback for the command MiniDrone.SettingsState.CutOutModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneSettingsStateCutOutModeChangedCallback (ARCOMMANDS_Decoder_MiniDroneSettingsStateCutOutModeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.FloodControlState.FloodControlChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneFloodControlStateFloodControlChangedCallback_t) (uint16_t delay, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.FloodControlState.FloodControlChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.FloodControlState.FloodControlChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneFloodControlStateFloodControlChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneFloodControlStateFloodControlChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.FloodControlState.FloodControlChanged
 * @param callback new callback for the command MiniDrone.FloodControlState.FloodControlChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneFloodControlStateFloodControlChangedCallback (ARCOMMANDS_Decoder_MiniDroneFloodControlStateFloodControlChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.UsbAccessoryState.LightState
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneUsbAccessoryStateLightStateCallback_t) (uint8_t id, eARCOMMANDS_MINIDRONE_USBACCESSORYSTATE_LIGHTSTATE_STATE state, uint8_t intensity, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.UsbAccessoryState.LightState
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.UsbAccessoryState.LightState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneUsbAccessoryStateLightStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneUsbAccessoryStateLightStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.UsbAccessoryState.LightState
 * @param callback new callback for the command MiniDrone.UsbAccessoryState.LightState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneUsbAccessoryStateLightStateCallback (ARCOMMANDS_Decoder_MiniDroneUsbAccessoryStateLightStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.UsbAccessoryState.ClawState
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneUsbAccessoryStateClawStateCallback_t) (uint8_t id, eARCOMMANDS_MINIDRONE_USBACCESSORYSTATE_CLAWSTATE_STATE state, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.UsbAccessoryState.ClawState
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.UsbAccessoryState.ClawState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneUsbAccessoryStateClawStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneUsbAccessoryStateClawStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.UsbAccessoryState.ClawState
 * @param callback new callback for the command MiniDrone.UsbAccessoryState.ClawState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneUsbAccessoryStateClawStateCallback (ARCOMMANDS_Decoder_MiniDroneUsbAccessoryStateClawStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.UsbAccessoryState.GunState
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneUsbAccessoryStateGunStateCallback_t) (uint8_t id, eARCOMMANDS_MINIDRONE_USBACCESSORYSTATE_GUNSTATE_STATE state, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.UsbAccessoryState.GunState
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.UsbAccessoryState.GunState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneUsbAccessoryStateGunStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneUsbAccessoryStateGunStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.UsbAccessoryState.GunState
 * @param callback new callback for the command MiniDrone.UsbAccessoryState.GunState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneUsbAccessoryStateGunStateCallback (ARCOMMANDS_Decoder_MiniDroneUsbAccessoryStateGunStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.NavigationDataState.DronePosition
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneNavigationDataStateDronePositionCallback_t) (float posx, float posy, int16_t posz, int16_t psi, int16_t ts, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.NavigationDataState.DronePosition
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.NavigationDataState.DronePosition
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneNavigationDataStateDronePositionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneNavigationDataStateDronePositionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.NavigationDataState.DronePosition
 * @param callback new callback for the command MiniDrone.NavigationDataState.DronePosition
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneNavigationDataStateDronePositionCallback (ARCOMMANDS_Decoder_MiniDroneNavigationDataStateDronePositionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.NavigationDataState.DroneSpeed
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneNavigationDataStateDroneSpeedCallback_t) (float speed_x, float speed_y, float speed_z, uint16_t ts, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.NavigationDataState.DroneSpeed
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.NavigationDataState.DroneSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneNavigationDataStateDroneSpeedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneNavigationDataStateDroneSpeedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.NavigationDataState.DroneSpeed
 * @param callback new callback for the command MiniDrone.NavigationDataState.DroneSpeed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneNavigationDataStateDroneSpeedCallback (ARCOMMANDS_Decoder_MiniDroneNavigationDataStateDroneSpeedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.NavigationDataState.DroneAltitude
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneNavigationDataStateDroneAltitudeCallback_t) (float altitude, uint16_t ts, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.NavigationDataState.DroneAltitude
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.NavigationDataState.DroneAltitude
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneNavigationDataStateDroneAltitudeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneNavigationDataStateDroneAltitudeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.NavigationDataState.DroneAltitude
 * @param callback new callback for the command MiniDrone.NavigationDataState.DroneAltitude
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneNavigationDataStateDroneAltitudeCallback (ARCOMMANDS_Decoder_MiniDroneNavigationDataStateDroneAltitudeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.NavigationDataState.DroneQuaternion
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneNavigationDataStateDroneQuaternionCallback_t) (float q_w, float q_x, float q_y, float q_z, uint16_t ts, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.NavigationDataState.DroneQuaternion
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.NavigationDataState.DroneQuaternion
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneNavigationDataStateDroneQuaternionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneNavigationDataStateDroneQuaternionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.NavigationDataState.DroneQuaternion
 * @param callback new callback for the command MiniDrone.NavigationDataState.DroneQuaternion
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneNavigationDataStateDroneQuaternionCallback (ARCOMMANDS_Decoder_MiniDroneNavigationDataStateDroneQuaternionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.MinicamState.PowerModeChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMinicamStatePowerModeChangedCallback_t) (eARCOMMANDS_MINIDRONE_MINICAMSTATE_POWERMODECHANGED_POWER_MODE power_mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.MinicamState.PowerModeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.MinicamState.PowerModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamStatePowerModeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMinicamStatePowerModeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.MinicamState.PowerModeChanged
 * @param callback new callback for the command MiniDrone.MinicamState.PowerModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamStatePowerModeChangedCallback (ARCOMMANDS_Decoder_MiniDroneMinicamStatePowerModeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.MinicamState.ProductSerialChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMinicamStateProductSerialChangedCallback_t) (const char * serial_number, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.MinicamState.ProductSerialChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.MinicamState.ProductSerialChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamStateProductSerialChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMinicamStateProductSerialChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.MinicamState.ProductSerialChanged
 * @param callback new callback for the command MiniDrone.MinicamState.ProductSerialChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamStateProductSerialChangedCallback (ARCOMMANDS_Decoder_MiniDroneMinicamStateProductSerialChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.MinicamState.StateChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMinicamStateStateChangedCallback_t) (eARCOMMANDS_MINIDRONE_MINICAMSTATE_STATECHANGED_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.MinicamState.StateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.MinicamState.StateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamStateStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMinicamStateStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.MinicamState.StateChanged
 * @param callback new callback for the command MiniDrone.MinicamState.StateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamStateStateChangedCallback (ARCOMMANDS_Decoder_MiniDroneMinicamStateStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.MinicamState.VersionChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMinicamStateVersionChangedCallback_t) (const char * software, const char * hardware, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.MinicamState.VersionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.MinicamState.VersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamStateVersionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMinicamStateVersionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.MinicamState.VersionChanged
 * @param callback new callback for the command MiniDrone.MinicamState.VersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamStateVersionChangedCallback (ARCOMMANDS_Decoder_MiniDroneMinicamStateVersionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.MinicamState.PictureChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMinicamStatePictureChangedCallback_t) (eARCOMMANDS_MINIDRONE_MINICAMSTATE_PICTURECHANGED_STATE state, eARCOMMANDS_MINIDRONE_MINICAMSTATE_PICTURECHANGED_RESULT result, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.MinicamState.PictureChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.MinicamState.PictureChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamStatePictureChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMinicamStatePictureChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.MinicamState.PictureChanged
 * @param callback new callback for the command MiniDrone.MinicamState.PictureChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamStatePictureChangedCallback (ARCOMMANDS_Decoder_MiniDroneMinicamStatePictureChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.MinicamState.VideoStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMinicamStateVideoStateChangedCallback_t) (eARCOMMANDS_MINIDRONE_MINICAMSTATE_VIDEOSTATECHANGED_STATE state, eARCOMMANDS_MINIDRONE_MINICAMSTATE_VIDEOSTATECHANGED_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.MinicamState.VideoStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.MinicamState.VideoStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamStateVideoStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMinicamStateVideoStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.MinicamState.VideoStateChanged
 * @param callback new callback for the command MiniDrone.MinicamState.VideoStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamStateVideoStateChangedCallback (ARCOMMANDS_Decoder_MiniDroneMinicamStateVideoStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.MinicamState.MassStorageFormatChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneMinicamStateMassStorageFormatChangedCallback_t) (uint8_t state, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.MinicamState.MassStorageFormatChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.MinicamState.MassStorageFormatChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamStateMassStorageFormatChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneMinicamStateMassStorageFormatChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.MinicamState.MassStorageFormatChanged
 * @param callback new callback for the command MiniDrone.MinicamState.MassStorageFormatChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneMinicamStateMassStorageFormatChangedCallback (ARCOMMANDS_Decoder_MiniDroneMinicamStateMassStorageFormatChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.VideoSettingsState.AutorecordChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneVideoSettingsStateAutorecordChangedCallback_t) (uint8_t enabled, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.VideoSettingsState.AutorecordChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.VideoSettingsState.AutorecordChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneVideoSettingsStateAutorecordChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneVideoSettingsStateAutorecordChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.VideoSettingsState.AutorecordChanged
 * @param callback new callback for the command MiniDrone.VideoSettingsState.AutorecordChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneVideoSettingsStateAutorecordChangedCallback (ARCOMMANDS_Decoder_MiniDroneVideoSettingsStateAutorecordChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.VideoSettingsState.ElectricFrequencyChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneVideoSettingsStateElectricFrequencyChangedCallback_t) (eARCOMMANDS_MINIDRONE_VIDEOSETTINGSSTATE_ELECTRICFREQUENCYCHANGED_FREQUENCY frequency, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.VideoSettingsState.ElectricFrequencyChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.VideoSettingsState.ElectricFrequencyChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneVideoSettingsStateElectricFrequencyChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneVideoSettingsStateElectricFrequencyChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.VideoSettingsState.ElectricFrequencyChanged
 * @param callback new callback for the command MiniDrone.VideoSettingsState.ElectricFrequencyChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneVideoSettingsStateElectricFrequencyChangedCallback (ARCOMMANDS_Decoder_MiniDroneVideoSettingsStateElectricFrequencyChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.VideoSettingsState.VideoResolutionChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneVideoSettingsStateVideoResolutionChangedCallback_t) (eARCOMMANDS_MINIDRONE_VIDEOSETTINGSSTATE_VIDEORESOLUTIONCHANGED_TYPE type, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.VideoSettingsState.VideoResolutionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.VideoSettingsState.VideoResolutionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneVideoSettingsStateVideoResolutionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneVideoSettingsStateVideoResolutionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.VideoSettingsState.VideoResolutionChanged
 * @param callback new callback for the command MiniDrone.VideoSettingsState.VideoResolutionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneVideoSettingsStateVideoResolutionChangedCallback (ARCOMMANDS_Decoder_MiniDroneVideoSettingsStateVideoResolutionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command MiniDrone.RemoteControllerState.ConnectionChanged
 */
typedef void (*ARCOMMANDS_Decoder_MiniDroneRemoteControllerStateConnectionChangedCallback_t) (uint8_t state, void *custom);
/**
 * @brief Set Decoder callback setter for the command MiniDrone.RemoteControllerState.ConnectionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command MiniDrone.RemoteControllerState.ConnectionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneRemoteControllerStateConnectionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_MiniDroneRemoteControllerStateConnectionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command MiniDrone.RemoteControllerState.ConnectionChanged
 * @param callback new callback for the command MiniDrone.RemoteControllerState.ConnectionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetMiniDroneRemoteControllerStateConnectionChangedCallback (ARCOMMANDS_Decoder_MiniDroneRemoteControllerStateConnectionChangedCallback_t callback, void *custom) DEPRECATED;

// Feature powerup


/**
 * @brief callback type for the command powerup.Piloting.PCMD
 */
typedef void (*ARCOMMANDS_Decoder_PowerupPilotingPCMDCallback_t) (uint8_t flag, uint8_t throttle, int8_t roll, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.Piloting.PCMD
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.Piloting.PCMD
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingPCMDCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupPilotingPCMDCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.Piloting.PCMD
 * @param callback new callback for the command powerup.Piloting.PCMD
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingPCMDCallback (ARCOMMANDS_Decoder_PowerupPilotingPCMDCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.Piloting.UserTakeOff
 */
typedef void (*ARCOMMANDS_Decoder_PowerupPilotingUserTakeOffCallback_t) (uint8_t state, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.Piloting.UserTakeOff
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.Piloting.UserTakeOff
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingUserTakeOffCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupPilotingUserTakeOffCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.Piloting.UserTakeOff
 * @param callback new callback for the command powerup.Piloting.UserTakeOff
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingUserTakeOffCallback (ARCOMMANDS_Decoder_PowerupPilotingUserTakeOffCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.Piloting.MotorMode
 */
typedef void (*ARCOMMANDS_Decoder_PowerupPilotingMotorModeCallback_t) (eARCOMMANDS_POWERUP_PILOTING_MOTORMODE_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.Piloting.MotorMode
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.Piloting.MotorMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingMotorModeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupPilotingMotorModeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.Piloting.MotorMode
 * @param callback new callback for the command powerup.Piloting.MotorMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingMotorModeCallback (ARCOMMANDS_Decoder_PowerupPilotingMotorModeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.PilotingSettings.set
 */
typedef void (*ARCOMMANDS_Decoder_PowerupPilotingSettingsSetCallback_t) (eARCOMMANDS_POWERUP_PILOTINGSETTINGS_SET_SETTING setting, float value, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.PilotingSettings.set
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.PilotingSettings.set
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingSettingsSetCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupPilotingSettingsSetCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.PilotingSettings.set
 * @param callback new callback for the command powerup.PilotingSettings.set
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingSettingsSetCallback (ARCOMMANDS_Decoder_PowerupPilotingSettingsSetCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.MediaRecord.PictureV2
 */
typedef void (*ARCOMMANDS_Decoder_PowerupMediaRecordPictureV2Callback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.MediaRecord.PictureV2
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.MediaRecord.PictureV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaRecordPictureV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupMediaRecordPictureV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.MediaRecord.PictureV2
 * @param callback new callback for the command powerup.MediaRecord.PictureV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaRecordPictureV2Callback (ARCOMMANDS_Decoder_PowerupMediaRecordPictureV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.MediaRecord.VideoV2
 */
typedef void (*ARCOMMANDS_Decoder_PowerupMediaRecordVideoV2Callback_t) (eARCOMMANDS_POWERUP_MEDIARECORD_VIDEOV2_RECORD record, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.MediaRecord.VideoV2
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.MediaRecord.VideoV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaRecordVideoV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupMediaRecordVideoV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.MediaRecord.VideoV2
 * @param callback new callback for the command powerup.MediaRecord.VideoV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaRecordVideoV2Callback (ARCOMMANDS_Decoder_PowerupMediaRecordVideoV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.NetworkSettings.WifiSelection
 */
typedef void (*ARCOMMANDS_Decoder_PowerupNetworkSettingsWifiSelectionCallback_t) (eARCOMMANDS_POWERUP_NETWORKSETTINGS_WIFISELECTION_TYPE type, eARCOMMANDS_POWERUP_NETWORKSETTINGS_WIFISELECTION_BAND band, uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.NetworkSettings.WifiSelection
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.NetworkSettings.WifiSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkSettingsWifiSelectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupNetworkSettingsWifiSelectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.NetworkSettings.WifiSelection
 * @param callback new callback for the command powerup.NetworkSettings.WifiSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkSettingsWifiSelectionCallback (ARCOMMANDS_Decoder_PowerupNetworkSettingsWifiSelectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.Network.WifiScan
 */
typedef void (*ARCOMMANDS_Decoder_PowerupNetworkWifiScanCallback_t) (eARCOMMANDS_POWERUP_NETWORK_WIFISCAN_BAND band, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.Network.WifiScan
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.Network.WifiScan
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkWifiScanCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupNetworkWifiScanCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.Network.WifiScan
 * @param callback new callback for the command powerup.Network.WifiScan
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkWifiScanCallback (ARCOMMANDS_Decoder_PowerupNetworkWifiScanCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.Network.WifiAuthChannel
 */
typedef void (*ARCOMMANDS_Decoder_PowerupNetworkWifiAuthChannelCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.Network.WifiAuthChannel
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.Network.WifiAuthChannel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkWifiAuthChannelCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupNetworkWifiAuthChannelCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.Network.WifiAuthChannel
 * @param callback new callback for the command powerup.Network.WifiAuthChannel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkWifiAuthChannelCallback (ARCOMMANDS_Decoder_PowerupNetworkWifiAuthChannelCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.MediaStreaming.VideoEnable
 */
typedef void (*ARCOMMANDS_Decoder_PowerupMediaStreamingVideoEnableCallback_t) (uint8_t enable, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.MediaStreaming.VideoEnable
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.MediaStreaming.VideoEnable
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaStreamingVideoEnableCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupMediaStreamingVideoEnableCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.MediaStreaming.VideoEnable
 * @param callback new callback for the command powerup.MediaStreaming.VideoEnable
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaStreamingVideoEnableCallback (ARCOMMANDS_Decoder_PowerupMediaStreamingVideoEnableCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.VideoSettings.Autorecord
 */
typedef void (*ARCOMMANDS_Decoder_PowerupVideoSettingsAutorecordCallback_t) (uint8_t enable, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.VideoSettings.Autorecord
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.VideoSettings.Autorecord
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupVideoSettingsAutorecordCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupVideoSettingsAutorecordCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.VideoSettings.Autorecord
 * @param callback new callback for the command powerup.VideoSettings.Autorecord
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupVideoSettingsAutorecordCallback (ARCOMMANDS_Decoder_PowerupVideoSettingsAutorecordCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.VideoSettings.VideoMode
 */
typedef void (*ARCOMMANDS_Decoder_PowerupVideoSettingsVideoModeCallback_t) (eARCOMMANDS_POWERUP_VIDEOSETTINGS_VIDEOMODE_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.VideoSettings.VideoMode
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.VideoSettings.VideoMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupVideoSettingsVideoModeCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupVideoSettingsVideoModeCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.VideoSettings.VideoMode
 * @param callback new callback for the command powerup.VideoSettings.VideoMode
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupVideoSettingsVideoModeCallback (ARCOMMANDS_Decoder_PowerupVideoSettingsVideoModeCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.Sounds.buzz
 */
typedef void (*ARCOMMANDS_Decoder_PowerupSoundsBuzzCallback_t) (uint8_t enable, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.Sounds.buzz
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.Sounds.buzz
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupSoundsBuzzCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupSoundsBuzzCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.Sounds.buzz
 * @param callback new callback for the command powerup.Sounds.buzz
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupSoundsBuzzCallback (ARCOMMANDS_Decoder_PowerupSoundsBuzzCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.PilotingState.AlertStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupPilotingStateAlertStateChangedCallback_t) (eARCOMMANDS_POWERUP_PILOTINGSTATE_ALERTSTATECHANGED_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.PilotingState.AlertStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.PilotingState.AlertStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingStateAlertStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupPilotingStateAlertStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.PilotingState.AlertStateChanged
 * @param callback new callback for the command powerup.PilotingState.AlertStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingStateAlertStateChangedCallback (ARCOMMANDS_Decoder_PowerupPilotingStateAlertStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.PilotingState.FlyingStateChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupPilotingStateFlyingStateChangedCallback_t) (eARCOMMANDS_POWERUP_PILOTINGSTATE_FLYINGSTATECHANGED_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.PilotingState.FlyingStateChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.PilotingState.FlyingStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingStateFlyingStateChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupPilotingStateFlyingStateChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.PilotingState.FlyingStateChanged
 * @param callback new callback for the command powerup.PilotingState.FlyingStateChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingStateFlyingStateChangedCallback (ARCOMMANDS_Decoder_PowerupPilotingStateFlyingStateChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.PilotingState.MotorModeChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupPilotingStateMotorModeChangedCallback_t) (eARCOMMANDS_POWERUP_PILOTINGSTATE_MOTORMODECHANGED_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.PilotingState.MotorModeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.PilotingState.MotorModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingStateMotorModeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupPilotingStateMotorModeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.PilotingState.MotorModeChanged
 * @param callback new callback for the command powerup.PilotingState.MotorModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingStateMotorModeChangedCallback (ARCOMMANDS_Decoder_PowerupPilotingStateMotorModeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.PilotingState.AttitudeChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupPilotingStateAttitudeChangedCallback_t) (float roll, float pitch, float yaw, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.PilotingState.AttitudeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.PilotingState.AttitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingStateAttitudeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupPilotingStateAttitudeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.PilotingState.AttitudeChanged
 * @param callback new callback for the command powerup.PilotingState.AttitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingStateAttitudeChangedCallback (ARCOMMANDS_Decoder_PowerupPilotingStateAttitudeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.PilotingState.AltitudeChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupPilotingStateAltitudeChangedCallback_t) (float altitude, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.PilotingState.AltitudeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.PilotingState.AltitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingStateAltitudeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupPilotingStateAltitudeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.PilotingState.AltitudeChanged
 * @param callback new callback for the command powerup.PilotingState.AltitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingStateAltitudeChangedCallback (ARCOMMANDS_Decoder_PowerupPilotingStateAltitudeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.PilotingSettingsState.settingChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupPilotingSettingsStateSettingChangedCallback_t) (eARCOMMANDS_POWERUP_PILOTINGSETTINGSSTATE_SETTINGCHANGED_SETTING setting, float current, float min, float max, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.PilotingSettingsState.settingChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.PilotingSettingsState.settingChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingSettingsStateSettingChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupPilotingSettingsStateSettingChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.PilotingSettingsState.settingChanged
 * @param callback new callback for the command powerup.PilotingSettingsState.settingChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupPilotingSettingsStateSettingChangedCallback (ARCOMMANDS_Decoder_PowerupPilotingSettingsStateSettingChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.MediaRecordState.PictureStateChangedV2
 */
typedef void (*ARCOMMANDS_Decoder_PowerupMediaRecordStatePictureStateChangedV2Callback_t) (eARCOMMANDS_POWERUP_MEDIARECORDSTATE_PICTURESTATECHANGEDV2_STATE state, eARCOMMANDS_POWERUP_MEDIARECORDSTATE_PICTURESTATECHANGEDV2_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.MediaRecordState.PictureStateChangedV2
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.MediaRecordState.PictureStateChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaRecordStatePictureStateChangedV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupMediaRecordStatePictureStateChangedV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.MediaRecordState.PictureStateChangedV2
 * @param callback new callback for the command powerup.MediaRecordState.PictureStateChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaRecordStatePictureStateChangedV2Callback (ARCOMMANDS_Decoder_PowerupMediaRecordStatePictureStateChangedV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.MediaRecordState.VideoStateChangedV2
 */
typedef void (*ARCOMMANDS_Decoder_PowerupMediaRecordStateVideoStateChangedV2Callback_t) (eARCOMMANDS_POWERUP_MEDIARECORDSTATE_VIDEOSTATECHANGEDV2_STATE state, eARCOMMANDS_POWERUP_MEDIARECORDSTATE_VIDEOSTATECHANGEDV2_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.MediaRecordState.VideoStateChangedV2
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.MediaRecordState.VideoStateChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaRecordStateVideoStateChangedV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupMediaRecordStateVideoStateChangedV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.MediaRecordState.VideoStateChangedV2
 * @param callback new callback for the command powerup.MediaRecordState.VideoStateChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaRecordStateVideoStateChangedV2Callback (ARCOMMANDS_Decoder_PowerupMediaRecordStateVideoStateChangedV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.MediaRecordEvent.PictureEventChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupMediaRecordEventPictureEventChangedCallback_t) (eARCOMMANDS_POWERUP_MEDIARECORDEVENT_PICTUREEVENTCHANGED_EVENT event, eARCOMMANDS_POWERUP_MEDIARECORDEVENT_PICTUREEVENTCHANGED_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.MediaRecordEvent.PictureEventChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.MediaRecordEvent.PictureEventChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaRecordEventPictureEventChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupMediaRecordEventPictureEventChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.MediaRecordEvent.PictureEventChanged
 * @param callback new callback for the command powerup.MediaRecordEvent.PictureEventChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaRecordEventPictureEventChangedCallback (ARCOMMANDS_Decoder_PowerupMediaRecordEventPictureEventChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.MediaRecordEvent.VideoEventChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupMediaRecordEventVideoEventChangedCallback_t) (eARCOMMANDS_POWERUP_MEDIARECORDEVENT_VIDEOEVENTCHANGED_EVENT event, eARCOMMANDS_POWERUP_MEDIARECORDEVENT_VIDEOEVENTCHANGED_ERROR error, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.MediaRecordEvent.VideoEventChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.MediaRecordEvent.VideoEventChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaRecordEventVideoEventChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupMediaRecordEventVideoEventChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.MediaRecordEvent.VideoEventChanged
 * @param callback new callback for the command powerup.MediaRecordEvent.VideoEventChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaRecordEventVideoEventChangedCallback (ARCOMMANDS_Decoder_PowerupMediaRecordEventVideoEventChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.NetworkSettingsState.WifiSelectionChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupNetworkSettingsStateWifiSelectionChangedCallback_t) (eARCOMMANDS_POWERUP_NETWORKSETTINGSSTATE_WIFISELECTIONCHANGED_TYPE type, eARCOMMANDS_POWERUP_NETWORKSETTINGSSTATE_WIFISELECTIONCHANGED_BAND band, uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.NetworkSettingsState.WifiSelectionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.NetworkSettingsState.WifiSelectionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkSettingsStateWifiSelectionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupNetworkSettingsStateWifiSelectionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.NetworkSettingsState.WifiSelectionChanged
 * @param callback new callback for the command powerup.NetworkSettingsState.WifiSelectionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkSettingsStateWifiSelectionChangedCallback (ARCOMMANDS_Decoder_PowerupNetworkSettingsStateWifiSelectionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.NetworkState.WifiScanListChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupNetworkStateWifiScanListChangedCallback_t) (const char * ssid, int16_t rssi, eARCOMMANDS_POWERUP_NETWORKSTATE_WIFISCANLISTCHANGED_BAND band, uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.NetworkState.WifiScanListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.NetworkState.WifiScanListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkStateWifiScanListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupNetworkStateWifiScanListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.NetworkState.WifiScanListChanged
 * @param callback new callback for the command powerup.NetworkState.WifiScanListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkStateWifiScanListChangedCallback (ARCOMMANDS_Decoder_PowerupNetworkStateWifiScanListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.NetworkState.AllWifiScanChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupNetworkStateAllWifiScanChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.NetworkState.AllWifiScanChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.NetworkState.AllWifiScanChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkStateAllWifiScanChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupNetworkStateAllWifiScanChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.NetworkState.AllWifiScanChanged
 * @param callback new callback for the command powerup.NetworkState.AllWifiScanChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkStateAllWifiScanChangedCallback (ARCOMMANDS_Decoder_PowerupNetworkStateAllWifiScanChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.NetworkState.WifiAuthChannelListChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupNetworkStateWifiAuthChannelListChangedCallback_t) (eARCOMMANDS_POWERUP_NETWORKSTATE_WIFIAUTHCHANNELLISTCHANGED_BAND band, uint8_t channel, uint8_t in_or_out, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.NetworkState.WifiAuthChannelListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.NetworkState.WifiAuthChannelListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkStateWifiAuthChannelListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupNetworkStateWifiAuthChannelListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.NetworkState.WifiAuthChannelListChanged
 * @param callback new callback for the command powerup.NetworkState.WifiAuthChannelListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkStateWifiAuthChannelListChangedCallback (ARCOMMANDS_Decoder_PowerupNetworkStateWifiAuthChannelListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.NetworkState.AllWifiAuthChannelChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupNetworkStateAllWifiAuthChannelChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.NetworkState.AllWifiAuthChannelChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.NetworkState.AllWifiAuthChannelChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkStateAllWifiAuthChannelChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupNetworkStateAllWifiAuthChannelChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.NetworkState.AllWifiAuthChannelChanged
 * @param callback new callback for the command powerup.NetworkState.AllWifiAuthChannelChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkStateAllWifiAuthChannelChangedCallback (ARCOMMANDS_Decoder_PowerupNetworkStateAllWifiAuthChannelChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.NetworkState.LinkQualityChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupNetworkStateLinkQualityChangedCallback_t) (uint8_t quality, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.NetworkState.LinkQualityChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.NetworkState.LinkQualityChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkStateLinkQualityChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupNetworkStateLinkQualityChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.NetworkState.LinkQualityChanged
 * @param callback new callback for the command powerup.NetworkState.LinkQualityChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupNetworkStateLinkQualityChangedCallback (ARCOMMANDS_Decoder_PowerupNetworkStateLinkQualityChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.MediaStreamingState.VideoEnableChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupMediaStreamingStateVideoEnableChangedCallback_t) (eARCOMMANDS_POWERUP_MEDIASTREAMINGSTATE_VIDEOENABLECHANGED_ENABLED enabled, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.MediaStreamingState.VideoEnableChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.MediaStreamingState.VideoEnableChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaStreamingStateVideoEnableChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupMediaStreamingStateVideoEnableChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.MediaStreamingState.VideoEnableChanged
 * @param callback new callback for the command powerup.MediaStreamingState.VideoEnableChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupMediaStreamingStateVideoEnableChangedCallback (ARCOMMANDS_Decoder_PowerupMediaStreamingStateVideoEnableChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.VideoSettingsState.AutorecordChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupVideoSettingsStateAutorecordChangedCallback_t) (uint8_t enabled, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.VideoSettingsState.AutorecordChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.VideoSettingsState.AutorecordChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupVideoSettingsStateAutorecordChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupVideoSettingsStateAutorecordChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.VideoSettingsState.AutorecordChanged
 * @param callback new callback for the command powerup.VideoSettingsState.AutorecordChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupVideoSettingsStateAutorecordChangedCallback (ARCOMMANDS_Decoder_PowerupVideoSettingsStateAutorecordChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.VideoSettingsState.VideoModeChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupVideoSettingsStateVideoModeChangedCallback_t) (eARCOMMANDS_POWERUP_VIDEOSETTINGSSTATE_VIDEOMODECHANGED_MODE mode, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.VideoSettingsState.VideoModeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.VideoSettingsState.VideoModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupVideoSettingsStateVideoModeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupVideoSettingsStateVideoModeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.VideoSettingsState.VideoModeChanged
 * @param callback new callback for the command powerup.VideoSettingsState.VideoModeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupVideoSettingsStateVideoModeChangedCallback (ARCOMMANDS_Decoder_PowerupVideoSettingsStateVideoModeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command powerup.SoundsState.buzzChanged
 */
typedef void (*ARCOMMANDS_Decoder_PowerupSoundsStateBuzzChangedCallback_t) (uint8_t enabled, void *custom);
/**
 * @brief Set Decoder callback setter for the command powerup.SoundsState.buzzChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command powerup.SoundsState.buzzChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupSoundsStateBuzzChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_PowerupSoundsStateBuzzChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command powerup.SoundsState.buzzChanged
 * @param callback new callback for the command powerup.SoundsState.buzzChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetPowerupSoundsStateBuzzChangedCallback (ARCOMMANDS_Decoder_PowerupSoundsStateBuzzChangedCallback_t callback, void *custom) DEPRECATED;

// Feature rc


/**
 * @brief callback type for the command rc.monitor_channels
 */
typedef void (*ARCOMMANDS_Decoder_RcMonitorChannelsCallback_t) (uint8_t enable, void *custom);
/**
 * @brief Set Decoder callback setter for the command rc.monitor_channels
 * @param decoder the decoder instance
 * @param callback new callback for the command rc.monitor_channels
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcMonitorChannelsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_RcMonitorChannelsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command rc.monitor_channels
 * @param callback new callback for the command rc.monitor_channels
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcMonitorChannelsCallback (ARCOMMANDS_Decoder_RcMonitorChannelsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command rc.start_calibration
 */
typedef void (*ARCOMMANDS_Decoder_RcStartCalibrationCallback_t) (eARCOMMANDS_RC_CALIBRATION_TYPE calibration_type, eARCOMMANDS_RC_CHANNEL_ACTION channel_action, eARCOMMANDS_RC_CHANNEL_TYPE channel_type, void *custom);
/**
 * @brief Set Decoder callback setter for the command rc.start_calibration
 * @param decoder the decoder instance
 * @param callback new callback for the command rc.start_calibration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcStartCalibrationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_RcStartCalibrationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command rc.start_calibration
 * @param callback new callback for the command rc.start_calibration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcStartCalibrationCallback (ARCOMMANDS_Decoder_RcStartCalibrationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command rc.invert_channel
 */
typedef void (*ARCOMMANDS_Decoder_RcInvertChannelCallback_t) (eARCOMMANDS_RC_CHANNEL_ACTION action, uint8_t flag, void *custom);
/**
 * @brief Set Decoder callback setter for the command rc.invert_channel
 * @param decoder the decoder instance
 * @param callback new callback for the command rc.invert_channel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcInvertChannelCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_RcInvertChannelCallback_t callback, void *custom);
/**
 * @brief callback setter for the command rc.invert_channel
 * @param callback new callback for the command rc.invert_channel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcInvertChannelCallback (ARCOMMANDS_Decoder_RcInvertChannelCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command rc.abort_calibration
 */
typedef void (*ARCOMMANDS_Decoder_RcAbortCalibrationCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command rc.abort_calibration
 * @param decoder the decoder instance
 * @param callback new callback for the command rc.abort_calibration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcAbortCalibrationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_RcAbortCalibrationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command rc.abort_calibration
 * @param callback new callback for the command rc.abort_calibration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcAbortCalibrationCallback (ARCOMMANDS_Decoder_RcAbortCalibrationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command rc.reset_calibration
 */
typedef void (*ARCOMMANDS_Decoder_RcResetCalibrationCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command rc.reset_calibration
 * @param decoder the decoder instance
 * @param callback new callback for the command rc.reset_calibration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcResetCalibrationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_RcResetCalibrationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command rc.reset_calibration
 * @param callback new callback for the command rc.reset_calibration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcResetCalibrationCallback (ARCOMMANDS_Decoder_RcResetCalibrationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command rc.enable_receiver
 */
typedef void (*ARCOMMANDS_Decoder_RcEnableReceiverCallback_t) (uint8_t enable, void *custom);
/**
 * @brief Set Decoder callback setter for the command rc.enable_receiver
 * @param decoder the decoder instance
 * @param callback new callback for the command rc.enable_receiver
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcEnableReceiverCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_RcEnableReceiverCallback_t callback, void *custom);
/**
 * @brief callback setter for the command rc.enable_receiver
 * @param callback new callback for the command rc.enable_receiver
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcEnableReceiverCallback (ARCOMMANDS_Decoder_RcEnableReceiverCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command rc.receiver_state
 */
typedef void (*ARCOMMANDS_Decoder_RcReceiverStateCallback_t) (eARCOMMANDS_RC_RECEIVER_STATE state, const char * protocol, uint8_t enabled, void *custom);
/**
 * @brief Set Decoder callback setter for the command rc.receiver_state
 * @param decoder the decoder instance
 * @param callback new callback for the command rc.receiver_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcReceiverStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_RcReceiverStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command rc.receiver_state
 * @param callback new callback for the command rc.receiver_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcReceiverStateCallback (ARCOMMANDS_Decoder_RcReceiverStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command rc.channels_monitor_state
 */
typedef void (*ARCOMMANDS_Decoder_RcChannelsMonitorStateCallback_t) (uint8_t state, void *custom);
/**
 * @brief Set Decoder callback setter for the command rc.channels_monitor_state
 * @param decoder the decoder instance
 * @param callback new callback for the command rc.channels_monitor_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcChannelsMonitorStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_RcChannelsMonitorStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command rc.channels_monitor_state
 * @param callback new callback for the command rc.channels_monitor_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcChannelsMonitorStateCallback (ARCOMMANDS_Decoder_RcChannelsMonitorStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command rc.channel_value
 */
typedef void (*ARCOMMANDS_Decoder_RcChannelValueCallback_t) (uint8_t id, eARCOMMANDS_RC_CHANNEL_ACTION action, int16_t value, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command rc.channel_value
 * @param decoder the decoder instance
 * @param callback new callback for the command rc.channel_value
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcChannelValueCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_RcChannelValueCallback_t callback, void *custom);
/**
 * @brief callback setter for the command rc.channel_value
 * @param callback new callback for the command rc.channel_value
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcChannelValueCallback (ARCOMMANDS_Decoder_RcChannelValueCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command rc.calibration_state
 */
typedef void (*ARCOMMANDS_Decoder_RcCalibrationStateCallback_t) (eARCOMMANDS_RC_CALIBRATION_TYPE calibration_type, eARCOMMANDS_RC_CHANNEL_ACTION channel_action, uint32_t required, uint32_t calibrated, uint8_t neutral_calibrated, void *custom);
/**
 * @brief Set Decoder callback setter for the command rc.calibration_state
 * @param decoder the decoder instance
 * @param callback new callback for the command rc.calibration_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcCalibrationStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_RcCalibrationStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command rc.calibration_state
 * @param callback new callback for the command rc.calibration_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcCalibrationStateCallback (ARCOMMANDS_Decoder_RcCalibrationStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command rc.channel_action_item
 */
typedef void (*ARCOMMANDS_Decoder_RcChannelActionItemCallback_t) (eARCOMMANDS_RC_CHANNEL_ACTION action, uint32_t supported_type, eARCOMMANDS_RC_CHANNEL_TYPE calibrated_type, uint8_t inverted, void *custom);
/**
 * @brief Set Decoder callback setter for the command rc.channel_action_item
 * @param decoder the decoder instance
 * @param callback new callback for the command rc.channel_action_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcChannelActionItemCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_RcChannelActionItemCallback_t callback, void *custom);
/**
 * @brief callback setter for the command rc.channel_action_item
 * @param callback new callback for the command rc.channel_action_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetRcChannelActionItemCallback (ARCOMMANDS_Decoder_RcChannelActionItemCallback_t callback, void *custom) DEPRECATED;

// Feature sequoia_cam


/**
 * @brief callback type for the command sequoia_cam.radiometric_calib_start
 */
typedef void (*ARCOMMANDS_Decoder_SequoiaCamRadiometricCalibStartCallback_t) (uint8_t id, void *custom);
/**
 * @brief Set Decoder callback setter for the command sequoia_cam.radiometric_calib_start
 * @param decoder the decoder instance
 * @param callback new callback for the command sequoia_cam.radiometric_calib_start
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSequoiaCamRadiometricCalibStartCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SequoiaCamRadiometricCalibStartCallback_t callback, void *custom);
/**
 * @brief callback setter for the command sequoia_cam.radiometric_calib_start
 * @param callback new callback for the command sequoia_cam.radiometric_calib_start
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSequoiaCamRadiometricCalibStartCallback (ARCOMMANDS_Decoder_SequoiaCamRadiometricCalibStartCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command sequoia_cam.radiometric_calib_status
 */
typedef void (*ARCOMMANDS_Decoder_SequoiaCamRadiometricCalibStatusCallback_t) (uint8_t cam_id, eARCOMMANDS_SEQUOIA_CAM_RADIOMETRIC_CALIB_STATUS_VALUE value, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command sequoia_cam.radiometric_calib_status
 * @param decoder the decoder instance
 * @param callback new callback for the command sequoia_cam.radiometric_calib_status
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSequoiaCamRadiometricCalibStatusCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SequoiaCamRadiometricCalibStatusCallback_t callback, void *custom);
/**
 * @brief callback setter for the command sequoia_cam.radiometric_calib_status
 * @param callback new callback for the command sequoia_cam.radiometric_calib_status
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSequoiaCamRadiometricCalibStatusCallback (ARCOMMANDS_Decoder_SequoiaCamRadiometricCalibStatusCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command sequoia_cam.radiometric_calib_result
 */
typedef void (*ARCOMMANDS_Decoder_SequoiaCamRadiometricCalibResultCallback_t) (uint8_t cam_id, eARCOMMANDS_SEQUOIA_CAM_RADIOMETRIC_CALIB_RESULT_VALUE value, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command sequoia_cam.radiometric_calib_result
 * @param decoder the decoder instance
 * @param callback new callback for the command sequoia_cam.radiometric_calib_result
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSequoiaCamRadiometricCalibResultCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SequoiaCamRadiometricCalibResultCallback_t callback, void *custom);
/**
 * @brief callback setter for the command sequoia_cam.radiometric_calib_result
 * @param callback new callback for the command sequoia_cam.radiometric_calib_result
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSequoiaCamRadiometricCalibResultCallback (ARCOMMANDS_Decoder_SequoiaCamRadiometricCalibResultCallback_t callback, void *custom) DEPRECATED;

// Feature SkyController


/**
 * @brief callback type for the command SkyController.Wifi.RequestWifiList
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerWifiRequestWifiListCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Wifi.RequestWifiList
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Wifi.RequestWifiList
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiRequestWifiListCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerWifiRequestWifiListCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Wifi.RequestWifiList
 * @param callback new callback for the command SkyController.Wifi.RequestWifiList
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiRequestWifiListCallback (ARCOMMANDS_Decoder_SkyControllerWifiRequestWifiListCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Wifi.RequestCurrentWifi
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerWifiRequestCurrentWifiCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Wifi.RequestCurrentWifi
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Wifi.RequestCurrentWifi
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiRequestCurrentWifiCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerWifiRequestCurrentWifiCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Wifi.RequestCurrentWifi
 * @param callback new callback for the command SkyController.Wifi.RequestCurrentWifi
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiRequestCurrentWifiCallback (ARCOMMANDS_Decoder_SkyControllerWifiRequestCurrentWifiCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Wifi.ConnectToWifi
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerWifiConnectToWifiCallback_t) (const char * bssid, const char * ssid, const char * passphrase, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Wifi.ConnectToWifi
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Wifi.ConnectToWifi
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiConnectToWifiCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerWifiConnectToWifiCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Wifi.ConnectToWifi
 * @param callback new callback for the command SkyController.Wifi.ConnectToWifi
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiConnectToWifiCallback (ARCOMMANDS_Decoder_SkyControllerWifiConnectToWifiCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Wifi.ForgetWifi
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerWifiForgetWifiCallback_t) (const char * ssid, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Wifi.ForgetWifi
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Wifi.ForgetWifi
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiForgetWifiCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerWifiForgetWifiCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Wifi.ForgetWifi
 * @param callback new callback for the command SkyController.Wifi.ForgetWifi
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiForgetWifiCallback (ARCOMMANDS_Decoder_SkyControllerWifiForgetWifiCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Wifi.WifiAuthChannel
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerWifiWifiAuthChannelCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Wifi.WifiAuthChannel
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Wifi.WifiAuthChannel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiWifiAuthChannelCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerWifiWifiAuthChannelCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Wifi.WifiAuthChannel
 * @param callback new callback for the command SkyController.Wifi.WifiAuthChannel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiWifiAuthChannelCallback (ARCOMMANDS_Decoder_SkyControllerWifiWifiAuthChannelCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Device.RequestDeviceList
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerDeviceRequestDeviceListCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Device.RequestDeviceList
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Device.RequestDeviceList
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerDeviceRequestDeviceListCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerDeviceRequestDeviceListCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Device.RequestDeviceList
 * @param callback new callback for the command SkyController.Device.RequestDeviceList
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerDeviceRequestDeviceListCallback (ARCOMMANDS_Decoder_SkyControllerDeviceRequestDeviceListCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Device.RequestCurrentDevice
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerDeviceRequestCurrentDeviceCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Device.RequestCurrentDevice
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Device.RequestCurrentDevice
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerDeviceRequestCurrentDeviceCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerDeviceRequestCurrentDeviceCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Device.RequestCurrentDevice
 * @param callback new callback for the command SkyController.Device.RequestCurrentDevice
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerDeviceRequestCurrentDeviceCallback (ARCOMMANDS_Decoder_SkyControllerDeviceRequestCurrentDeviceCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Device.ConnectToDevice
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerDeviceConnectToDeviceCallback_t) (const char * deviceName, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Device.ConnectToDevice
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Device.ConnectToDevice
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerDeviceConnectToDeviceCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerDeviceConnectToDeviceCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Device.ConnectToDevice
 * @param callback new callback for the command SkyController.Device.ConnectToDevice
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerDeviceConnectToDeviceCallback (ARCOMMANDS_Decoder_SkyControllerDeviceConnectToDeviceCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Settings.AllSettings
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerSettingsAllSettingsCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Settings.AllSettings
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Settings.AllSettings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsAllSettingsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerSettingsAllSettingsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Settings.AllSettings
 * @param callback new callback for the command SkyController.Settings.AllSettings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsAllSettingsCallback (ARCOMMANDS_Decoder_SkyControllerSettingsAllSettingsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Settings.Reset
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerSettingsResetCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Settings.Reset
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Settings.Reset
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsResetCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerSettingsResetCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Settings.Reset
 * @param callback new callback for the command SkyController.Settings.Reset
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsResetCallback (ARCOMMANDS_Decoder_SkyControllerSettingsResetCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Common.AllStates
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerCommonAllStatesCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Common.AllStates
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Common.AllStates
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCommonAllStatesCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerCommonAllStatesCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Common.AllStates
 * @param callback new callback for the command SkyController.Common.AllStates
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCommonAllStatesCallback (ARCOMMANDS_Decoder_SkyControllerCommonAllStatesCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AccessPointSettings.AccessPointSSID
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsAccessPointSSIDCallback_t) (const char * ssid, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AccessPointSettings.AccessPointSSID
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AccessPointSettings.AccessPointSSID
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsAccessPointSSIDCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsAccessPointSSIDCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AccessPointSettings.AccessPointSSID
 * @param callback new callback for the command SkyController.AccessPointSettings.AccessPointSSID
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsAccessPointSSIDCallback (ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsAccessPointSSIDCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AccessPointSettings.AccessPointChannel
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsAccessPointChannelCallback_t) (uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AccessPointSettings.AccessPointChannel
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AccessPointSettings.AccessPointChannel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsAccessPointChannelCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsAccessPointChannelCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AccessPointSettings.AccessPointChannel
 * @param callback new callback for the command SkyController.AccessPointSettings.AccessPointChannel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsAccessPointChannelCallback (ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsAccessPointChannelCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AccessPointSettings.WifiSelection
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsWifiSelectionCallback_t) (eARCOMMANDS_SKYCONTROLLER_ACCESSPOINTSETTINGS_WIFISELECTION_TYPE type, eARCOMMANDS_SKYCONTROLLER_ACCESSPOINTSETTINGS_WIFISELECTION_BAND band, uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AccessPointSettings.WifiSelection
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AccessPointSettings.WifiSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsWifiSelectionCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsWifiSelectionCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AccessPointSettings.WifiSelection
 * @param callback new callback for the command SkyController.AccessPointSettings.WifiSelection
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsWifiSelectionCallback (ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsWifiSelectionCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AccessPointSettings.WifiSecurity
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsWifiSecurityCallback_t) (eARCOMMANDS_SKYCONTROLLER_ACCESSPOINTSETTINGS_WIFISECURITY_SECURITY_TYPE security_type, const char * key, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AccessPointSettings.WifiSecurity
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AccessPointSettings.WifiSecurity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsWifiSecurityCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsWifiSecurityCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AccessPointSettings.WifiSecurity
 * @param callback new callback for the command SkyController.AccessPointSettings.WifiSecurity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsWifiSecurityCallback (ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsWifiSecurityCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Camera.ResetOrientation
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerCameraResetOrientationCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Camera.ResetOrientation
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Camera.ResetOrientation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCameraResetOrientationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerCameraResetOrientationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Camera.ResetOrientation
 * @param callback new callback for the command SkyController.Camera.ResetOrientation
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCameraResetOrientationCallback (ARCOMMANDS_Decoder_SkyControllerCameraResetOrientationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.GamepadInfos.getGamepadControls
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerGamepadInfosGetGamepadControlsCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.GamepadInfos.getGamepadControls
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.GamepadInfos.getGamepadControls
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerGamepadInfosGetGamepadControlsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerGamepadInfosGetGamepadControlsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.GamepadInfos.getGamepadControls
 * @param callback new callback for the command SkyController.GamepadInfos.getGamepadControls
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerGamepadInfosGetGamepadControlsCallback (ARCOMMANDS_Decoder_SkyControllerGamepadInfosGetGamepadControlsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.ButtonMappings.getCurrentButtonMappings
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerButtonMappingsGetCurrentButtonMappingsCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.ButtonMappings.getCurrentButtonMappings
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.ButtonMappings.getCurrentButtonMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsGetCurrentButtonMappingsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerButtonMappingsGetCurrentButtonMappingsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.ButtonMappings.getCurrentButtonMappings
 * @param callback new callback for the command SkyController.ButtonMappings.getCurrentButtonMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsGetCurrentButtonMappingsCallback (ARCOMMANDS_Decoder_SkyControllerButtonMappingsGetCurrentButtonMappingsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.ButtonMappings.getAvailableButtonMappings
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerButtonMappingsGetAvailableButtonMappingsCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.ButtonMappings.getAvailableButtonMappings
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.ButtonMappings.getAvailableButtonMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsGetAvailableButtonMappingsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerButtonMappingsGetAvailableButtonMappingsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.ButtonMappings.getAvailableButtonMappings
 * @param callback new callback for the command SkyController.ButtonMappings.getAvailableButtonMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsGetAvailableButtonMappingsCallback (ARCOMMANDS_Decoder_SkyControllerButtonMappingsGetAvailableButtonMappingsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.ButtonMappings.setButtonMapping
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerButtonMappingsSetButtonMappingCallback_t) (int32_t key_id, const char * mapping_uid, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.ButtonMappings.setButtonMapping
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.ButtonMappings.setButtonMapping
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsSetButtonMappingCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerButtonMappingsSetButtonMappingCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.ButtonMappings.setButtonMapping
 * @param callback new callback for the command SkyController.ButtonMappings.setButtonMapping
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsSetButtonMappingCallback (ARCOMMANDS_Decoder_SkyControllerButtonMappingsSetButtonMappingCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.ButtonMappings.defaultButtonMapping
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerButtonMappingsDefaultButtonMappingCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.ButtonMappings.defaultButtonMapping
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.ButtonMappings.defaultButtonMapping
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsDefaultButtonMappingCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerButtonMappingsDefaultButtonMappingCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.ButtonMappings.defaultButtonMapping
 * @param callback new callback for the command SkyController.ButtonMappings.defaultButtonMapping
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsDefaultButtonMappingCallback (ARCOMMANDS_Decoder_SkyControllerButtonMappingsDefaultButtonMappingCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisMappings.getCurrentAxisMappings
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisMappingsGetCurrentAxisMappingsCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisMappings.getCurrentAxisMappings
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisMappings.getCurrentAxisMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsGetCurrentAxisMappingsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisMappingsGetCurrentAxisMappingsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisMappings.getCurrentAxisMappings
 * @param callback new callback for the command SkyController.AxisMappings.getCurrentAxisMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsGetCurrentAxisMappingsCallback (ARCOMMANDS_Decoder_SkyControllerAxisMappingsGetCurrentAxisMappingsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisMappings.getAvailableAxisMappings
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisMappingsGetAvailableAxisMappingsCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisMappings.getAvailableAxisMappings
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisMappings.getAvailableAxisMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsGetAvailableAxisMappingsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisMappingsGetAvailableAxisMappingsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisMappings.getAvailableAxisMappings
 * @param callback new callback for the command SkyController.AxisMappings.getAvailableAxisMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsGetAvailableAxisMappingsCallback (ARCOMMANDS_Decoder_SkyControllerAxisMappingsGetAvailableAxisMappingsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisMappings.setAxisMapping
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisMappingsSetAxisMappingCallback_t) (int32_t axis_id, const char * mapping_uid, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisMappings.setAxisMapping
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisMappings.setAxisMapping
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsSetAxisMappingCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisMappingsSetAxisMappingCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisMappings.setAxisMapping
 * @param callback new callback for the command SkyController.AxisMappings.setAxisMapping
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsSetAxisMappingCallback (ARCOMMANDS_Decoder_SkyControllerAxisMappingsSetAxisMappingCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisMappings.defaultAxisMapping
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisMappingsDefaultAxisMappingCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisMappings.defaultAxisMapping
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisMappings.defaultAxisMapping
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsDefaultAxisMappingCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisMappingsDefaultAxisMappingCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisMappings.defaultAxisMapping
 * @param callback new callback for the command SkyController.AxisMappings.defaultAxisMapping
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsDefaultAxisMappingCallback (ARCOMMANDS_Decoder_SkyControllerAxisMappingsDefaultAxisMappingCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisFilters.getCurrentAxisFilters
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisFiltersGetCurrentAxisFiltersCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisFilters.getCurrentAxisFilters
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisFilters.getCurrentAxisFilters
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersGetCurrentAxisFiltersCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisFiltersGetCurrentAxisFiltersCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisFilters.getCurrentAxisFilters
 * @param callback new callback for the command SkyController.AxisFilters.getCurrentAxisFilters
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersGetCurrentAxisFiltersCallback (ARCOMMANDS_Decoder_SkyControllerAxisFiltersGetCurrentAxisFiltersCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisFilters.getPresetAxisFilters
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisFiltersGetPresetAxisFiltersCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisFilters.getPresetAxisFilters
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisFilters.getPresetAxisFilters
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersGetPresetAxisFiltersCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisFiltersGetPresetAxisFiltersCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisFilters.getPresetAxisFilters
 * @param callback new callback for the command SkyController.AxisFilters.getPresetAxisFilters
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersGetPresetAxisFiltersCallback (ARCOMMANDS_Decoder_SkyControllerAxisFiltersGetPresetAxisFiltersCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisFilters.setAxisFilter
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisFiltersSetAxisFilterCallback_t) (int32_t axis_id, const char * filter_uid_or_builder, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisFilters.setAxisFilter
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisFilters.setAxisFilter
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersSetAxisFilterCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisFiltersSetAxisFilterCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisFilters.setAxisFilter
 * @param callback new callback for the command SkyController.AxisFilters.setAxisFilter
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersSetAxisFilterCallback (ARCOMMANDS_Decoder_SkyControllerAxisFiltersSetAxisFilterCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisFilters.defaultAxisFilters
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisFiltersDefaultAxisFiltersCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisFilters.defaultAxisFilters
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisFilters.defaultAxisFilters
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersDefaultAxisFiltersCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisFiltersDefaultAxisFiltersCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisFilters.defaultAxisFilters
 * @param callback new callback for the command SkyController.AxisFilters.defaultAxisFilters
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersDefaultAxisFiltersCallback (ARCOMMANDS_Decoder_SkyControllerAxisFiltersDefaultAxisFiltersCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.CoPiloting.setPilotingSource
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerCoPilotingSetPilotingSourceCallback_t) (eARCOMMANDS_SKYCONTROLLER_COPILOTING_SETPILOTINGSOURCE_SOURCE source, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.CoPiloting.setPilotingSource
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.CoPiloting.setPilotingSource
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCoPilotingSetPilotingSourceCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerCoPilotingSetPilotingSourceCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.CoPiloting.setPilotingSource
 * @param callback new callback for the command SkyController.CoPiloting.setPilotingSource
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCoPilotingSetPilotingSourceCallback (ARCOMMANDS_Decoder_SkyControllerCoPilotingSetPilotingSourceCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Calibration.enableMagnetoCalibrationQualityUpdates
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerCalibrationEnableMagnetoCalibrationQualityUpdatesCallback_t) (uint8_t enable, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Calibration.enableMagnetoCalibrationQualityUpdates
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Calibration.enableMagnetoCalibrationQualityUpdates
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCalibrationEnableMagnetoCalibrationQualityUpdatesCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerCalibrationEnableMagnetoCalibrationQualityUpdatesCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Calibration.enableMagnetoCalibrationQualityUpdates
 * @param callback new callback for the command SkyController.Calibration.enableMagnetoCalibrationQualityUpdates
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCalibrationEnableMagnetoCalibrationQualityUpdatesCallback (ARCOMMANDS_Decoder_SkyControllerCalibrationEnableMagnetoCalibrationQualityUpdatesCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Calibration.StartCalibration
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerCalibrationStartCalibrationCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Calibration.StartCalibration
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Calibration.StartCalibration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCalibrationStartCalibrationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerCalibrationStartCalibrationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Calibration.StartCalibration
 * @param callback new callback for the command SkyController.Calibration.StartCalibration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCalibrationStartCalibrationCallback (ARCOMMANDS_Decoder_SkyControllerCalibrationStartCalibrationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Calibration.AbortCalibration
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerCalibrationAbortCalibrationCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Calibration.AbortCalibration
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Calibration.AbortCalibration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCalibrationAbortCalibrationCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerCalibrationAbortCalibrationCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Calibration.AbortCalibration
 * @param callback new callback for the command SkyController.Calibration.AbortCalibration
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCalibrationAbortCalibrationCallback (ARCOMMANDS_Decoder_SkyControllerCalibrationAbortCalibrationCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.Factory.Reset
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerFactoryResetCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.Factory.Reset
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.Factory.Reset
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerFactoryResetCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerFactoryResetCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.Factory.Reset
 * @param callback new callback for the command SkyController.Factory.Reset
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerFactoryResetCallback (ARCOMMANDS_Decoder_SkyControllerFactoryResetCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.WifiState.WifiList
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerWifiStateWifiListCallback_t) (const char * bssid, const char * ssid, uint8_t secured, uint8_t saved, int32_t rssi, int32_t frequency, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.WifiState.WifiList
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.WifiState.WifiList
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateWifiListCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerWifiStateWifiListCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.WifiState.WifiList
 * @param callback new callback for the command SkyController.WifiState.WifiList
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateWifiListCallback (ARCOMMANDS_Decoder_SkyControllerWifiStateWifiListCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.WifiState.ConnexionChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerWifiStateConnexionChangedCallback_t) (const char * ssid, eARCOMMANDS_SKYCONTROLLER_WIFISTATE_CONNEXIONCHANGED_STATUS status, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.WifiState.ConnexionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.WifiState.ConnexionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateConnexionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerWifiStateConnexionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.WifiState.ConnexionChanged
 * @param callback new callback for the command SkyController.WifiState.ConnexionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateConnexionChangedCallback (ARCOMMANDS_Decoder_SkyControllerWifiStateConnexionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.WifiState.WifiAuthChannelListChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerWifiStateWifiAuthChannelListChangedCallback_t) (eARCOMMANDS_SKYCONTROLLER_WIFISTATE_WIFIAUTHCHANNELLISTCHANGED_BAND band, uint8_t channel, uint8_t in_or_out, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.WifiState.WifiAuthChannelListChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.WifiState.WifiAuthChannelListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateWifiAuthChannelListChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerWifiStateWifiAuthChannelListChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.WifiState.WifiAuthChannelListChanged
 * @param callback new callback for the command SkyController.WifiState.WifiAuthChannelListChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateWifiAuthChannelListChangedCallback (ARCOMMANDS_Decoder_SkyControllerWifiStateWifiAuthChannelListChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.WifiState.AllWifiAuthChannelChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerWifiStateAllWifiAuthChannelChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.WifiState.AllWifiAuthChannelChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.WifiState.AllWifiAuthChannelChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateAllWifiAuthChannelChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerWifiStateAllWifiAuthChannelChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.WifiState.AllWifiAuthChannelChanged
 * @param callback new callback for the command SkyController.WifiState.AllWifiAuthChannelChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateAllWifiAuthChannelChangedCallback (ARCOMMANDS_Decoder_SkyControllerWifiStateAllWifiAuthChannelChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.WifiState.WifiSignalChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerWifiStateWifiSignalChangedCallback_t) (uint8_t level, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.WifiState.WifiSignalChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.WifiState.WifiSignalChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateWifiSignalChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerWifiStateWifiSignalChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.WifiState.WifiSignalChanged
 * @param callback new callback for the command SkyController.WifiState.WifiSignalChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateWifiSignalChangedCallback (ARCOMMANDS_Decoder_SkyControllerWifiStateWifiSignalChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.WifiState.WifiAuthChannelListChangedV2
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerWifiStateWifiAuthChannelListChangedV2Callback_t) (eARCOMMANDS_SKYCONTROLLER_WIFISTATE_WIFIAUTHCHANNELLISTCHANGEDV2_BAND band, uint8_t channel, uint8_t in_or_out, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.WifiState.WifiAuthChannelListChangedV2
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.WifiState.WifiAuthChannelListChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateWifiAuthChannelListChangedV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerWifiStateWifiAuthChannelListChangedV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.WifiState.WifiAuthChannelListChangedV2
 * @param callback new callback for the command SkyController.WifiState.WifiAuthChannelListChangedV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateWifiAuthChannelListChangedV2Callback (ARCOMMANDS_Decoder_SkyControllerWifiStateWifiAuthChannelListChangedV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.WifiState.WifiCountryChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerWifiStateWifiCountryChangedCallback_t) (const char * code, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.WifiState.WifiCountryChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.WifiState.WifiCountryChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateWifiCountryChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerWifiStateWifiCountryChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.WifiState.WifiCountryChanged
 * @param callback new callback for the command SkyController.WifiState.WifiCountryChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateWifiCountryChangedCallback (ARCOMMANDS_Decoder_SkyControllerWifiStateWifiCountryChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.WifiState.WifiEnvironmentChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerWifiStateWifiEnvironmentChangedCallback_t) (eARCOMMANDS_SKYCONTROLLER_WIFISTATE_WIFIENVIRONMENTCHANGED_ENVIRONMENT environment, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.WifiState.WifiEnvironmentChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.WifiState.WifiEnvironmentChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateWifiEnvironmentChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerWifiStateWifiEnvironmentChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.WifiState.WifiEnvironmentChanged
 * @param callback new callback for the command SkyController.WifiState.WifiEnvironmentChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerWifiStateWifiEnvironmentChangedCallback (ARCOMMANDS_Decoder_SkyControllerWifiStateWifiEnvironmentChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.DeviceState.DeviceList
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerDeviceStateDeviceListCallback_t) (const char * name, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.DeviceState.DeviceList
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.DeviceState.DeviceList
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerDeviceStateDeviceListCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerDeviceStateDeviceListCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.DeviceState.DeviceList
 * @param callback new callback for the command SkyController.DeviceState.DeviceList
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerDeviceStateDeviceListCallback (ARCOMMANDS_Decoder_SkyControllerDeviceStateDeviceListCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.DeviceState.ConnexionChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerDeviceStateConnexionChangedCallback_t) (eARCOMMANDS_SKYCONTROLLER_DEVICESTATE_CONNEXIONCHANGED_STATUS status, const char * deviceName, uint16_t deviceProductID, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.DeviceState.ConnexionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.DeviceState.ConnexionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerDeviceStateConnexionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerDeviceStateConnexionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.DeviceState.ConnexionChanged
 * @param callback new callback for the command SkyController.DeviceState.ConnexionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerDeviceStateConnexionChangedCallback (ARCOMMANDS_Decoder_SkyControllerDeviceStateConnexionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.SettingsState.AllSettingsChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerSettingsStateAllSettingsChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.SettingsState.AllSettingsChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.SettingsState.AllSettingsChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsStateAllSettingsChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerSettingsStateAllSettingsChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.SettingsState.AllSettingsChanged
 * @param callback new callback for the command SkyController.SettingsState.AllSettingsChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsStateAllSettingsChangedCallback (ARCOMMANDS_Decoder_SkyControllerSettingsStateAllSettingsChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.SettingsState.ResetChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerSettingsStateResetChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.SettingsState.ResetChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.SettingsState.ResetChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsStateResetChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerSettingsStateResetChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.SettingsState.ResetChanged
 * @param callback new callback for the command SkyController.SettingsState.ResetChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsStateResetChangedCallback (ARCOMMANDS_Decoder_SkyControllerSettingsStateResetChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.SettingsState.ProductSerialChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerSettingsStateProductSerialChangedCallback_t) (const char * serialNumber, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.SettingsState.ProductSerialChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.SettingsState.ProductSerialChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsStateProductSerialChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerSettingsStateProductSerialChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.SettingsState.ProductSerialChanged
 * @param callback new callback for the command SkyController.SettingsState.ProductSerialChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsStateProductSerialChangedCallback (ARCOMMANDS_Decoder_SkyControllerSettingsStateProductSerialChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.SettingsState.ProductVariantChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerSettingsStateProductVariantChangedCallback_t) (eARCOMMANDS_SKYCONTROLLER_SETTINGSSTATE_PRODUCTVARIANTCHANGED_VARIANT variant, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.SettingsState.ProductVariantChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.SettingsState.ProductVariantChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsStateProductVariantChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerSettingsStateProductVariantChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.SettingsState.ProductVariantChanged
 * @param callback new callback for the command SkyController.SettingsState.ProductVariantChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsStateProductVariantChangedCallback (ARCOMMANDS_Decoder_SkyControllerSettingsStateProductVariantChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.SettingsState.ProductVersionChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerSettingsStateProductVersionChangedCallback_t) (const char * software, const char * hardware, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.SettingsState.ProductVersionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.SettingsState.ProductVersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsStateProductVersionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerSettingsStateProductVersionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.SettingsState.ProductVersionChanged
 * @param callback new callback for the command SkyController.SettingsState.ProductVersionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsStateProductVersionChangedCallback (ARCOMMANDS_Decoder_SkyControllerSettingsStateProductVersionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.SettingsState.CPUID
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerSettingsStateCPUIDCallback_t) (const char * id, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.SettingsState.CPUID
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.SettingsState.CPUID
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsStateCPUIDCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerSettingsStateCPUIDCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.SettingsState.CPUID
 * @param callback new callback for the command SkyController.SettingsState.CPUID
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSettingsStateCPUIDCallback (ARCOMMANDS_Decoder_SkyControllerSettingsStateCPUIDCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.CommonState.AllStatesChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerCommonStateAllStatesChangedCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.CommonState.AllStatesChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.CommonState.AllStatesChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCommonStateAllStatesChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerCommonStateAllStatesChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.CommonState.AllStatesChanged
 * @param callback new callback for the command SkyController.CommonState.AllStatesChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCommonStateAllStatesChangedCallback (ARCOMMANDS_Decoder_SkyControllerCommonStateAllStatesChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.SkyControllerState.BatteryChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerSkyControllerStateBatteryChangedCallback_t) (uint8_t percent, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.SkyControllerState.BatteryChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.SkyControllerState.BatteryChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSkyControllerStateBatteryChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerSkyControllerStateBatteryChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.SkyControllerState.BatteryChanged
 * @param callback new callback for the command SkyController.SkyControllerState.BatteryChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSkyControllerStateBatteryChangedCallback (ARCOMMANDS_Decoder_SkyControllerSkyControllerStateBatteryChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.SkyControllerState.GpsFixChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerSkyControllerStateGpsFixChangedCallback_t) (uint8_t fixed, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.SkyControllerState.GpsFixChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.SkyControllerState.GpsFixChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSkyControllerStateGpsFixChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerSkyControllerStateGpsFixChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.SkyControllerState.GpsFixChanged
 * @param callback new callback for the command SkyController.SkyControllerState.GpsFixChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSkyControllerStateGpsFixChangedCallback (ARCOMMANDS_Decoder_SkyControllerSkyControllerStateGpsFixChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.SkyControllerState.GpsPositionChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerSkyControllerStateGpsPositionChangedCallback_t) (double latitude, double longitude, double altitude, float heading, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.SkyControllerState.GpsPositionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.SkyControllerState.GpsPositionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSkyControllerStateGpsPositionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerSkyControllerStateGpsPositionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.SkyControllerState.GpsPositionChanged
 * @param callback new callback for the command SkyController.SkyControllerState.GpsPositionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSkyControllerStateGpsPositionChangedCallback (ARCOMMANDS_Decoder_SkyControllerSkyControllerStateGpsPositionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.SkyControllerState.BatteryState
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerSkyControllerStateBatteryStateCallback_t) (eARCOMMANDS_SKYCONTROLLER_SKYCONTROLLERSTATE_BATTERYSTATE_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.SkyControllerState.BatteryState
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.SkyControllerState.BatteryState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSkyControllerStateBatteryStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerSkyControllerStateBatteryStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.SkyControllerState.BatteryState
 * @param callback new callback for the command SkyController.SkyControllerState.BatteryState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSkyControllerStateBatteryStateCallback (ARCOMMANDS_Decoder_SkyControllerSkyControllerStateBatteryStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.SkyControllerState.AttitudeChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerSkyControllerStateAttitudeChangedCallback_t) (float q0, float q1, float q2, float q3, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.SkyControllerState.AttitudeChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.SkyControllerState.AttitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSkyControllerStateAttitudeChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerSkyControllerStateAttitudeChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.SkyControllerState.AttitudeChanged
 * @param callback new callback for the command SkyController.SkyControllerState.AttitudeChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerSkyControllerStateAttitudeChangedCallback (ARCOMMANDS_Decoder_SkyControllerSkyControllerStateAttitudeChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AccessPointSettingsState.AccessPointSSIDChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsStateAccessPointSSIDChangedCallback_t) (const char * ssid, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AccessPointSettingsState.AccessPointSSIDChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AccessPointSettingsState.AccessPointSSIDChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsStateAccessPointSSIDChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsStateAccessPointSSIDChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AccessPointSettingsState.AccessPointSSIDChanged
 * @param callback new callback for the command SkyController.AccessPointSettingsState.AccessPointSSIDChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsStateAccessPointSSIDChangedCallback (ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsStateAccessPointSSIDChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AccessPointSettingsState.AccessPointChannelChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsStateAccessPointChannelChangedCallback_t) (uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AccessPointSettingsState.AccessPointChannelChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AccessPointSettingsState.AccessPointChannelChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsStateAccessPointChannelChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsStateAccessPointChannelChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AccessPointSettingsState.AccessPointChannelChanged
 * @param callback new callback for the command SkyController.AccessPointSettingsState.AccessPointChannelChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsStateAccessPointChannelChangedCallback (ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsStateAccessPointChannelChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AccessPointSettingsState.WifiSelectionChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsStateWifiSelectionChangedCallback_t) (eARCOMMANDS_SKYCONTROLLER_ACCESSPOINTSETTINGSSTATE_WIFISELECTIONCHANGED_TYPE type, eARCOMMANDS_SKYCONTROLLER_ACCESSPOINTSETTINGSSTATE_WIFISELECTIONCHANGED_BAND band, uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AccessPointSettingsState.WifiSelectionChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AccessPointSettingsState.WifiSelectionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsStateWifiSelectionChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsStateWifiSelectionChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AccessPointSettingsState.WifiSelectionChanged
 * @param callback new callback for the command SkyController.AccessPointSettingsState.WifiSelectionChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsStateWifiSelectionChangedCallback (ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsStateWifiSelectionChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AccessPointSettingsState.WifiSecurityChanged
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsStateWifiSecurityChangedCallback_t) (eARCOMMANDS_SKYCONTROLLER_ACCESSPOINTSETTINGSSTATE_WIFISECURITYCHANGED_SECURITY_TYPE security_type, const char * key, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AccessPointSettingsState.WifiSecurityChanged
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AccessPointSettingsState.WifiSecurityChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsStateWifiSecurityChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsStateWifiSecurityChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AccessPointSettingsState.WifiSecurityChanged
 * @param callback new callback for the command SkyController.AccessPointSettingsState.WifiSecurityChanged
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAccessPointSettingsStateWifiSecurityChangedCallback (ARCOMMANDS_Decoder_SkyControllerAccessPointSettingsStateWifiSecurityChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.GamepadInfosState.gamepadControl
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerGamepadInfosStateGamepadControlCallback_t) (eARCOMMANDS_SKYCONTROLLER_GAMEPADINFOSSTATE_GAMEPADCONTROL_TYPE type, int32_t id, const char * name, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.GamepadInfosState.gamepadControl
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.GamepadInfosState.gamepadControl
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerGamepadInfosStateGamepadControlCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerGamepadInfosStateGamepadControlCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.GamepadInfosState.gamepadControl
 * @param callback new callback for the command SkyController.GamepadInfosState.gamepadControl
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerGamepadInfosStateGamepadControlCallback (ARCOMMANDS_Decoder_SkyControllerGamepadInfosStateGamepadControlCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.GamepadInfosState.allGamepadControlsSent
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerGamepadInfosStateAllGamepadControlsSentCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.GamepadInfosState.allGamepadControlsSent
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.GamepadInfosState.allGamepadControlsSent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerGamepadInfosStateAllGamepadControlsSentCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerGamepadInfosStateAllGamepadControlsSentCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.GamepadInfosState.allGamepadControlsSent
 * @param callback new callback for the command SkyController.GamepadInfosState.allGamepadControlsSent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerGamepadInfosStateAllGamepadControlsSentCallback (ARCOMMANDS_Decoder_SkyControllerGamepadInfosStateAllGamepadControlsSentCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.ButtonMappingsState.currentButtonMappings
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerButtonMappingsStateCurrentButtonMappingsCallback_t) (int32_t key_id, const char * mapping_uid, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.ButtonMappingsState.currentButtonMappings
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.ButtonMappingsState.currentButtonMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsStateCurrentButtonMappingsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerButtonMappingsStateCurrentButtonMappingsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.ButtonMappingsState.currentButtonMappings
 * @param callback new callback for the command SkyController.ButtonMappingsState.currentButtonMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsStateCurrentButtonMappingsCallback (ARCOMMANDS_Decoder_SkyControllerButtonMappingsStateCurrentButtonMappingsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.ButtonMappingsState.allCurrentButtonMappingsSent
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerButtonMappingsStateAllCurrentButtonMappingsSentCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.ButtonMappingsState.allCurrentButtonMappingsSent
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.ButtonMappingsState.allCurrentButtonMappingsSent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsStateAllCurrentButtonMappingsSentCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerButtonMappingsStateAllCurrentButtonMappingsSentCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.ButtonMappingsState.allCurrentButtonMappingsSent
 * @param callback new callback for the command SkyController.ButtonMappingsState.allCurrentButtonMappingsSent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsStateAllCurrentButtonMappingsSentCallback (ARCOMMANDS_Decoder_SkyControllerButtonMappingsStateAllCurrentButtonMappingsSentCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.ButtonMappingsState.availableButtonMappings
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerButtonMappingsStateAvailableButtonMappingsCallback_t) (const char * mapping_uid, const char * name, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.ButtonMappingsState.availableButtonMappings
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.ButtonMappingsState.availableButtonMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsStateAvailableButtonMappingsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerButtonMappingsStateAvailableButtonMappingsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.ButtonMappingsState.availableButtonMappings
 * @param callback new callback for the command SkyController.ButtonMappingsState.availableButtonMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsStateAvailableButtonMappingsCallback (ARCOMMANDS_Decoder_SkyControllerButtonMappingsStateAvailableButtonMappingsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.ButtonMappingsState.allAvailableButtonsMappingsSent
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerButtonMappingsStateAllAvailableButtonsMappingsSentCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.ButtonMappingsState.allAvailableButtonsMappingsSent
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.ButtonMappingsState.allAvailableButtonsMappingsSent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsStateAllAvailableButtonsMappingsSentCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerButtonMappingsStateAllAvailableButtonsMappingsSentCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.ButtonMappingsState.allAvailableButtonsMappingsSent
 * @param callback new callback for the command SkyController.ButtonMappingsState.allAvailableButtonsMappingsSent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonMappingsStateAllAvailableButtonsMappingsSentCallback (ARCOMMANDS_Decoder_SkyControllerButtonMappingsStateAllAvailableButtonsMappingsSentCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisMappingsState.currentAxisMappings
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisMappingsStateCurrentAxisMappingsCallback_t) (int32_t axis_id, const char * mapping_uid, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisMappingsState.currentAxisMappings
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisMappingsState.currentAxisMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsStateCurrentAxisMappingsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisMappingsStateCurrentAxisMappingsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisMappingsState.currentAxisMappings
 * @param callback new callback for the command SkyController.AxisMappingsState.currentAxisMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsStateCurrentAxisMappingsCallback (ARCOMMANDS_Decoder_SkyControllerAxisMappingsStateCurrentAxisMappingsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisMappingsState.allCurrentAxisMappingsSent
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisMappingsStateAllCurrentAxisMappingsSentCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisMappingsState.allCurrentAxisMappingsSent
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisMappingsState.allCurrentAxisMappingsSent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsStateAllCurrentAxisMappingsSentCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisMappingsStateAllCurrentAxisMappingsSentCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisMappingsState.allCurrentAxisMappingsSent
 * @param callback new callback for the command SkyController.AxisMappingsState.allCurrentAxisMappingsSent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsStateAllCurrentAxisMappingsSentCallback (ARCOMMANDS_Decoder_SkyControllerAxisMappingsStateAllCurrentAxisMappingsSentCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisMappingsState.availableAxisMappings
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisMappingsStateAvailableAxisMappingsCallback_t) (const char * mapping_uid, const char * name, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisMappingsState.availableAxisMappings
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisMappingsState.availableAxisMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsStateAvailableAxisMappingsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisMappingsStateAvailableAxisMappingsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisMappingsState.availableAxisMappings
 * @param callback new callback for the command SkyController.AxisMappingsState.availableAxisMappings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsStateAvailableAxisMappingsCallback (ARCOMMANDS_Decoder_SkyControllerAxisMappingsStateAvailableAxisMappingsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisMappingsState.allAvailableAxisMappingsSent
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisMappingsStateAllAvailableAxisMappingsSentCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisMappingsState.allAvailableAxisMappingsSent
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisMappingsState.allAvailableAxisMappingsSent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsStateAllAvailableAxisMappingsSentCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisMappingsStateAllAvailableAxisMappingsSentCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisMappingsState.allAvailableAxisMappingsSent
 * @param callback new callback for the command SkyController.AxisMappingsState.allAvailableAxisMappingsSent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisMappingsStateAllAvailableAxisMappingsSentCallback (ARCOMMANDS_Decoder_SkyControllerAxisMappingsStateAllAvailableAxisMappingsSentCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisFiltersState.currentAxisFilters
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisFiltersStateCurrentAxisFiltersCallback_t) (int32_t axis_id, const char * filter_uid_or_builder, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisFiltersState.currentAxisFilters
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisFiltersState.currentAxisFilters
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersStateCurrentAxisFiltersCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisFiltersStateCurrentAxisFiltersCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisFiltersState.currentAxisFilters
 * @param callback new callback for the command SkyController.AxisFiltersState.currentAxisFilters
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersStateCurrentAxisFiltersCallback (ARCOMMANDS_Decoder_SkyControllerAxisFiltersStateCurrentAxisFiltersCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisFiltersState.allCurrentFiltersSent
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisFiltersStateAllCurrentFiltersSentCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisFiltersState.allCurrentFiltersSent
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisFiltersState.allCurrentFiltersSent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersStateAllCurrentFiltersSentCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisFiltersStateAllCurrentFiltersSentCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisFiltersState.allCurrentFiltersSent
 * @param callback new callback for the command SkyController.AxisFiltersState.allCurrentFiltersSent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersStateAllCurrentFiltersSentCallback (ARCOMMANDS_Decoder_SkyControllerAxisFiltersStateAllCurrentFiltersSentCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisFiltersState.presetAxisFilters
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisFiltersStatePresetAxisFiltersCallback_t) (const char * filter_uid, const char * name, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisFiltersState.presetAxisFilters
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisFiltersState.presetAxisFilters
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersStatePresetAxisFiltersCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisFiltersStatePresetAxisFiltersCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisFiltersState.presetAxisFilters
 * @param callback new callback for the command SkyController.AxisFiltersState.presetAxisFilters
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersStatePresetAxisFiltersCallback (ARCOMMANDS_Decoder_SkyControllerAxisFiltersStatePresetAxisFiltersCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.AxisFiltersState.allPresetFiltersSent
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerAxisFiltersStateAllPresetFiltersSentCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.AxisFiltersState.allPresetFiltersSent
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.AxisFiltersState.allPresetFiltersSent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersStateAllPresetFiltersSentCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerAxisFiltersStateAllPresetFiltersSentCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.AxisFiltersState.allPresetFiltersSent
 * @param callback new callback for the command SkyController.AxisFiltersState.allPresetFiltersSent
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerAxisFiltersStateAllPresetFiltersSentCallback (ARCOMMANDS_Decoder_SkyControllerAxisFiltersStateAllPresetFiltersSentCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.CoPilotingState.pilotingSource
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerCoPilotingStatePilotingSourceCallback_t) (eARCOMMANDS_SKYCONTROLLER_COPILOTINGSTATE_PILOTINGSOURCE_SOURCE source, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.CoPilotingState.pilotingSource
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.CoPilotingState.pilotingSource
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCoPilotingStatePilotingSourceCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerCoPilotingStatePilotingSourceCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.CoPilotingState.pilotingSource
 * @param callback new callback for the command SkyController.CoPilotingState.pilotingSource
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCoPilotingStatePilotingSourceCallback (ARCOMMANDS_Decoder_SkyControllerCoPilotingStatePilotingSourceCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.CalibrationState.MagnetoCalibrationState
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerCalibrationStateMagnetoCalibrationStateCallback_t) (eARCOMMANDS_SKYCONTROLLER_CALIBRATIONSTATE_MAGNETOCALIBRATIONSTATE_STATUS status, uint8_t X_Quality, uint8_t Y_Quality, uint8_t Z_Quality, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.CalibrationState.MagnetoCalibrationState
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.CalibrationState.MagnetoCalibrationState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCalibrationStateMagnetoCalibrationStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerCalibrationStateMagnetoCalibrationStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.CalibrationState.MagnetoCalibrationState
 * @param callback new callback for the command SkyController.CalibrationState.MagnetoCalibrationState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCalibrationStateMagnetoCalibrationStateCallback (ARCOMMANDS_Decoder_SkyControllerCalibrationStateMagnetoCalibrationStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.CalibrationState.MagnetoCalibrationQualityUpdatesState
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerCalibrationStateMagnetoCalibrationQualityUpdatesStateCallback_t) (uint8_t enabled, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.CalibrationState.MagnetoCalibrationQualityUpdatesState
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.CalibrationState.MagnetoCalibrationQualityUpdatesState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCalibrationStateMagnetoCalibrationQualityUpdatesStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerCalibrationStateMagnetoCalibrationQualityUpdatesStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.CalibrationState.MagnetoCalibrationQualityUpdatesState
 * @param callback new callback for the command SkyController.CalibrationState.MagnetoCalibrationQualityUpdatesState
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCalibrationStateMagnetoCalibrationQualityUpdatesStateCallback (ARCOMMANDS_Decoder_SkyControllerCalibrationStateMagnetoCalibrationQualityUpdatesStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.CalibrationState.MagnetoCalibrationStateV2
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerCalibrationStateMagnetoCalibrationStateV2Callback_t) (eARCOMMANDS_SKYCONTROLLER_CALIBRATIONSTATE_MAGNETOCALIBRATIONSTATEV2_STATE state, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.CalibrationState.MagnetoCalibrationStateV2
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.CalibrationState.MagnetoCalibrationStateV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCalibrationStateMagnetoCalibrationStateV2Cb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerCalibrationStateMagnetoCalibrationStateV2Callback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.CalibrationState.MagnetoCalibrationStateV2
 * @param callback new callback for the command SkyController.CalibrationState.MagnetoCalibrationStateV2
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCalibrationStateMagnetoCalibrationStateV2Callback (ARCOMMANDS_Decoder_SkyControllerCalibrationStateMagnetoCalibrationStateV2Callback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.ButtonEvents.Settings
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerButtonEventsSettingsCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.ButtonEvents.Settings
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.ButtonEvents.Settings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonEventsSettingsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerButtonEventsSettingsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.ButtonEvents.Settings
 * @param callback new callback for the command SkyController.ButtonEvents.Settings
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerButtonEventsSettingsCallback (ARCOMMANDS_Decoder_SkyControllerButtonEventsSettingsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command SkyController.CommonEventState.Shutdown
 */
typedef void (*ARCOMMANDS_Decoder_SkyControllerCommonEventStateShutdownCallback_t) (eARCOMMANDS_SKYCONTROLLER_COMMONEVENTSTATE_SHUTDOWN_REASON reason, void *custom);
/**
 * @brief Set Decoder callback setter for the command SkyController.CommonEventState.Shutdown
 * @param decoder the decoder instance
 * @param callback new callback for the command SkyController.CommonEventState.Shutdown
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCommonEventStateShutdownCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_SkyControllerCommonEventStateShutdownCallback_t callback, void *custom);
/**
 * @brief callback setter for the command SkyController.CommonEventState.Shutdown
 * @param callback new callback for the command SkyController.CommonEventState.Shutdown
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetSkyControllerCommonEventStateShutdownCallback (ARCOMMANDS_Decoder_SkyControllerCommonEventStateShutdownCallback_t callback, void *custom) DEPRECATED;

// Feature thermal_cam


/**
 * @brief callback type for the command thermal_cam.activate
 */
typedef void (*ARCOMMANDS_Decoder_ThermalCamActivateCallback_t) (uint8_t cam_id, void *custom);
/**
 * @brief Set Decoder callback setter for the command thermal_cam.activate
 * @param decoder the decoder instance
 * @param callback new callback for the command thermal_cam.activate
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetThermalCamActivateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ThermalCamActivateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command thermal_cam.activate
 * @param callback new callback for the command thermal_cam.activate
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetThermalCamActivateCallback (ARCOMMANDS_Decoder_ThermalCamActivateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command thermal_cam.deactivate
 */
typedef void (*ARCOMMANDS_Decoder_ThermalCamDeactivateCallback_t) (uint8_t cam_id, void *custom);
/**
 * @brief Set Decoder callback setter for the command thermal_cam.deactivate
 * @param decoder the decoder instance
 * @param callback new callback for the command thermal_cam.deactivate
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetThermalCamDeactivateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ThermalCamDeactivateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command thermal_cam.deactivate
 * @param callback new callback for the command thermal_cam.deactivate
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetThermalCamDeactivateCallback (ARCOMMANDS_Decoder_ThermalCamDeactivateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command thermal_cam.set_sensitivity
 */
typedef void (*ARCOMMANDS_Decoder_ThermalCamSetSensitivityCallback_t) (uint8_t cam_id, eARCOMMANDS_THERMAL_CAM_RANGE range, void *custom);
/**
 * @brief Set Decoder callback setter for the command thermal_cam.set_sensitivity
 * @param decoder the decoder instance
 * @param callback new callback for the command thermal_cam.set_sensitivity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetThermalCamSetSensitivityCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ThermalCamSetSensitivityCallback_t callback, void *custom);
/**
 * @brief callback setter for the command thermal_cam.set_sensitivity
 * @param callback new callback for the command thermal_cam.set_sensitivity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetThermalCamSetSensitivityCallback (ARCOMMANDS_Decoder_ThermalCamSetSensitivityCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command thermal_cam.camera_state
 */
typedef void (*ARCOMMANDS_Decoder_ThermalCamCameraStateCallback_t) (uint8_t cam_id, eARCOMMANDS_THERMAL_CAM_STATE state, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command thermal_cam.camera_state
 * @param decoder the decoder instance
 * @param callback new callback for the command thermal_cam.camera_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetThermalCamCameraStateCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ThermalCamCameraStateCallback_t callback, void *custom);
/**
 * @brief callback setter for the command thermal_cam.camera_state
 * @param callback new callback for the command thermal_cam.camera_state
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetThermalCamCameraStateCallback (ARCOMMANDS_Decoder_ThermalCamCameraStateCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command thermal_cam.sensitivity
 */
typedef void (*ARCOMMANDS_Decoder_ThermalCamSensitivityCallback_t) (uint8_t cam_id, eARCOMMANDS_THERMAL_CAM_RANGE current_range, uint8_t available_ranges, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command thermal_cam.sensitivity
 * @param decoder the decoder instance
 * @param callback new callback for the command thermal_cam.sensitivity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetThermalCamSensitivityCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ThermalCamSensitivityCallback_t callback, void *custom);
/**
 * @brief callback setter for the command thermal_cam.sensitivity
 * @param callback new callback for the command thermal_cam.sensitivity
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetThermalCamSensitivityCallback (ARCOMMANDS_Decoder_ThermalCamSensitivityCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command thermal_cam.calibration_infos
 */
typedef void (*ARCOMMANDS_Decoder_ThermalCamCalibrationInfosCallback_t) (uint8_t cam_id, float roll, float pitch, float yaw, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command thermal_cam.calibration_infos
 * @param decoder the decoder instance
 * @param callback new callback for the command thermal_cam.calibration_infos
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetThermalCamCalibrationInfosCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_ThermalCamCalibrationInfosCallback_t callback, void *custom);
/**
 * @brief callback setter for the command thermal_cam.calibration_infos
 * @param callback new callback for the command thermal_cam.calibration_infos
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetThermalCamCalibrationInfosCallback (ARCOMMANDS_Decoder_ThermalCamCalibrationInfosCallback_t callback, void *custom) DEPRECATED;

// Feature wifi


/**
 * @brief callback type for the command wifi.scan
 */
typedef void (*ARCOMMANDS_Decoder_WifiScanCallback_t) (uint8_t band, void *custom);
/**
 * @brief Set Decoder callback setter for the command wifi.scan
 * @param decoder the decoder instance
 * @param callback new callback for the command wifi.scan
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiScanCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_WifiScanCallback_t callback, void *custom);
/**
 * @brief callback setter for the command wifi.scan
 * @param callback new callback for the command wifi.scan
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiScanCallback (ARCOMMANDS_Decoder_WifiScanCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command wifi.update_authorized_channels
 */
typedef void (*ARCOMMANDS_Decoder_WifiUpdateAuthorizedChannelsCallback_t) (void *custom);
/**
 * @brief Set Decoder callback setter for the command wifi.update_authorized_channels
 * @param decoder the decoder instance
 * @param callback new callback for the command wifi.update_authorized_channels
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiUpdateAuthorizedChannelsCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_WifiUpdateAuthorizedChannelsCallback_t callback, void *custom);
/**
 * @brief callback setter for the command wifi.update_authorized_channels
 * @param callback new callback for the command wifi.update_authorized_channels
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiUpdateAuthorizedChannelsCallback (ARCOMMANDS_Decoder_WifiUpdateAuthorizedChannelsCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command wifi.set_ap_channel
 */
typedef void (*ARCOMMANDS_Decoder_WifiSetApChannelCallback_t) (eARCOMMANDS_WIFI_SELECTION_TYPE type, eARCOMMANDS_WIFI_BAND band, uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command wifi.set_ap_channel
 * @param decoder the decoder instance
 * @param callback new callback for the command wifi.set_ap_channel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiSetApChannelCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_WifiSetApChannelCallback_t callback, void *custom);
/**
 * @brief callback setter for the command wifi.set_ap_channel
 * @param callback new callback for the command wifi.set_ap_channel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiSetApChannelCallback (ARCOMMANDS_Decoder_WifiSetApChannelCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command wifi.set_security
 */
typedef void (*ARCOMMANDS_Decoder_WifiSetSecurityCallback_t) (eARCOMMANDS_WIFI_SECURITY_TYPE type, const char * key, eARCOMMANDS_WIFI_SECURITY_KEY_TYPE key_type, void *custom);
/**
 * @brief Set Decoder callback setter for the command wifi.set_security
 * @param decoder the decoder instance
 * @param callback new callback for the command wifi.set_security
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiSetSecurityCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_WifiSetSecurityCallback_t callback, void *custom);
/**
 * @brief callback setter for the command wifi.set_security
 * @param callback new callback for the command wifi.set_security
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiSetSecurityCallback (ARCOMMANDS_Decoder_WifiSetSecurityCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command wifi.set_country
 */
typedef void (*ARCOMMANDS_Decoder_WifiSetCountryCallback_t) (eARCOMMANDS_WIFI_COUNTRY_SELECTION selection_mode, const char * code, void *custom);
/**
 * @brief Set Decoder callback setter for the command wifi.set_country
 * @param decoder the decoder instance
 * @param callback new callback for the command wifi.set_country
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiSetCountryCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_WifiSetCountryCallback_t callback, void *custom);
/**
 * @brief callback setter for the command wifi.set_country
 * @param callback new callback for the command wifi.set_country
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiSetCountryCallback (ARCOMMANDS_Decoder_WifiSetCountryCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command wifi.set_environment
 */
typedef void (*ARCOMMANDS_Decoder_WifiSetEnvironmentCallback_t) (eARCOMMANDS_WIFI_ENVIRONMENT environment, void *custom);
/**
 * @brief Set Decoder callback setter for the command wifi.set_environment
 * @param decoder the decoder instance
 * @param callback new callback for the command wifi.set_environment
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiSetEnvironmentCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_WifiSetEnvironmentCallback_t callback, void *custom);
/**
 * @brief callback setter for the command wifi.set_environment
 * @param callback new callback for the command wifi.set_environment
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiSetEnvironmentCallback (ARCOMMANDS_Decoder_WifiSetEnvironmentCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command wifi.scanned_item
 */
typedef void (*ARCOMMANDS_Decoder_WifiScannedItemCallback_t) (const char * ssid, int16_t rssi, eARCOMMANDS_WIFI_BAND band, uint8_t channel, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command wifi.scanned_item
 * @param decoder the decoder instance
 * @param callback new callback for the command wifi.scanned_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiScannedItemCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_WifiScannedItemCallback_t callback, void *custom);
/**
 * @brief callback setter for the command wifi.scanned_item
 * @param callback new callback for the command wifi.scanned_item
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiScannedItemCallback (ARCOMMANDS_Decoder_WifiScannedItemCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command wifi.authorized_channel
 */
typedef void (*ARCOMMANDS_Decoder_WifiAuthorizedChannelCallback_t) (eARCOMMANDS_WIFI_BAND band, uint8_t channel, uint8_t environment, uint8_t list_flags, void *custom);
/**
 * @brief Set Decoder callback setter for the command wifi.authorized_channel
 * @param decoder the decoder instance
 * @param callback new callback for the command wifi.authorized_channel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiAuthorizedChannelCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_WifiAuthorizedChannelCallback_t callback, void *custom);
/**
 * @brief callback setter for the command wifi.authorized_channel
 * @param callback new callback for the command wifi.authorized_channel
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiAuthorizedChannelCallback (ARCOMMANDS_Decoder_WifiAuthorizedChannelCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command wifi.ap_channel_changed
 */
typedef void (*ARCOMMANDS_Decoder_WifiApChannelChangedCallback_t) (eARCOMMANDS_WIFI_SELECTION_TYPE type, eARCOMMANDS_WIFI_BAND band, uint8_t channel, void *custom);
/**
 * @brief Set Decoder callback setter for the command wifi.ap_channel_changed
 * @param decoder the decoder instance
 * @param callback new callback for the command wifi.ap_channel_changed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiApChannelChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_WifiApChannelChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command wifi.ap_channel_changed
 * @param callback new callback for the command wifi.ap_channel_changed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiApChannelChangedCallback (ARCOMMANDS_Decoder_WifiApChannelChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command wifi.security_changed
 */
typedef void (*ARCOMMANDS_Decoder_WifiSecurityChangedCallback_t) (const char * key, eARCOMMANDS_WIFI_SECURITY_TYPE key_type, void *custom);
/**
 * @brief Set Decoder callback setter for the command wifi.security_changed
 * @param decoder the decoder instance
 * @param callback new callback for the command wifi.security_changed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiSecurityChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_WifiSecurityChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command wifi.security_changed
 * @param callback new callback for the command wifi.security_changed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiSecurityChangedCallback (ARCOMMANDS_Decoder_WifiSecurityChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command wifi.country_changed
 */
typedef void (*ARCOMMANDS_Decoder_WifiCountryChangedCallback_t) (eARCOMMANDS_WIFI_COUNTRY_SELECTION selection_mode, const char * code, void *custom);
/**
 * @brief Set Decoder callback setter for the command wifi.country_changed
 * @param decoder the decoder instance
 * @param callback new callback for the command wifi.country_changed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiCountryChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_WifiCountryChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command wifi.country_changed
 * @param callback new callback for the command wifi.country_changed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiCountryChangedCallback (ARCOMMANDS_Decoder_WifiCountryChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command wifi.environment_changed
 */
typedef void (*ARCOMMANDS_Decoder_WifiEnvironmentChangedCallback_t) (eARCOMMANDS_WIFI_ENVIRONMENT environment, void *custom);
/**
 * @brief Set Decoder callback setter for the command wifi.environment_changed
 * @param decoder the decoder instance
 * @param callback new callback for the command wifi.environment_changed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiEnvironmentChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_WifiEnvironmentChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command wifi.environment_changed
 * @param callback new callback for the command wifi.environment_changed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiEnvironmentChangedCallback (ARCOMMANDS_Decoder_WifiEnvironmentChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command wifi.rssi_changed
 */
typedef void (*ARCOMMANDS_Decoder_WifiRssiChangedCallback_t) (int16_t rssi, void *custom);
/**
 * @brief Set Decoder callback setter for the command wifi.rssi_changed
 * @param decoder the decoder instance
 * @param callback new callback for the command wifi.rssi_changed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiRssiChangedCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_WifiRssiChangedCallback_t callback, void *custom);
/**
 * @brief callback setter for the command wifi.rssi_changed
 * @param callback new callback for the command wifi.rssi_changed
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiRssiChangedCallback (ARCOMMANDS_Decoder_WifiRssiChangedCallback_t callback, void *custom) DEPRECATED;

/**
 * @brief callback type for the command wifi.supported_countries
 */
typedef void (*ARCOMMANDS_Decoder_WifiSupportedCountriesCallback_t) (const char * countries, void *custom);
/**
 * @brief Set Decoder callback setter for the command wifi.supported_countries
 * @param decoder the decoder instance
 * @param callback new callback for the command wifi.supported_countries
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiSupportedCountriesCb (ARCOMMANDS_Decoder_t *decoder, ARCOMMANDS_Decoder_WifiSupportedCountriesCallback_t callback, void *custom);
/**
 * @brief callback setter for the command wifi.supported_countries
 * @param callback new callback for the command wifi.supported_countries
 * @param custom pointer that will be passed to all calls to the callback
 */
void ARCOMMANDS_Decoder_SetWifiSupportedCountriesCallback (ARCOMMANDS_Decoder_WifiSupportedCountriesCallback_t callback, void *custom) DEPRECATED;

#endif /* _LIBARCOMMANDS_ARCOMMANDS_DECODER_H_ */
